/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __IVUG_DB_H__
#define __IVUG_DB_H__

#include <stdbool.h>

#include "ivug-datatypes.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef void *filter_handle;
typedef void *media_handle;
typedef void *tag_handle;

typedef bool (*ivug_db_cb)(media_handle media, void *user_data);
typedef void (*ivug_db_path_cb)(media_handle media, const char *path, void *user_data);

#define IVUG_MAX_CONDITION_LEN (1024)

typedef struct _Ivug_DB_h
{
	ivug_db_path_cb callback;
	media_handle *m_handle;
	void *data;
	int key;
}Ivug_DB_h;

bool ivug_db_create(void);
bool ivug_db_destroy(void);

bool ivug_db_update(media_handle media);
bool ivug_db_refresh(const char *media_id);
bool ivug_db_rename(media_handle m_handle, const char *dest);

bool ivug_db_create_filter(filter_handle *filter);
bool ivug_db_destroy_filter(filter_handle filter);
bool ivug_db_set_filter(filter_handle filter, ivug_view_by view_by, ivug_media_type media_type, char *condition);
bool ivug_db_set_filter_offset(filter_handle filter, int stp, int endp);

media_handle ivug_db_insert_file_to_DB(const char* filepath);

media_handle ivug_db_get_file_handle(const char* filepath);
media_handle ivug_db_get_file_handle_from_media_id(const char* media_id);
media_handle ivug_db_get_folder_handle(const char* folderpath);
tag_handle ivug_db_get_tag_handle(const char* tagname);

bool ivug_db_destroy_file_handle(media_handle m_handle);
bool ivug_db_destroy_folder_handle(media_handle m_handle);

bool ivug_db_get_all_folder_list(ivug_db_cb callback, void *data);
bool ivug_db_get_all_tag_list(ivug_db_cb callback, void *data);

bool ivug_db_create_tag(tag_handle *tag_h, const char *tag_name);
bool ivug_db_destroy_tag(tag_handle tag_h);
bool ivug_db_set_tag(media_handle m_handle, tag_handle t_handle);

bool ivug_db_set_favorite(media_handle media, bool set);

int ivug_db_get_tag_id(tag_handle media);

/*
	below functions returned values should be freed by user
*/
char *ivug_db_get_file_path(media_handle media);
char *ivug_db_get_file_id(media_handle media);
char *ivug_db_get_thumbnail_path(media_handle media);
char *ivug_db_get_folder_name(media_handle media);
char *ivug_db_get_folder_id(media_handle media);
char *ivug_db_get_folder_path(media_handle media);
char *ivug_db_get_tag_name(tag_handle media);

bool ivug_db_get_file_size(media_handle media, unsigned long long *size);


Ivug_DB_h * ivug_db_create_thumbnail(media_handle media, ivug_db_path_cb callback, void *data);
bool ivug_db_cancel_thumbnail(Ivug_DB_h *db_h);

#ifdef __cplusplus
}
#endif


#endif // __IVUG_DB_H__

