/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __IVUG_FILE_INFO_H__
#define __IVUG_FILE_INFO_H__

#include <stdbool.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
	ivug_fileinfo_*() returns false when error. otherwise return true and fill proper value in [out] param
*/
bool ivug_fileinfo_get_image_gps_info(const char* filepath, double * /* OUT */ latitude, double * /* OUT */ longitude);
bool ivug_fileinfo_get_video_gps_info(const char *filepath, double * /* OUT */ latitude, double * /* OUT */ longitude);

bool ivug_fileinfo_get_video_resolution(const char *filepath, int * /* OUT */ pWidth, int * /* OUT */pHeight);
bool ivug_fileinfo_get_image_resolution(const char *filepath, int * /* OUT */ pWidth, int * /* OUT */pHeight);


/*
	return file extension string. if file is DRM file, decrypted file extension is returned.

	CAUTION : user should free returned string.
*/
char *ivug_fileinfo_get_file_extension(const char *filepath);


/*
	return mine type from drm or non-drm file.

	CAUTION : user should free returned string.
*/
char *ivug_fileinfo_get_mime_type(const char *filepath);
char *ivug_fileinfo_get_focal_length(const char *path);
char *ivug_fileinfo_get_model_name(const char *path);
char *ivug_fileinfo_get_maker_name(const char *path);
char *ivug_fileinfo_get_aperture(const char *path);
char *ivug_fileinfo_get_exposure_time(const char *path);
char *ivug_fileinfo_get_iso(const char *path);

int ivug_fileinfo_get_flash_status(const char *path);
int ivug_fileinfo_get_orientation(const char *path);
int ivug_fileinfo_get_white_balance(const char *path);

#ifdef __cplusplus
}
#endif


#endif 		// __IVUG_FILE_INFO_H__

