/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "ivug-dir.h"

#include <stdio.h>
#include <dirent.h>
#include <stdlib.h>

#include <linux/limits.h>

bool EnumFilesInDir(const char *dir, EnumFilesCB pFunc, void *data)
{
	struct dirent *dptr;

	DIR *dirp;

	if((dirp=opendir(dir))==NULL)
	{
		printf("Cannot open dir : %s", dir);
		return false;
	}

	static char fullpath[PATH_MAX];

	while( (dptr = readdir(dirp)) != NULL )
	{
		if ( dptr->d_type == DT_REG )		// Only for gegular file
		{
			if ( pFunc )
			{
				sprintf(fullpath, "%s/%s", dir, dptr->d_name);

				pFunc(fullpath, data);
			}
		}
	}

	closedir(dirp);

	return true;
}


int GetFilesCountInDir(const char *dir)
{

	struct dirent *dptr;
	DIR *dirp;

	if( (dirp=opendir(dir)) == NULL)
	{
		printf("Cannot open dir : %s", dir);
		return -1;
	}

	int nCount = 0;

	while( (dptr = readdir(dirp)) != NULL )
	{
		if ( dptr->d_type == DT_REG )		// Only for gegular file
		{
			nCount++;
		}
	}

	closedir(dirp);

	return nCount;
}


#if 0
void _MyCB(const char *fname, void *data)
{
	printf("%s\n", fname);
}


int main(int argc, char *argv[])
{
	EnumFilesInDir("/home/rookie/slp2.0", _MyCB, NULL);
}
#endif



