/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __IVUG_DETAIL_COMMON_H__
#define __IVUG_DETAIL_COMMON_H__

#include <unicode/udat.h>

#include "ivug-common.h"
#include "ivug-drm-common.h"
#include "ivug-media.h"

typedef enum {
	IVUG_DETAIL_ERROR_TYPE = -1,
	IVUG_DETAIL_IMAGE_TYPE,
	IVUG_DETAIL_VIDEO_TYPE
} ivug_detail_file_t;

typedef enum
{
	FILE_INFO_NAME,
	FILE_INFO_FORMAT,
	FILE_INFO_DATE_CREATED,
	FILE_INFO_SIZE,
	FILE_INFO_RESOLUTION,
	FILE_INFO_FILE_LOCATION,
	FILE_INFO_MAX
}file_info_type_e;

typedef enum
{
	EXIF_INFO_ORIENTATION,
	EXIF_INFO_LATITUDE,
	EXIF_INFO_LONGITTUDE,
	EXIF_INFO_MANUFACTURE,
	EXIF_INFO_MODEL,
	EXIF_INFO_FLASH,
	EXIF_INFO_FOCAL_LENGTH,
	EXIF_INFO_WHITE_BALANCE,
	EXIF_INFO_APERTURE,
	EXIF_INFO_EXPOSURE_TIME,
	EXIF_INFO_ISO,
	EXIF_INFO_MAX
}exif_info_type_e;

typedef enum
{
	DRM_INFO_VENDOR,
	DRM_INFO_FORWARDING,
	DRM_INFO_AVAILABLE_USES,
	DRM_INFO_TYPE,
	DRM_INFO_VALIDITY,
	DRM_INFO_MAX
}drm_info_type_e;

#ifdef __cplusplus
extern "C" {
#endif

bool
ivug_datail_get_exif_info(/*out*/char** name_, /*out*/char **contents_,
	const char *path, Media_Type stype, exif_info_type_e type);

bool
ivug_datail_get_file_info(/*out*/char** name, /*out*/char **contents,
	const char *path, Media_Type stype, file_info_type_e type);

bool
ivug_datail_get_drm_info(/*out*/char** name, /*out*/char **contents,
	const char *path, Media_Type stype, drm_info_type_e type);



#ifdef __cplusplus
}
#endif


#endif	// __IVUG_DETAIL_COMMON_H__

