/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __IVUG_CROP_VIEW_H__
#define __IVUG_CROP_VIEW_H__

#include "ivug-define.h"


typedef enum {
	CROP_ERROR_TYPE_NONE,
	CROP_ERROR_TYPE_UNKNOWN_FORMAT,
	CROP_ERROR_TYPE_PERMISSION_DENIED,
	CROP_ERROR_TYPE_INVALID_FILE,
	CROP_ERROR_TYPE_GENERAL,
} Crop_Error;

typedef struct {
	Evas_Object *layout;
	Evas_Object *notify;

	Evas_Object *photocam;
	Evas_Object *cropbox;

	Evas_Object *gesture;

	Evas_Object *btn_back;

	Evas_Object *contents_area;
	Evas_Object *notify_area;
	Evas_Object *btn_ok;

	bool bShowMenu;
	char *file_path;
	char *result_path;

	char *dest_dir;
	char *dest_name;

	int w;
	int h;

	int prev_x;
	int prev_y;
	int prev_w;
	int prev_h;

	Ecore_Timer *box_timer;

	int PrevRotate;
} IvugCropView;

/*
	signals
		"loaded" with error code
		"ok,clicked"		- Button "OK" clicked with filepath
		"cancel,clicked"	- Button "Cancel" clicked
		"destroyed"
*/

#ifdef __cplusplus
extern "C" {
#endif

/* this api is for trasition */
void _ivug_crop_view_box_delete(IvugCropView *pCropView);

IvugCropView *ivug_crop_view_create(Evas_Object *parent);

bool ivug_crop_view_box_size_set(IvugCropView *pCropView, int w, int h);

bool ivug_crop_view_box_ratio_fix(IvugCropView *pCropView, bool bFix);

bool ivug_crop_view_file_set(IvugCropView *pCropView, const char *file);

bool ivug_crop_view_destination_set(IvugCropView *pCropView, const char *dir, const char *name);

void ivug_crop_view_destroy(IvugCropView *pCropView);

Evas_Object *ivug_crop_view_get_object(IvugCropView *pCropView);

void ivug_crop_view_create_menu(IvugCropView *pCropView, Evas_Object *navi_bar);

#ifdef __cplusplus
}
#endif

#endif 		// __IVUG_CROP_VIEW_H__

