/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __IVUG_MAIN_VIEWER_H__
#define __IVUG_MAIN_VIEWER_H__

#include "ivug-common.h"
#include "ivug-parameter.h"
#include "ivug-medialist.h"

#include "ivug-name-view.h"
#include "ivug-details-view.h"
#include "ivug-setas-view.h"
#include "ivug-crop-view.h"


#define MAX_CTRL_ITEM_CNT 5
#define MAX_NAVI_ITEM_CNT 3

//navigation bar type
typedef enum {
	NAVI_BAR_TYPE_NONE = 0x00,
	NAVI_BAR_TYPE_EMPTY,			// no control bar
	NAVI_BAR_TYPE_IMAGE,			// [Edit, Set, Info] default
	NAVI_BAR_TYPE_FILE,				// [Set, Info] default
	NAVI_BAR_TYPE_VIDEO,			// [Trim, Set, Info] default
	NAVI_BAR_TYPE_SELECT,			// [Ok, Cancel] from take photo
} ivug_toolbar;

//control bar type
typedef enum {
	CTRL_BAR_TYPE_NONE = 0x00,
	CTRL_BAR_TYPE_EMPTY,				// not exist ctrl bar
	CTRL_BAR_TYPE_FILE,					// [Delete,Share,Set,Info]  file manager, video at gallery
	CTRL_BAR_TYPE_READ_ONLY,			// [Share, Set, Save, Info]
	CTRL_BAR_TYPE_IMAGE,				// [Delete,Share, Add tag, Slide show, Editor] from gallery
	CTRL_BAR_TYPE_VIDEO,				// [Delete,Share, Add tag, Slide show, Trim] from gallery
} ivug_ctrlbar;

typedef enum {
// only shows in ctrlbar
	TOOLBUTTON_DELETE,
	TOOLBUTTON_SHARE,
	TOOLBUTTON_ADDTAG,
	TOOLBUTTON_SLIDESHOW,
	TOOLBUTTON_SAVE,

// shows in either toolbar or ctrlbar
	TOOLBUTTON_EDIT,
	TOOLBUTTON_SETAS,
	TOOLBUTTON_DETAILS,
	TOOLBUTTON_TRIM,

// only shows in toolbar
	TOOLBUTTON_OK,
	TOOLBUTTON_CANCEL,

// Navigation hearder
	TOOLBUTTON_TOOL,

// NULL button for spacing
	TOOLBUTTON_NULL,
	TOOLBUTTON_MAX,
} ToolButtonType;

typedef enum {
	TOOLBUTTON_IN_NONE,
	TOOLBUTTON_IN_TOOLBAR,
	TOOLBUTTON_IN_CTRLBAR,
} TooButtonPos;

typedef struct {
	Elm_Object_Item *item;		// Control bar item in navigation header. ex) SetAs, Favorite, Info
	TooButtonPos pos;
} ControlBar_Item;


typedef struct _SlideShow SlideShow;

typedef struct _Ivug_MainView Ivug_MainView;

#undef FIX_LAST_BLINK

#ifdef __cplusplus
extern "C" {
#endif

/*
	Create MainView layout
*/
Ivug_MainView *
ivug_main_view_create(Evas_Object* parent, ivug_mode mode, ivug_view_by view_by);

Evas_Object *
ivug_main_view_object_get(Ivug_MainView *pMainView);

void ivug_main_view_start(Ivug_MainView *pMainView);

/*
	Load media list from parameter.
*/
bool ivug_main_view_set_list(Ivug_MainView *pMainView, ivug_parameter *ivug_param);

/*
	Start slide show.
*/
void ivug_main_view_start_slideshow(Ivug_MainView *pMainView, Eina_Bool bSlideFirst);

void ivug_main_view_resume(Ivug_MainView *pMainView);
void ivug_main_view_pause(Ivug_MainView *pMainView);


void ivug_main_view_destroy(Ivug_MainView *pMainView);


/*
	Control GUI
*/
void ivug_main_view_show_menu_bar(Ivug_MainView *pMainView);
void ivug_main_view_hide_menu_bar(Ivug_MainView *pMainView);

void ivug_main_view_set_hide_timer(Ivug_MainView *pMainView);
void ivug_main_view_del_hide_timer(Ivug_MainView *pMainView);

void _ivug_main_on_mmc_state_changed(keynode_t* node, void *data);

#ifdef __cplusplus
}
#endif

#endif //__IVUG_MAIN_VIEWER_H__

