/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __IVUG_NAME_VIEW_H__
#define __IVUG_NAME_VIEW_H__

#include "ivug-common.h"
#include "ivug-media.h"

typedef enum {
	NAME_VIEW_RESPONSE_OK,
	NAME_VIEW_RESPONSE_CANCEL,
} ivug_name_response;

typedef void (*FNResponse)(ivug_name_response resp, const char *string, void *pClientData);

typedef struct _Ivug_NameView
{
	Evas_Object *bg;
	Evas_Object *layout;
	Evas_Object *content;

	Evas_Object *navibar;

	Evas_Object *editfield;
	Evas_Object *entry;
	Evas_Object *btn_done;

	bool sip_show;

	FNResponse fnresponse;
	void *clientdata;

} Ivug_NameView;

#ifdef __cplusplus
extern "C" {
#endif

Ivug_NameView *
ivug_name_view_create(Evas_Object *parent, const char *title);

void
ivug_name_view_set_title(Ivug_NameView *pNameView, const char *title);

void
ivug_name_view_set_entry(Ivug_NameView *pNameView, const char *str);

void
ivug_name_view_set_response_callback(Ivug_NameView *pNameView, FNResponse resp, void *data);

void
ivug_name_view_destroy(Ivug_NameView *pNameView);

Evas_Object *
ivug_name_view_object_get(Ivug_NameView *pNameView);

void
ivug_name_view_set_focus(Ivug_NameView *pNameView);

void
ivug_name_view_set_max_length(Ivug_NameView *pNameView, int max_len);

#ifdef __cplusplus
}
#endif

#endif	// __IVUG_NAME_VIEW_H__


