/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __IVUG_SETAS_VIEW_H__
#define __IVUG_SETAS_VIEW_H__

#include "ivug-common.h"

typedef enum {
	NOTI_SUCCESS,
	NOTI_FAIL,
} TEXT_POPUP_TYPE;

typedef enum {
	IVUG_CTRLBAR_SET_SCREEN_HOME,
	IVUG_CTRLBAR_SET_SCREEN_LOCK,
	IVUG_CTRLBAR_SET_SCREEN_BOTH,
	IVUG_CTRLBAR_SET_SCREEN_UNDEFINED,		// Show select popup when click ok button.
	IVUG_CTRLBAR_SET_SCREEN_CALLERID,
	IVUG_CTRLBAR_SET_SCREEN_VIDEO_CALL_ID,
	IVUG_CTRLBAR_SET_SCREEN_MAX
} ivug_set_screen_type;

typedef enum {
	IVUG_SETAS_NORMAL,	// launched from mainview
	IVUG_SETAS_UG,		// ug mode
	IVUG_SETAS_APPSVC,	// appsvc mode
	IVUG_SETAS_CROP		// crop mode
} ivug_setas_mode;

typedef enum {
	SETAS_ERROR_TYPE_NONE,
	SETAS_ERROR_TYPE_UNKNOWN_FORMAT,
	SETAS_ERROR_TYPE_PERMISSION_DENIED,
	SETAS_ERROR_TYPE_INVALID_FILE,
	SETAS_ERROR_TYPE_GENERAL,
} SetAs_Error;

typedef struct {
	Evas_Object *ly_effect;		// Layout for transition effect

	Evas_Object *content;		// content
	Evas_Object *genlist_popup;

	char *filename;

	ivug_setas_mode Mode;
	ivug_set_screen_type Type;

	bool bInitFace;
} Ivug_SetAsView;

#ifdef __cplusplus
extern "C" {
#endif

Ivug_SetAsView *
ivug_setas_view_callerid_ug_create(Evas_Object *parent, const char* filepath, ivug_setas_mode mode, int output_width, int output_height);

Ivug_SetAsView *
ivug_setas_view_video_call_id_ug_create(Evas_Object *parent, const char* filepath, ivug_setas_mode mode, int output_width, int output_height);

void
ivug_setas_view_destroy(Ivug_SetAsView *pSetAsView);

Evas_Object *
ivug_setas_view_object_get(Ivug_SetAsView *pSetAsView);

// Called by main_view
Ivug_SetAsView *
ivug_setas_view_screen_create(Evas_Object *parent, ivug_setas_mode mode, ivug_set_screen_type screen_type);

void ivug_setas_view_create_menu(Ivug_SetAsView *pSetAsView, Evas_Object *navi_bar, ivug_set_screen_type type);

void ivug_setas_view_load_file(Ivug_SetAsView *pSetAsView, const char* filepath, ivug_set_screen_type type);

#ifdef __cplusplus
}
#endif


#endif //__IVUG_SETAS_VIEW_H__
