/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

#include "ivug-common.h"
#include "ivug-medialist.h"
#include "ivug-slider.h"


typedef struct _Slide_Item Slide_Item;

typedef void (*OnSliderStateChanged)(slide_state_t state, Slide_Item *si, void *client_data);

typedef enum
{
	SITEM_TYPE_UNKNOWN,
	SITEM_TYPE_IMAGE,
	SITEM_TYPE_VIDEO,
} sitem_type_t;

struct _Slide_Item {
	int x, y, w, h;			// Object geometry
//Data
	slide_state_t state;
	sitem_type_t type;

	Media_Item *mitem;

//flag
	Eina_Bool thumbnail_show;

// UI
	Evas_Object* layout;
	Evas_Object* thumbnail;	// thumbnail

	Evas_Object* vIcon;		//video play icon

	Evas_Object* photocam;	// photocam

	Evas_Object* progress;	//progress bar for download.

// Callback
	bool bMenuVisible;
	OnSliderStateChanged cb;
	void *client_data;

	drm_handle_t drm_handle;
};

#ifdef __cplusplus
extern "C" {
#endif

// GUI related
Slide_Item*
ivug_slider_item_add(Evas_Object *slider);

void
ivug_slider_item_del(Slide_Item *si);

// Geometry
void
ivug_slider_item_hide(Slide_Item *si);

void
ivug_slider_item_show(Slide_Item *si);

void
ivug_slider_item_move(Slide_Item *si, int x, int y);

void
ivug_slider_item_resize(Slide_Item *si, int w, int h);

void
ivug_slider_item_clip_set(Slide_Item *si, Evas_Object *clipper);

void
ivug_slider_item_clip_unset(Slide_Item *si);


// Callback
bool
ivug_slider_item_set_callback(Slide_Item* si, OnSliderStateChanged cb, void *data);


// Data
const Media_Item *
ivug_slider_item_data_get(Slide_Item *si);

/*
	When item is NULL, Data is cleared
*/
bool
ivug_slider_item_data_set(Slide_Item *si, Media_Item *item);

bool
ivug_slider_item_icon_click_check(Slide_Item *si, int x, int y);

drm_data_t *
ivug_slider_item_check_drm_constraint(Slide_Item *si);


// Item state
bool
ivug_slider_item_load(Slide_Item *si);

void
ivug_slider_item_play(Slide_Item* si);

void
ivug_slider_item_pause(Slide_Item* si);		// --> Hide

void
ivug_slider_item_resume(Slide_Item* si);	// --> Show

void
ivug_slider_item_stop(Slide_Item* si);

void
ivug_slider_item_video_play(Slide_Item* si);

void
ivug_slider_item_video_pause(Slide_Item* si);


// Control
bool
ivug_slider_item_region_size_get(Slide_Item *si, int *x, int *y, int *w, int *h);

void
ivug_slider_item_hold_set(Slide_Item *si, Eina_Bool hold);		// Ignore mouse event

void
ivug_slider_item_zoom_reset(Slide_Item *si, Eina_Bool bAnim, int center_x, int center_y);

void
ivug_slider_item_zoom_set(Slide_Item *si, double zoom, int center_x, int center_y);

double
ivug_slider_item_zoom_get(Slide_Item* si);


bool
ivug_slider_item_image_rotate_set(Slide_Item *si, int angle);


void
ivug_slider_item_hide_menu(Slide_Item* si);

void
ivug_slider_item_show_menu(Slide_Item* si);

Evas_Object *
ivug_slider_item_image_get(Slide_Item* si);

void
ivug_slider_item_image_size_get(Slide_Item* si, int *w, int *h);

void ivug_slider_item_reload(Slide_Item* si);


#ifdef __cplusplus
}
#endif



