/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __IVUG_SLIDER_H__
#define __IVUG_SLIDER_H__

#include "ivug-define.h"

#include <Elementary.h>
#include "ivug-medialist.h"
#include "ivug-drm-common.h"

typedef enum {
	SLIDE_STATE_NONE = 0x00,
	SLIDE_STATE_SET,	// Thumbnail is displayed.

	SLIDE_STATE_DOWNLOADING,			// Web downloading.
	SLIDE_STATE_DRM_CHECKING,			// wait for drm popup

	SLIDE_STATE_READY,		// Got original media

	SLIDE_STATE_LOADING,			// Load main image or drm decryption.
	SLIDE_STATE_LOADED,				// Image is shown

// Error state
	SLIDE_STATE_DOWNLOAD_FAILED,	// web download failed
	SLIDE_STATE_NO_RIGHT,
	SLIDE_STATE_ERROR,
} slide_state_t;


typedef enum {
	SLIDE_TO_RIGHT = 0x00,
	SLIDE_TO_LEFT,
	SLIDE_INSTANTLY,
} slide_dir_t;

typedef enum {
	LONGTAP_ENDED,
	LONGTAP_CANCELED,
} longtap_state;

typedef enum {
	CENTER_SLIDE,
	NEXT_SLIDE,
	PREV_SLIDE,
	MAX_SLIDE,
} slide_index_t;

/*
	Slider signal list.

	"slider,clicked" : when click once
	"slider,longpress,start" : when long press start on slider
	"slider,longpress,end" : when long press end on slider
	"slider,clicked,icon" : video icon clicked.
	"slider,item,changed" :

	"slider,item,state,changed"
	"slider,item,deleted"

	Object Configuration

	"conf-use-inline" : 1 for enable inline-play otherwise disabled


*/

typedef struct _Slide_Item Slide_Item;

#ifdef __cplusplus
extern "C" {
#endif

Evas_Object*
ivug_slider_add(Evas_Object* parent);

void
ivug_slider_start(Evas_Object *obj);

Eina_Bool
ivug_slider_set_medialist(Evas_Object* obj, Media_List *mList, Media_Item *current);

void
ivug_slider_set_item(Evas_Object* obj, Media_Item *item, slide_dir_t dir);

void
ivug_slider_set_item_to_slide(Evas_Object* obj, slide_index_t where, Media_Item *newItem);

/*
	Delete current media item
*/
void
ivug_slider_delete_item(Evas_Object* obj);

Slide_Item *
ivug_slider_get_sitem(Evas_Object* obj);

Media_Item *
ivug_slider_get_item(Evas_Object* obj);

slide_state_t
ivug_slider_get_state(Evas_Object* obj);

/*
	Go background - stop animation
*/
void
ivug_slider_pause(Evas_Object* obj);

/*
	Go foreground - start animation
*/
void
ivug_slider_resume(Evas_Object* obj);

void
ivug_slider_skip(Evas_Object* obj);

bool
ivug_slider_load(Evas_Object* obj);

void
ivug_slider_dump_photocam(Evas_Object* obj);

bool
ivug_slider_image_rotate(Evas_Object* obj, int angle);

void
ivug_slider_set_menu_visibility(Evas_Object* obj, Eina_Bool bVisible);

unsigned int
ivug_slider_zoom_level_get(Evas_Object *obj);

void
ivug_slider_zoom_in(Evas_Object *obj);

void
ivug_slider_zoom_out(Evas_Object *obj);

void
ivug_slider_zoom_reset(Evas_Object *obj);

bool
ivug_slider_reload(Evas_Object* obj);

#ifdef __cplusplus
}
#endif


#endif	// __IVUG_SLIDER_H__

