/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "ivug-common.h"
#include "ivug-slider-item.h"
#include "ivug-slider.h"

#include "ivug-slider-priv.h"
#include "ECoreAnimator.h"

#define LOGNTAP_THRESHOLD	(100)		// allow moving distance(pixel)

#define LOG_LVL DBG_MSG_LVL_HIGH
#define LOG_CAT "IV-SMOUSE"


void
_ivug_slider_pass_event_to_item(struct Smart_Data *sd, Eina_Bool bPass)
{
	sd->bPassEvent = bPass;

	if ( bPass == EINA_TRUE)
	{
		MSG_SLIDER_MED("Photocam can get Event");
	}
	else
	{
		MSG_SLIDER_MED("Photocam can not get Event");
	}

	for (int i = 0; i < MAX_SLIDE; i++)
	{
		ivug_slider_item_hold_set(sd->slide[i], !bPass);
	}
}


bool _ivug_slider_is_visible(struct Smart_Data *sd, Slide_Item* si)
{
	if ( si->x + si->w < 0 )
	{
		return false;
	}

	if ( si->x > sd->x + sd->w )
	{
		return false;
	}

	return true;
}

#define _ivug_slider_edge_state_get(si) __ivug_slider_edge_state_get(si, __func__, __LINE__)

slide_edge_t __ivug_slider_edge_state_get(Slide_Item *si, const char *func, int line)
{
	int nx, ny, nw, nh;

	slide_edge_t result = EDGE::SLIDE_NO_EDGE;

	if ( ivug_slider_item_region_size_get(si, &nx, &ny, &nw, &nh) == false)
	{
		// false means video file item. in this case, no photocam exists.
		MSG_MED("[%s:%4d] Slide fit. Thumbnail only", func, line);
		result = EDGE::SLIDE_FIT;

		return result;
	}
	else
	{
		if ( nw <= (si->w + IMAGE_SCROLL_MARGIN) && (nh <= si->h + IMAGE_SCROLL_MARGIN))
		{
			MSG_MED("[%20s:%4d] Slide fit", func, line);
			result = EDGE::SLIDE_FIT;
		}
		else
		{
			if ( nx >= si->x - IMAGE_MOVE_MARGIN)
			{
				MSG_MED("[%20s:%4d] image edge left. N(%d,%d) Si(%d,%d)", func, line, nx, nw, si->x, si->w);

				result = (result | EDGE::SLIDE_LEFT_EDGE);
			}

			if ( nx + nw <= si->x + si->w + IMAGE_MOVE_MARGIN)
			{	//edge right
				MSG_MED("[%20s:%4d] image edge right. N(%d,%d) Si(%d,%d)", func, line, nx, nw, si->x, si->w);
				result = (result | EDGE::SLIDE_RIGHT_EDGE);
			}
#ifdef ENABLE_Y_SCR
			if ( ny >= si->y )
			{
				MSG_MED("[%20s:%4d] image edge top", func, line);
				result = (result | EDGE::SLIDE_TOP_EDGE);
			}

			if ( ny + nh <= si->y + si->h)
			{	//edge right
				MSG_MED("[%20s:%4d] image edge bottom", func, line);
				result = (result | EDGE::SLIDE_BOTTOM_EDGE);
			}
#endif
		}
	}

	if ( result == EDGE::SLIDE_NO_EDGE )
	{
		MSG_MED("[%20s:%4d] image over screen. Edge=%d N(%d,%d,%d,%d) Si(%d,%d,%d,%d)", func, line, result, nx, ny, nw, nh, si->x, si->y, si->w, si->h);
	}

	return result;
}

void _put_originate(struct Smart_Data *sd, shift_direction_t dir, int momentum)
{
	if ( dir == SLIDE_SHIFT_TO_LEFT )
	{
		if ( sd->slide[PREV_SLIDE]->state == SLIDE_STATE_NONE )
		{
			dir = SLIDE_SHIFT_NONE;
		}

		MSG_MED("Prev slide state : %d", sd->slide[NEXT_SLIDE]->state);
	}

	if ( dir == SLIDE_SHIFT_TO_RIGHT )
	{
		if ( sd->slide[NEXT_SLIDE]->state == SLIDE_STATE_NONE )
		{
			dir = SLIDE_SHIFT_NONE;
		}

		MSG_MED("Next slide state : %d", sd->slide[NEXT_SLIDE]->state);
	}

	int src_x, dst_x;

	switch(dir)
	{
	case SLIDE_SHIFT_TO_LEFT:
		src_x = sd->slide[CENTER_SLIDE]->x;
		dst_x = sd->x + sd->slide[CENTER_SLIDE]->w + IMAGE_BETWEEN_MARGIN;

		MSG_HIGH("Scroll --->. Move from %d, %d", src_x, dst_x);

		_ivug_slider_start_slide(sd, src_x, dst_x, momentum);
		break;

	case SLIDE_SHIFT_TO_RIGHT:
		src_x = sd->slide[CENTER_SLIDE]->x;
		dst_x = sd->x -sd->slide[CENTER_SLIDE]->w - IMAGE_BETWEEN_MARGIN;

		MSG_HIGH("Scroll <---. Move from %d, %d", src_x, dst_x);

		_ivug_slider_start_slide(sd, src_x, dst_x, momentum);
		break;

	case SLIDE_SHIFT_NONE:
		src_x = sd->slide[CENTER_SLIDE]->x;
		dst_x = sd->x;

		MSG_HIGH("Scroll to center. Move from %d, %d", src_x , dst_x);

		if (src_x == dst_x )
		{
			MSG_WARN("No need to animation");
			return ;
		}
		// Defaut velocity
		_ivug_slider_start_slide(sd, src_x, dst_x, ANIN_VELOCITY);

		break;
	}

}

shift_direction_t _get_shift_direction(struct Smart_Data *sd)
{
#define SHIFT_THRESHOLD	(0.0f)

	Slide_Item* si = sd->slide[CENTER_SLIDE];

	if ( abs( si->x - sd->x ) < 10 )
	{
		MSG_HIGH("Shift None. si->x=%d sd->x=%d", si->x, sd->x);
		return SLIDE_SHIFT_NONE;
	}

	if ( (si->x - sd->x) > sd->w * SHIFT_THRESHOLD )
	{
		// Scroll to --->
		MSG_HIGH("Shift ---->. si->x=%d sd->x=%d", si->x, sd->x);
		return SLIDE_SHIFT_TO_LEFT;
	}
	else if ((si->x - sd->x) + si->w < sd->w * (1.0f - SHIFT_THRESHOLD) )
	{
		// Scroll to <---
		MSG_HIGH("Shift <----. si->x=%d sd->x=%d", si->x, sd->x);
		return SLIDE_SHIFT_TO_RIGHT;
	}

	return SLIDE_SHIFT_NONE;

}


Evas_Event_Flags _zoom_start(void *data, void *event_info)
{
	Elm_Gesture_Zoom_Info *p = (Elm_Gesture_Zoom_Info *) event_info;
	MSG_LOW("zoom start <%d,%d> <%f>", p->x, p->y, p->zoom);

	struct Smart_Data *sd = (struct Smart_Data *) evas_object_smart_data_get((Evas_Object *)data);
	MSG_ASSERT(sd != NULL);

	Slide_Item* si = sd->slide[CENTER_SLIDE];

	MSG_MED("%s MoveX=%d", __func__, si->x);

	sd->bZooming = false;

	if ( abs(si->x - sd->x) < IMAGE_MOVE_MARGIN)
	{
		_ivug_slider_slide_move(sd, 0, si->y);	// Move to Center
		_ivug_slider_pass_event_to_item(sd, EINA_TRUE);
		sd->bZooming = true;
	}

	return EVAS_EVENT_FLAG_NONE;
}

Evas_Event_Flags _zoom_move(void *data, void *event_info)
{
	Elm_Gesture_Zoom_Info *p = (Elm_Gesture_Zoom_Info *) event_info;
	MSG_LOW("zoom move <%d,%d> <%f>", p->x, p->y, p->zoom);

	struct Smart_Data *sd = (struct Smart_Data *) evas_object_smart_data_get((Evas_Object *)data);
	MSG_ASSERT(sd != NULL);

	Slide_Item* si = sd->slide[CENTER_SLIDE];

	if ( sd->bZooming == false ) return EVAS_EVENT_FLAG_NONE;

	MSG_MED("%s MoveX=%d", __func__, si->x);

	return EVAS_EVENT_FLAG_NONE;
}

Evas_Event_Flags _zoom_end(void *data, void *event_info)
{
	Elm_Gesture_Zoom_Info *p = (Elm_Gesture_Zoom_Info *) event_info;
	MSG_LOW("zoom end <%d,%d> <%f>", p->x, p->y, p->zoom);

	struct Smart_Data *sd = (struct Smart_Data *) evas_object_smart_data_get((Evas_Object *)data);
	MSG_ASSERT(sd != NULL);

	Slide_Item* si = sd->slide[CENTER_SLIDE];

	if ( sd->bZooming == false )
	{
// Flick image
		shift_direction_t dir = _get_shift_direction(sd);
		_put_originate(sd, dir, ANIN_VELOCITY);

		return EVAS_EVENT_FLAG_NONE;
	}

	MSG_MED("%s. CenteSize WH(%d,%d)", __func__, si->w, si->h);	// Si->w,h didnot change. after zoom

	sd->edge = _ivug_slider_edge_state_get(si);	// Update Edge

	return EVAS_EVENT_FLAG_NONE;
}

Evas_Event_Flags _zoom_abort(void *data, void *event_info)
{
	Elm_Gesture_Zoom_Info *p = (Elm_Gesture_Zoom_Info *) event_info;
	MSG_LOW("zoom abort <%d,%d> <%f>", p->x, p->y, p->zoom);

	struct Smart_Data *sd = (struct Smart_Data *) evas_object_smart_data_get((Evas_Object *)data);
	MSG_ASSERT(sd != NULL);

	Slide_Item* si = sd->slide[CENTER_SLIDE];

	if ( sd->bZooming == false )
	{
// Flick image
		shift_direction_t dir = _get_shift_direction(sd);
		_put_originate(sd, dir, ANIN_VELOCITY);

		return EVAS_EVENT_FLAG_NONE;
	}

	MSG_MED("%s. CenteSize WH(%d,%d)", __func__, si->w, si->h);	// Si->w,h didnot change. after zoom

	sd->edge = _ivug_slider_edge_state_get(si);	// Update Edge

	return EVAS_EVENT_FLAG_NONE;
}

Evas_Event_Flags _momentum_start(void *data, void *event_info)
{
	Elm_Gesture_Momentum_Info *p = (Elm_Gesture_Momentum_Info *) event_info;
	MSG_MED("momentum_start <%d,%d>", p->x2, p->y2);

	if ( p->n != 1 ) return EVAS_EVENT_FLAG_NONE;

	struct Smart_Data *sd = (struct Smart_Data *) evas_object_smart_data_get((Evas_Object *)data);
	MSG_ASSERT(sd != NULL);

	Slide_Item* si = sd->slide[CENTER_SLIDE];

//save down position.
	sd->prev_x = p->x2;
	sd->prev_y = p->y2;

	sd->anim->Stop();

// Update Center!.

	sd->edge = _ivug_slider_edge_state_get(si);

	sd->bMomentumStarted = true;

	return EVAS_EVENT_FLAG_NONE;
}

Evas_Event_Flags _momentum_move(void *data, void *event_info)
{
	Elm_Gesture_Momentum_Info *p = (Elm_Gesture_Momentum_Info *) event_info;
	// _momentum_move is comming when pinch zoom
	if ( p->n != 1 ) return EVAS_EVENT_FLAG_NONE;

	MSG_LOW("momentum move <%d,%d>, <%d,%d>, p->n=%d", p->x1, p->y1, p->x2, p->y2, p->n);

	struct Smart_Data *sd = (struct Smart_Data *) evas_object_smart_data_get((Evas_Object *)data);
	MSG_ASSERT(sd != NULL);

	Slide_Item* si = sd->slide[CENTER_SLIDE];

	int dist_down_x = p->x2 - p->x1;
	int dist_down_y = p->y2 - p->y1;

	if(sqrt(dist_down_x*dist_down_x + dist_down_y*dist_down_y) > LOGNTAP_THRESHOLD)
	{
		MSG_LOW("Long press cancel");
		sd->bLongtapEnable = false;
	}

	if(sd->bLongtapEnable == true)
	{
		int longtap_dx = p->x2 - sd->longtap_prev_x;
		int longtap_dy = p->y2 - sd->longtap_prev_y;

		if(sqrt(longtap_dx*longtap_dx + longtap_dy*longtap_dy) > LOGNTAP_THRESHOLD)
		{
			MSG_LOW("Long press cancel");
			evas_object_smart_callback_call(sd->obj, "slider,longpress,end", (void*)LONGTAP_ENDED);
		}
	}

//	MSG_MED("%s. XY(%d,%d)", __func__, cur->x, cur->y);
	if ( sd->edge == EDGE::SLIDE_NO_EDGE )
	{
		MSG_HIGH("Photocam handle this event");
		return EVAS_EVENT_FLAG_NONE;
	}

	int dx, dy;

	dx = p->x2 - sd->prev_x;
	dy = p->y2 - sd->prev_y;

	if ( dx == 0 ) return EVAS_EVENT_FLAG_NONE;

	sd->prev_x = p->x2;
	sd->prev_y = p->y2;

	int dst_x, dst_y;

	dst_x = si->x + (dx);
	dst_y = si->y + (dy);

	if ( sd->edge == EDGE::SLIDE_FIT)
	{
		MSG_MED("Case 1. edge=%d, X=%d dx=%d si->x=%d", sd->edge, dst_x, dx, si->x);
		_ivug_slider_pass_event_to_item(sd, EINA_FALSE);

		if ( dst_x >= 0 && sd->slide[PREV_SLIDE]->state == SLIDE_STATE_NONE)
		{
			MSG_HIGH("first file");
			dst_x = 0;

			// Update left shadow.
		}

		if ( dst_x < 0 && sd->slide[NEXT_SLIDE]->state == SLIDE_STATE_NONE)
		{
			MSG_HIGH("last file");
			dst_x = 0;
		}

		_ivug_slider_slide_move(sd, dst_x , si->y);

		return EVAS_EVENT_FLAG_NONE;
	}

	if ( (si->x - sd->x) * (dst_x - sd->x) < 0 )
	{
		MSG_MED("Case a. edge=%d, X=%d dx=%d si->x=%d", sd->edge, dst_x, dx, si->x);

		dst_x = 0;
		_ivug_slider_pass_event_to_item(sd, EINA_TRUE);
		_ivug_slider_slide_move(sd, dst_x , si->y);

		return EVAS_EVENT_FLAG_NONE;
	}

	if ( si->x != sd->x )
	{
		MSG_MED("Case 2. edge=%d, X=%d dx=%d si->x=%d", sd->edge, dst_x, dx, si->x);

		_ivug_slider_pass_event_to_item(sd, EINA_FALSE);

		if ( dst_x >= 0 && sd->slide[PREV_SLIDE]->state == SLIDE_STATE_NONE)
		{
			MSG_HIGH("first file");
			dst_x = 0;
		}

		if ( dst_x < 0 && sd->slide[NEXT_SLIDE]->state == SLIDE_STATE_NONE)
		{
			MSG_HIGH("last file");
			dst_x = 0;
		}

		_ivug_slider_slide_move(sd, dst_x , si->y);

		return EVAS_EVENT_FLAG_NONE;
	}

	if ( sd->edge & EDGE::SLIDE_LEFT_EDGE)
	{
		if ( dx > 0 ) // Mouse -->
		{
			MSG_MED("Case 3. edge=%d, X=%d dx=%d si->x=%d", sd->edge, dst_x, dx, si->x);

			_ivug_slider_pass_event_to_item(sd, EINA_FALSE);

			if ( dst_x >= 0 && sd->slide[PREV_SLIDE]->state == SLIDE_STATE_NONE)
			{
				MSG_HIGH("first file");
				dst_x = 0;
			}

			_ivug_slider_slide_move(sd, dst_x , si->y);

			return EVAS_EVENT_FLAG_NONE;
		}
		else
		{
			MSG_MED("Case 4. edge=%d, X=%d dx=%d si->x=%d", sd->edge, dst_x, dx, si->x);

			_ivug_slider_pass_event_to_item(sd, EINA_TRUE);
		}
	}

	if ( sd->edge & EDGE::SLIDE_RIGHT_EDGE)
	{
		if ( dx < 0 ) // Mouse <--
		{
			MSG_MED("Case 5. edge=%d, X=%d dx=%d si->x=%d", sd->edge, dst_x, dx, si->x);

			_ivug_slider_pass_event_to_item(sd, EINA_FALSE);

			if ( dst_x < 0 && sd->slide[NEXT_SLIDE]->state == SLIDE_STATE_NONE)
			{
				MSG_HIGH("last file");
				dst_x = 0;
			}

			_ivug_slider_slide_move(sd, dst_x , si->y);

			return EVAS_EVENT_FLAG_NONE;
		}
		else
		{

			MSG_MED("Case 6. edge=%d, X=%d dx=%d si->x=%d", sd->edge, dst_x, dx, si->x);

			_ivug_slider_pass_event_to_item(sd, EINA_TRUE);

		}
	}

#ifdef ENABLE_Y_SCR

	if ( sd->edge & EDGE::SLIDE_TOP_EDGE)
	{
		if ( dy > 0 ) // Mouse downward
		{
			MSG_MED("Case 7. edge=%d, Y=%d dy=%d si->y=%d", sd->edge, dst_y, dy, si->y);

			_ivug_slider_pass_event_to_item(sd, EINA_FALSE);

			return EVAS_EVENT_FLAG_NONE;
		}
		else
		{

			MSG_MED("Case 8. edge=%d, Y=%d dy=%d si->y=%d", sd->edge, dst_y, dy, si->y);

			_ivug_slider_pass_event_to_item(sd, EINA_TRUE);

		}
	}

	if ( sd->edge & EDGE::SLIDE_BOTTOM_EDGE)
	{
		if ( dy < 0 ) // Mouse downward. ^
		{
			MSG_MED("Case 9. edge=%d, Y=%d dy=%d si->y=%d", sd->edge, dst_y, dy, si->y);

			_ivug_slider_pass_event_to_item(sd, EINA_FALSE);

			return EVAS_EVENT_FLAG_NONE;
		}
		else
		{

			MSG_MED("Case 10. edge=%d, Y=%d dy=%d si->y=%d", sd->edge, dst_y, dy, si->y);

			_ivug_slider_pass_event_to_item(sd, EINA_TRUE);

		}
	}
#endif

	sd->edge = _ivug_slider_edge_state_get(si);
	return EVAS_EVENT_FLAG_NONE;

}

Evas_Event_Flags _momentum_end(void *data, void *event_info)
{
	Elm_Gesture_Momentum_Info *p = (Elm_Gesture_Momentum_Info *) event_info;

	//if ( p->n != 1 ) return EVAS_EVENT_FLAG_NONE;

	MSG_MED("momentum end Cur=<%d,%d> M=<%d,%d>", p->x2, p->y2, p->mx, p->my);

	struct Smart_Data *sd = (struct Smart_Data *) evas_object_smart_data_get((Evas_Object *)data);
	MSG_ASSERT(sd != NULL);

	Slide_Item *si = sd->slide[CENTER_SLIDE];

	MSG_MED("Mouse Up : si->XYWH(%d,%d,%d,%d)", si->x, si->y, si->w, si->h);

// Find Scroll Direction
	sd->edge = _ivug_slider_edge_state_get(si);

	shift_direction_t dir = SLIDE_SHIFT_NONE;

	bool isFlick = abs(p->mx) < 1500 ? false : true;

	if ( isFlick == true )
	{
// momentum 0 ƴ (flick animation ߰ ٽ click) center Ʈ ؾߵǳ? ׷ ϸ ε巯 .. ѵ.. ϴ pass
		MSG_HIGH("Velocity = %d Momentum=%d", sd->momentum, p->mx);

		sd->momentum += p->mx;

		if ( (sd->edge & EDGE::SLIDE_LEFT_EDGE) && p->mx > 0)
		{
			// Flick
			if (  p->mx + (si->x - sd->x) > sd->w )
			{
				// Flick right
				dir = SLIDE_SHIFT_TO_LEFT;

				MSG_HIGH("Flick Right ----------->");

				if ( sd->slide[PREV_SLIDE]->state == SLIDE_STATE_NONE )
				{
					dir = SLIDE_SHIFT_NONE;
					sd->momentum = 0;
				}

				MSG_MED("Prev slide state : %d", sd->slide[NEXT_SLIDE]->state);
			}
		}

		if ( (sd->edge & EDGE::SLIDE_RIGHT_EDGE) && p->mx < 0)
		{
			// Flick
			if (  p->mx + ((si->x - sd->x) + si->w ) < 0 )
			{
				// Flick left
				dir = SLIDE_SHIFT_TO_RIGHT;
				MSG_HIGH("Flick Left <-----------");

				if ( sd->slide[NEXT_SLIDE]->state == SLIDE_STATE_NONE )
				{
					dir = SLIDE_SHIFT_NONE;
					sd->momentum = 0;
				}

				MSG_MED("Next slide state : %d", sd->slide[NEXT_SLIDE]->state);
			}
		}

		MSG_MED("Velocity2 = %d", sd->momentum);
		_put_originate(sd, dir, sd->momentum);

	}
	else
	{
		MSG_MED("Si->x=%d", si->x);

		dir = _get_shift_direction(sd);
		_put_originate(sd, dir, ANIN_VELOCITY);

	}

	sd->bLongtapEnable = true;

	sd->bMomentumStarted = false;

	return EVAS_EVENT_FLAG_NONE;
}

Evas_Event_Flags _momentum_abort(void *data, void *event_info)
{
	Elm_Gesture_Momentum_Info *p = (Elm_Gesture_Momentum_Info *) event_info;

	MSG_MED("momentum abort Cur=<%d,%d> M=<%d,%d> N=%d", p->x2, p->y2, p->mx, p->my, p->n);

	//if ( p->n != 1 ) return EVAS_EVENT_FLAG_NONE;

	struct Smart_Data *sd = (struct Smart_Data *) evas_object_smart_data_get((Evas_Object *)data);
	MSG_ASSERT(sd != NULL);

	Slide_Item *si = sd->slide[CENTER_SLIDE];

	MSG_MED("Mouse Up : si->XYWH(%d,%d,%d,%d)", si->x, si->y, si->w, si->h);

// Find Scroll Direction
	sd->edge = _ivug_slider_edge_state_get(si);

	shift_direction_t dir = SLIDE_SHIFT_NONE;

	bool isFlick = abs(p->mx) < 1500 ? false : true;

	if ( isFlick == true )
	{
		if ( (sd->edge & EDGE::SLIDE_LEFT_EDGE) && p->mx > 0)
		{
			// Flick
			if (  p->mx + ((si->x - sd->x) ) > sd->w )
			{
				// Flick right
				MSG_MED("Flick Right");
				dir = SLIDE_SHIFT_TO_LEFT;
			}
		}

		if ( (sd->edge & EDGE::SLIDE_RIGHT_EDGE) && p->mx < 0)
		{
			// Flick
			if (  p->mx + ((si->x - sd->x) + si->w ) < 0 )
			{
				// Flick left
				MSG_MED("Flick Left");
				dir = SLIDE_SHIFT_TO_RIGHT;
			}
		}
	}
	else
	{
		MSG_MED("Si->x=%d", si->x);
		dir = _get_shift_direction(sd);
	}

	_put_originate(sd, dir, ANIN_VELOCITY);

	sd->bLongtapEnable = true;

	sd->bMomentumStarted = false;

	return EVAS_EVENT_FLAG_NONE;
}



Evas_Event_Flags _flick_start(void *data, void *event_info)
{
	Elm_Gesture_Line_Info *p = (Elm_Gesture_Line_Info *) event_info;

	MSG_LOW("flick started x1,x2=(%d,%d) tx=%u mx=%d n=%u",
		  p->momentum.x1, p->momentum.x2, p->momentum.tx,
		  p->momentum.mx, p->momentum.n);

	return EVAS_EVENT_FLAG_NONE;
}


Evas_Event_Flags _flick_move(void *data, void *event_info)
{
	Elm_Gesture_Line_Info *p = (Elm_Gesture_Line_Info *) event_info;

	MSG_LOW("flick moving x1,x2=(%d,%d) tx=%u mx=%d n=%u",
		  p->momentum.x1, p->momentum.x2, p->momentum.tx,
		  p->momentum.mx, p->momentum.n);

	return EVAS_EVENT_FLAG_NONE;

}


Evas_Event_Flags _flick_end(void *data, void *event_info)
{
	Elm_Gesture_Line_Info *p = (Elm_Gesture_Line_Info *) event_info;

	MSG_LOW("flick ended x1,x2=(%d,%d) tx=%u mx=%d n=%u",
		  p->momentum.x1, p->momentum.x2, p->momentum.tx,
		  p->momentum.mx, p->momentum.n);
	return EVAS_EVENT_FLAG_NONE;
}


Evas_Event_Flags _flick_abort(void *data, void *event_info)
{
	Elm_Gesture_Line_Info *p = (Elm_Gesture_Line_Info *) event_info;

	MSG_LOW("flick aborted x1,x2=(%d,%d) tx=%u mx=%d n=%u",
		  p->momentum.x1, p->momentum.x2, p->momentum.tx,
		  p->momentum.mx, p->momentum.n);
	return EVAS_EVENT_FLAG_NONE;
}



Evas_Event_Flags _dbl_click_start(void *data , void *event_info)
{
	Elm_Gesture_Taps_Info *p = (Elm_Gesture_Taps_Info *) event_info;

	MSG_MED("Double click start");

	if ( p->n != 1 ) return EVAS_EVENT_FLAG_NONE;

	return EVAS_EVENT_FLAG_NONE;
}

Evas_Event_Flags _dbl_click_end(void *data , void *event_info)
{
   Elm_Gesture_Taps_Info *p = (Elm_Gesture_Taps_Info *) event_info;

   MSG_MED("Double click end");

   if ( p->n != 1 ) return EVAS_EVENT_FLAG_NONE;

   return EVAS_EVENT_FLAG_NONE;
}

Evas_Event_Flags _dbl_click_abort(void *data , void *event_info)
{
   Elm_Gesture_Taps_Info *p = (Elm_Gesture_Taps_Info *) event_info;

   MSG_MED("Double click abort");

   if ( p->n != 1 ) return EVAS_EVENT_FLAG_NONE;

   return EVAS_EVENT_FLAG_NONE;
}

Evas_Event_Flags n_finger_tap_start(void *data , void *event_info)
{
	Elm_Gesture_Taps_Info *p = (Elm_Gesture_Taps_Info *) event_info;

	MSG_MED("Finger tab start. Time=%d", p->timestamp);

	if ( p->n != 1 ) return EVAS_EVENT_FLAG_NONE;

	return EVAS_EVENT_FLAG_NONE;
}


Evas_Event_Flags n_finger_tap_end(void *data , void *event_info)
{
	Elm_Gesture_Taps_Info *p = (Elm_Gesture_Taps_Info *) event_info;

	MSG_MED("Finger tab end. Time=%d", p->timestamp);

	if ( p->n != 1 ) return EVAS_EVENT_FLAG_NONE;

	struct Smart_Data *sd = (struct Smart_Data *) evas_object_smart_data_get((Evas_Object *)data);
	MSG_ASSERT(sd != NULL);

	Slide_Item *si = sd->slide[CENTER_SLIDE];

	if ( ivug_slider_item_icon_click_check(si, p->x, p->y ) == true )
	{
		evas_object_smart_callback_call(sd->obj, "slider,clicked,icon", si);	// Clicked video icon
	}
	else
	{
		evas_object_smart_callback_call(sd->obj, "slider,clicked", si);
	}

	return EVAS_EVENT_FLAG_NONE;
}

Evas_Event_Flags n_finger_tap_abort(void *data , void *event_info)
{
	Elm_Gesture_Taps_Info *p = (Elm_Gesture_Taps_Info *) event_info;

	MSG_MED("Finger tab aborted. Time=%d", p->timestamp);

	if ( p->n != 1 ) return EVAS_EVENT_FLAG_NONE;

	return EVAS_EVENT_FLAG_NONE;
}


Evas_Event_Flags n_long_tap_start(void *data , void *event_info)
{
	Elm_Gesture_Taps_Info *p = (Elm_Gesture_Taps_Info *) event_info;

	MSG_MED("Long tab start, x=%d, y=%d", p->x, p->y);

	if ( p->n != 1 ) return EVAS_EVENT_FLAG_NONE;

	struct Smart_Data *sd = (struct Smart_Data *) evas_object_smart_data_get((Evas_Object *)data);
	MSG_ASSERT(sd != NULL);

	sd->longtap_count = 0;

	return EVAS_EVENT_FLAG_NONE;

}

Evas_Event_Flags n_long_tap_move(void *data , void *event_info)
{
	Elm_Gesture_Taps_Info *p = (Elm_Gesture_Taps_Info *) event_info;

	MSG_MED("Long tab move");

	if ( p->n != 1 ) return EVAS_EVENT_FLAG_NONE;

	struct Smart_Data *sd = (struct Smart_Data *) evas_object_smart_data_get((Evas_Object *)data);
	MSG_ASSERT(sd != NULL);

	if(sd->longtap_count != 0)
	{
		return EVAS_EVENT_FLAG_NONE;
	}
	sd->longtap_count = 1;

	if(sd->bLongtapEnable == false)
	{
		MSG_HIGH("It is not long tap case");
		return EVAS_EVENT_FLAG_NONE;
	}

	if(sd->bMomentumStarted == true) // compare down and longpressed position
	{
		sd->longtap_prev_x = p->x;
		sd->longtap_prev_y = p->y;

		int longtap_dx = sd->prev_x - sd->longtap_prev_x;
		int longtap_dy = sd->prev_y - sd->longtap_prev_y;

		if(sqrt(longtap_dx*longtap_dx + longtap_dy*longtap_dy) > LOGNTAP_THRESHOLD)
		{
			MSG_HIGH("It is not long tap");
			return EVAS_EVENT_FLAG_NONE;
		}
	}

	evas_object_smart_callback_call(sd->obj, "slider,longpress,start", p);

	return EVAS_EVENT_FLAG_NONE;
}

Evas_Event_Flags n_long_tap_end(void *data , void *event_info)
{
	Elm_Gesture_Taps_Info *p = (Elm_Gesture_Taps_Info *) event_info;

	MSG_MED("Long tab end");

	if ( p->n != 1 ) return EVAS_EVENT_FLAG_NONE;

	MSG_MED("Long press end");

	struct Smart_Data *sd = (struct Smart_Data *) evas_object_smart_data_get((Evas_Object *)data);
	MSG_ASSERT(sd != NULL);

	evas_object_smart_callback_call(sd->obj, "slider,longpress,end", (void*)LONGTAP_ENDED);

	return EVAS_EVENT_FLAG_NONE;

}

Evas_Event_Flags n_long_tap_abort(void *data , void *event_info)
{
	Elm_Gesture_Taps_Info *p = (Elm_Gesture_Taps_Info *) event_info;

	MSG_MED("Long tab abort");

	if ( p->n != 1 ) return EVAS_EVENT_FLAG_NONE;

	return EVAS_EVENT_FLAG_NONE;

}


