/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __IVUG_SLIDER_PRIV_H__
#define __IVUG_SLIDER_PRIV_H__

#include "ivug-define.h"
#include "ivug-medialist.h"

/*********************************************************************/
/* Configuration                                                                                                 */
/*********************************************************************/

/* elm_config_finger_size_get = 40 */
#define IMAGE_BETWEEN_MARGIN		(30)					// The value is width between photocam(unit is pixel).
#define IMAGE_MOVE_MARGIN 			((int)IMAGE_BETWEEN_MARGIN*0.8)
/* If moved width is larger than IMAGE_SCROLL_MARGIN, image is scrolled */
#define IMAGE_SCROLL_MARGIN 		(5) /* vertical move doen't over 5 pixel width */
#define LONG_PRESS_MARGIN 			((int)(elm_config_finger_size_get() / 3))


#define ANIN_VELOCITY (5000)

#undef ENABLE_Y_SCR

typedef int slide_edge_t;

namespace EDGE {
	const int SLIDE_NO_EDGE = (1<<0);		// 1

	const int SLIDE_LEFT_EDGE = (1<<1);		// 2
	const int SLIDE_RIGHT_EDGE = (1<<2);	// 4

#ifdef ENABLE_Y_SCR
	const int SLIDE_TOP_EDGE = (1<<3);		// 8
	const int SLIDE_BOTTOM_EDGE = (1<<4);	// 16
#endif

	const int SLIDE_FIT = SLIDE_LEFT_EDGE | SLIDE_RIGHT_EDGE
#ifdef ENABLE_Y_SCR
		| SLIDE_TOP_EDGE | SLIDE_BOTTOM_EDGE
#endif
	;
};

typedef enum {
	SLIDE_SHIFT_NONE = 0x00,
	SLIDE_SHIFT_TO_RIGHT,		// <---------------------Mouse
	SLIDE_SHIFT_TO_LEFT,		// Mouse--------------------->
} shift_direction_t;

class ECoreAnimator;
class CAnimator;

#undef USE_CALC_JOB

// slider
typedef struct Smart_Data
{
 	Evas_Object *obj;				// smart object itself.
	Evas_Coord x, y, w, h;			// Geometry of slider

	Evas_Object *clipper;

//silde
	Slide_Item* slide[MAX_SLIDE];

//data list
	Eina_Bool bShow;

	Media_List*	media_list;		// Storing entire media list.

// for flick anim
	CAnimator *tween;
	ECoreAnimator *anim;

	shift_direction_t last_dir;
	int momentum;

//Mouse event handler
	Evas_Object *event_obj;	// Mouse event receiver.
	Evas_Object *gesture;

	bool bPassEvent;
	bool bZooming;

	int prev_x;
	int prev_y;

	bool bLongtapEnable;
	bool bMomentumStarted;

	int longtap_count;
	int longtap_prev_x;
	int longtap_prev_y;

	slide_edge_t edge;

	Ecore_Idler *load_idler;
	Ecore_Idler *item_load_idler;

	unsigned int zoom_level;

#ifdef USE_CALC_JOB
	Ecore_Job *calc_job;
#endif


} _Smart_Data;


#ifdef __cplusplus
extern "C" {
#endif



// Common function between slide show and slider.

void _ivug_slider_slide_move(struct Smart_Data *sd, Evas_Coord x, Evas_Coord y);

// Common function between slide and slide mouse.

void _ivug_slider_start_slide(struct Smart_Data *sd, int src_x, int dst_x, int momentum);

void _ivug_slider_slide_shift(struct Smart_Data *sd, shift_direction_t direction);

Evas_Event_Flags _zoom_start(void *data, void *event_info);

Evas_Event_Flags _zoom_move(void *data, void *event_info);

Evas_Event_Flags _zoom_end(void *data, void *event_info);

Evas_Event_Flags _zoom_abort(void *data, void *event_info);

Evas_Event_Flags _momentum_start(void *data, void *event_info);

Evas_Event_Flags _momentum_move(void *data, void *event_info);

Evas_Event_Flags _momentum_end(void *data, void *event_info);

Evas_Event_Flags _momentum_abort(void *data, void *event_info);

Evas_Event_Flags _dbl_click_start(void *data , void *event_info);

Evas_Event_Flags _dbl_click_end(void *data , void *event_info);

Evas_Event_Flags _dbl_click_abort(void *data , void *event_info);

Evas_Event_Flags _flick_start(void *data, void *event_info);

Evas_Event_Flags _flick_move(void *data, void *event_info);

Evas_Event_Flags _flick_end(void *data, void *event_info);

Evas_Event_Flags _flick_abort(void *data, void *event_info);

Evas_Event_Flags n_finger_tap_start(void *data , void *event_info);

Evas_Event_Flags n_finger_tap_end(void *data , void *event_info);

Evas_Event_Flags n_finger_tap_abort(void *data , void *event_info);

Evas_Event_Flags n_long_tap_start(void *data , void *event_info);

Evas_Event_Flags n_long_tap_move(void *data , void *event_info);

Evas_Event_Flags n_long_tap_end(void *data , void *event_info);

Evas_Event_Flags n_long_tap_abort(void *data , void *event_info);

void _put_originate(struct Smart_Data *sd, shift_direction_t dir, int momentum);


#ifdef __cplusplus
}
#endif



#endif // __IVUG_SLIDER_PRIV_H__

