/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <ui-gadget-module.h>
#include <app.h>

#include <Ecore_X.h> 		/*EFL*/

#include "ivug-common.h"	//common
#include "ivug-util.h"
#include "ivug-main-view.h"
#include "ivug-setas-view.h"
#include "ivug-crop-ug.h"
#include "ivug-details-view.h"
#include "ivug-popup.h"
#include "ivug-slideshow-view.h"


#include "EFLUtil.h"


#include "ivug-callback.h"
#include "ivug-context.h"

#include "ivug-parameter.h"

#define SHORTCUT_ICON_PATH	"/usr/share/icons/default/small/org.tizen.image-viewer.png"

typedef struct ug_data {
	Evas_Object *base;			// UG layout

// View Data;
	Ivug_MainView *main_view;
	Ivug_SetAsView *setas_view;
	IvugCropUG *crop_ug;

	Ivug_SlideShowView *ss_view;

	ivug_parameter* ivug_param;

	bool bError;
	char *bErrMsg;

	Evas_Object *icon;
	Ecore_Timer *exit_timer;

	Evas_Object *navi_bar;
	Elm_Object_Item *navi_it;
} _AppData;


#undef TRACE_RENDER

static void _on_base_resize(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	Evas_Coord x,y,w,h;
	evas_object_geometry_get(obj, &x, &y, &w, &h);

	MSG_IMAGEVIEW_HIGH("Base layout resized geomtery XYWH(%d,%d,%d,%d) angle=%d", x, y, w, h, elm_win_rotation_get((Evas_Object *)ug_get_window()));
}

static void _on_base_move(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	Evas_Coord x,y,w,h;
	evas_object_geometry_get(obj, &x, &y, &w, &h);

	MSG_IMAGEVIEW_HIGH("Base layout moved geomtery XYWH(%d,%d,%d,%d) angle=%d", x, y, w, h, elm_win_rotation_get((Evas_Object *)ug_get_window()));
}


static void _on_base_show(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	MSG_IMAGEVIEW_HIGH("Base layout show");
}

static void _on_base_hide(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	MSG_IMAGEVIEW_HIGH("Base layout hide");
}

static void _on_receive_mouse_down (void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	MSG_MAIN_HIGH("Base layout clicked : %s Layer=%d",evas_object_name_get(obj), evas_object_layer_get(obj));
}

static void _print_render_method(Evas *e)
{
	IV_ASSERT(e != NULL );

	Eina_List *engines, *l;
	int cur_id;
	int id;

	engines = evas_render_method_list ();
	if (!engines) {
		MSG_IMAGEVIEW_ERROR("No engine is specified");
		return;
	}

	cur_id = evas_output_method_get (e);

	char *name;
	void *data;
	EINA_LIST_FOREACH (engines, l, data) {
		name = (char *)data;
		id = evas_render_method_lookup (name);
		if (name && id == cur_id)
		{
			MSG_IMAGEVIEW_HIGH("Render method is %s", name);
			break;
		}
	}

}


#ifdef TRACE_RENDER
static void _on_render_post(void *data, Evas *e, void *event_info)
{
	MSG_IMAGEVIEW_HIGH("***************render_post***************");

#if 0
	struct ug_data *ugd = (struct ug_data *)data;

	if ( ugd->main_view && ugd->base )
	{
		EFL::dump_obj(ugd->base,0);
	}
#endif

}

static void _on_render_pre(void *data, Evas *e,  void *event_info)
{
	MSG_IMAGEVIEW_HIGH("***************render_pre ***************");

#if 0
	struct ug_data *ugd = (struct ug_data *)data;

	if ( ugd->main_view && ugd->base )
	{
		EFL::dump_obj(ugd->base,0);
	}
#endif
}
#endif



static void _on_setas_view_destroyed(void *data, Evas_Object *obj, void *event_info)
{
	IV_ASSERT(data != NULL);
	struct ug_data *ugd = (struct ug_data *)data;

	if(ugd->setas_view)
	{
		ivug_setas_view_destroy(ugd->setas_view);
		ugd->setas_view = NULL;
	}

	ug_destroy_me(gGetUGHandle());
}


static void
_on_timeout_response(void *data, Evas_Object *obj, void *event_info)
{
	IV_ASSERT(data != NULL);

	ui_gadget_h ug = (ui_gadget_h)data;

	MSG_IMAGEVIEW_ERROR("ERROR EXIT");

	ug_destroy_me(ug);
}

static Evas_Object* _set_win_icon(void)
{
// set window icon
	Evas_Object *o = evas_object_image_add(evas_object_evas_get((Evas_Object *)ug_get_window()));
	evas_object_image_file_set(o, SHORTCUT_ICON_PATH, NULL);
	elm_win_icon_object_set((Evas_Object *)ug_get_window(), o);

	return o;
}

static void _win_profile_changed_cb(void *data, Evas_Object * obj, void *event_info)
{
	if (data == NULL) return;

	struct ug_data *ugd = (struct ug_data *)data;

	const char *profile = elm_config_profile_get();

	if (!strcmp(profile, "desktop"))
	{  // desktop mode
		gSetDesktopMode(true);

		// hide layout's indicator area
		elm_win_indicator_mode_set((Evas_Object *)ug_get_window(), ELM_WIN_INDICATOR_HIDE);

		// set window icon
		if (!ugd->icon)
		{
			ugd->icon = _set_win_icon();
		}
	}
	else
	{   // mobile
		gSetDesktopMode(false);

		// show layout's indicator area
		elm_win_indicator_mode_set((Evas_Object *)ug_get_window(), ELM_WIN_INDICATOR_SHOW);
	}
}

static Eina_Bool _on_exit_timer_expired(void *data)
{
	ivug_retv_if(!data, ECORE_CALLBACK_CANCEL);

	struct ug_data *ugd = (struct ug_data *)data;

	ugd->exit_timer = NULL;

	DESTROY_ME();

	return ECORE_CALLBACK_CANCEL;
}

static bool _is_mmc_inserted(void)
{
	int noti_message;
	int ret = -1;

	ret = vconf_get_int(VCONFKEY_SYSMAN_MMC_STATUS, &noti_message);

	if ( ret != 0 ) // falied
	{
		MSG_NOTI_ERROR("Get %s is failed", VCONFKEY_SYSMAN_MMC_STATUS);
		return false;
	}

	if(noti_message == VCONFKEY_SYSMAN_MMC_MOUNTED)
	{
		return true;
	}

	return false;

}

static void _on_mmc_state_changed(keynode_t* node, void *data)
{
	struct ug_data *ugd = (struct ug_data *)data;
	if (NULL == ugd) {
		return;
	}

	if(_is_mmc_inserted() == false)
	{
		MSG_IMAGEVIEW_WARN("MMC Removed!");
		if(strncmp(ugd->ivug_param->filepath, PATH_SDCARD, strlen(PATH_SDCARD)) != 0
			&& ugd->ivug_param->view_by != IVUG_VIEW_BY_ALL)
		{
			return;
		}

		MSG_IMAGEVIEW_WARN("Request destroy UG=0x%08x", gGetUGHandle());

		if(ugd->main_view)
		{
			_ivug_main_on_mmc_state_changed(node, ugd->main_view);
		}
		else if(ugd->ss_view)
		{
			_ivug_slideshow_view_on_mmc_state_changed(node, ugd->ss_view);
		}
		else
		{
			//ivug_timeout_popup_show(ugd->base, _on_timeout_response, gGetUGHandle(), IDS_ERROR, IDS_SD_CARD_REMOVED);
			if(ugd->exit_timer == NULL)
			{
				ugd->exit_timer = ecore_timer_add(0.2, _on_exit_timer_expired, ugd);
			}
		}
	}
}

static Evas_Object *create_fullview(Evas_Object *win, struct ug_data *ugd)
{
	Evas_Object *base;

	/* Create Full view */
	base = EFL::create_layout(win, EDJ_PATH"/ivug-base.edj", "ivug_base");
	if (base == NULL)
	{
		MSG_IMAGEVIEW_HIGH("Cannot set layout. EDJ=%s Group=%s", EDJ_PATH"/ivug-base.edj", "ivug_base");
		return NULL;
	}

	evas_object_name_set(base, "Base layout");

	evas_object_size_hint_expand_set(base, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	return base;
}

static Evas_Object *create_frameview(Evas_Object *parent, struct ug_data *ugd)
{
	Evas_Object *base;

	/* Create Frame view */
	base = elm_layout_add(parent);
	evas_object_name_set(base, "Base layout frame");
	elm_layout_theme_set(base, "layout", "application", "default");

	return base;
}


static void *on_create(ui_gadget_h ug, enum ug_mode mode, service_h service, void *priv)
{
	struct ug_data *ugd;
	Evas_Object *conformant;

	PERF_CHECK_END(LVL0, "UG_MODULE_INIT -> On Create");

	PERF_CHECK_BEGIN(LVL0, "On Create");

	MSG_IMAGEVIEW_HIGH("Image Viewer : %s ug=0x%08x", __func__, ug);

	if (!ug || !priv)
	{
		MSG_IMAGEVIEW_ERROR("Error. ug=0x%08x priv=0x%08x", ug, priv);
		return NULL;
	}

	ugd = (struct ug_data *)priv;

	Evas_Object *win = (Evas_Object *)ug_get_window();

	// it is for camera, camera can rotate even rotation is locked
	if(ivug_config_get_can_rotate() == false)
	{
		elm_win_rotation_with_resize_set(win, 0);
	}

	int wx, wy, ww, wh;
	int ret = -1;

	evas_object_geometry_get(win, &wx, &wy, &ww, &wh);

	MSG_IMAGEVIEW_HIGH("Parent Info. Layout(0x%08x) Win(0x%08x) Size(%d,%d,%d,%d)",
			ug_get_parent_layout(ug), win, wx, wy, ww, wh);

	PERF_CHECK_BEGIN(LVL1, "init context");
	//init
	if (!ivug_context_init(ug))
	{
		MSG_IMAGEVIEW_ERROR("ivug_main_init error");
		return NULL;
	}
	PERF_CHECK_END(LVL1, "init context");

	PERF_CHECK_BEGIN(LVL1, "parse bundle");

	ugd->ivug_param = ivug_param_create_from_bundle(service);
	if (  ugd->ivug_param == NULL )
	{
		MSG_IMAGEVIEW_ERROR("Cannot parse parameter");
		ugd->bError = true;
		ugd->bErrMsg = strdup(IDS_UNABLE_TO_OPEN_FILE);
		goto ON_CREATE_ERROR;
	}
	PERF_CHECK_END(LVL1, "parse bundle");

	PERF_CHECK_BEGIN(LVL1, "creating base");

	//create base
	if (mode == UG_MODE_FULLVIEW)
	{
		MSG_IMAGEVIEW_HIGH("create base layout for FullView");
		ugd->base = create_fullview(win, ugd);
	} else {
		MSG_IMAGEVIEW_HIGH("create base layout for FrameView");
		ugd->base = create_frameview(win, ugd);
	}

	PERF_CHECK_END(LVL1, "creating base");

	if ( ugd->base == NULL )
	{
		MSG_IMAGEVIEW_ERROR("Cannot create base view");
		ugd->bError = true;
		ugd->bErrMsg = strdup(IDS_UNABLE_TO_OPEN_FILE);
		goto ON_CREATE_ERROR;
	}

#ifdef TRACE_RENDER
	evas_event_callback_add(evas_object_evas_get(ugd->base), EVAS_CALLBACK_RENDER_PRE, _on_render_pre, ugd);
	evas_event_callback_add(evas_object_evas_get(ugd->base), EVAS_CALLBACK_RENDER_POST, _on_render_post, ugd);
#endif

	int ux1, uy1, uw1, uh1;

	evas_object_geometry_get(ugd->base, &ux1, &uy1, &uw1, &uh1);

	MSG_IMAGEVIEW_HIGH("UG base created : 0x%08x (%d,%d,%d,%d)", ugd->base, ux1, uy1, uw1, uh1);

	evas_object_event_callback_add(ugd->base, EVAS_CALLBACK_MOUSE_DOWN, _on_receive_mouse_down, NULL);

	evas_object_event_callback_add(ugd->base, EVAS_CALLBACK_RESIZE, _on_base_resize, ugd);
	evas_object_event_callback_add(ugd->base, EVAS_CALLBACK_MOVE, _on_base_move, NULL);

	evas_object_event_callback_add(ugd->base, EVAS_CALLBACK_SHOW, _on_base_show, NULL);
	evas_object_event_callback_add(ugd->base, EVAS_CALLBACK_HIDE, _on_base_hide, NULL);

	evas_object_smart_callback_add(win, "profile,changed", _win_profile_changed_cb, ugd);

	ret = vconf_notify_key_changed(VCONFKEY_SYSMAN_MMC_STATUS, _on_mmc_state_changed, ugd);
	if ( ret == -1 )
	{
		MSG_IMAGEVIEW_ERROR("Register %s is failed", VCONFKEY_SYSMAN_MMC_STATUS);
	}

	if(ugd->ivug_param->mode == IVUG_MODE_SETAS)
	{
		MSG_IMAGEVIEW_HIGH("UG types=%d", ugd->ivug_param->setas_type);

		ivug_set_screen_type type = IVUG_CTRLBAR_SET_SCREEN_MAX;

		ivug_setas_mode setas_mode = IVUG_SETAS_UG;
		if(ugd->ivug_param->bStandalone == true)
		{
			setas_mode = IVUG_SETAS_APPSVC;
		}

		if(ugd->ivug_param->setas_type == IVUG_SET_AS_UG_TYPE_CALLER_ID)
		{
			type = IVUG_CTRLBAR_SET_SCREEN_CALLERID;
			ugd->setas_view = ivug_setas_view_callerid_ug_create(ugd->base,	ugd->ivug_param->filepath,
				setas_mode, ugd->ivug_param->width, ugd->ivug_param->height);
		}
		else if(ugd->ivug_param->setas_type == IVUG_SET_AS_UG_TYPE_VIDEO_CALL_ID)
		{
			type = IVUG_CTRLBAR_SET_SCREEN_VIDEO_CALL_ID;
			ugd->setas_view = ivug_setas_view_video_call_id_ug_create(ugd->base, ugd->ivug_param->filepath,
				setas_mode, ugd->ivug_param->width, ugd->ivug_param->height);
		}
		else if(ugd->ivug_param->setas_type == IVUG_SET_AS_UG_TYPE_WALLPAPER)
		{
			type = IVUG_CTRLBAR_SET_SCREEN_HOME;
			ugd->crop_ug = ivug_setas_crop_ug_create(ugd->base, ugd->ivug_param->filepath,
				setas_mode, type);
		}
		else if(ugd->ivug_param->setas_type == IVUG_SET_AS_UG_TYPE_LOCKSCREEN)
		{
			type = IVUG_CTRLBAR_SET_SCREEN_LOCK;
			ugd->crop_ug = ivug_setas_crop_ug_create(ugd->base, ugd->ivug_param->filepath,
				setas_mode, type);
		}
		else if(ugd->ivug_param->setas_type == IVUG_SET_AS_UG_TYPE_WALLPAPER_N_LOCKSCREEN)
		{
			type = IVUG_CTRLBAR_SET_SCREEN_BOTH;
			ugd->crop_ug = ivug_setas_crop_ug_create(ugd->base, ugd->ivug_param->filepath,
				setas_mode, type);
		}
		else if(ugd->ivug_param->setas_type == IVUG_SET_AS_UG_TYPE_WALLPAPER_CROP)
		{
			type = IVUG_CTRLBAR_SET_SCREEN_HOME;
			setas_mode = IVUG_SETAS_CROP;
			ugd->crop_ug = ivug_setas_crop_ug_create(ugd->base, ugd->ivug_param->filepath,
				setas_mode, type);
		}
		else if(ugd->ivug_param->setas_type == IVUG_SET_AS_UG_TYPE_CROP)
		{
			// Crop UG
			ugd->crop_ug = ivug_crop_ug_create(ugd->base,
				ugd->ivug_param->width, ugd->ivug_param->height,
				ugd->ivug_param->bRatioFix, ugd->ivug_param->filepath);
		}
		else
		{
			MSG_IMAGEVIEW_ERROR("Invalid SetAS UG Type:%d", ugd->ivug_param->setas_type);
			return NULL;
		}

		if(ugd->setas_view == NULL && ugd->crop_ug == NULL)
		{
			MSG_IMAGEVIEW_ERROR("SetAS UG creation failed Type:%d", ugd->ivug_param->setas_type);
			ugd->bError = true;
			ugd->bErrMsg = strdup("Layout Loading Fail");
			goto ON_CREATE_ERROR;
		}

		Evas_Object *layout = NULL;

		if(ugd->setas_view)
		{
			Evas_Object *setasOjbect = ivug_setas_view_object_get(ugd->setas_view);
			evas_object_smart_callback_add(setasOjbect, "destroy", _on_setas_view_destroyed, ugd);

			ugd->navi_bar = ivug_naviframe_add(ugd->base, NULL);

			evas_object_name_set(ugd->navi_bar, "UG naviframe");
			// Layout life cycle is controlled by application explictily.
			elm_naviframe_content_preserve_on_pop_set(ugd->navi_bar, EINA_TRUE);

			Evas_Object *back_btn = ivug_button_add(ugd->navi_bar, "naviframe/end_btn/default",
						IDS_BACK, NULL, _on_setas_view_destroyed, ugd);

			ugd->navi_it = elm_naviframe_item_push(ugd->navi_bar,
									NULL, back_btn, NULL, setasOjbect, NULL);
			elm_naviframe_item_title_visible_set(ugd->navi_it, EINA_FALSE);

			//conformant = (Evas_Object *)ug_get_conformant();
			//elm_object_signal_emit(conformant, "elm,state,indicator,nooverlap", "");

			ivug_setas_view_create_menu(ugd->setas_view, ugd->navi_bar, type);

			ivug_setas_view_load_file(ugd->setas_view, ugd->ivug_param->filepath, type);

			//layout = ivug_setas_view_object_get(ugd->setas_view);
			layout = ugd->navi_bar;
		}
		else if(ugd->crop_ug)
		{
			layout = ivug_crop_ug_get_layout(ugd->crop_ug);
		}

		elm_object_part_content_set(ugd->base, "elm.swallow.content", layout);	//swallow
	}
	else if (ugd->ivug_param->mode == IVUG_MODE_SLIDESHOW || ugd->ivug_param->footsteps)
	{
		PERF_CHECK_BEGIN(LVL1, "slideshow_view_create");

		ugd->ss_view = ivug_slideshow_view_create(ugd->base, ugd->ivug_param->mode, ugd->ivug_param->view_by);

		PERF_CHECK_END(LVL1, "slideshow_view_create");

		if (ugd->ss_view == NULL) //set main view.
		{
			MSG_IMAGEVIEW_ERROR("Best View Layout Lading Fail");
			ugd->bError = true;
			ugd->bErrMsg = strdup("Layout Loading Fail");
			goto ON_CREATE_ERROR;
		}

// Load list.
		PERF_CHECK_BEGIN(LVL1, "slideshow_view_load_list");

		if ( ivug_slideshow_view_set_list(ugd->ss_view, ugd->ivug_param) == false)
		{
			MSG_IMAGEVIEW_ERROR("Cannot load media list for bestview.");
			ugd->bError = true;
			ugd->bErrMsg = strdup(IDS_UNABLE_TO_OPEN_FILE);
			goto ON_CREATE_ERROR;
		}

		Evas_Object *layout = ivug_slideshow_view_object_get(ugd->ss_view);

		elm_object_part_content_set(ugd->base, "elm.swallow.content", layout);	//swallow

		PERF_CHECK_END(LVL1, "slideshow_view_load_list");

	}
	else
	{
		PERF_CHECK_BEGIN(LVL1, "main_view_create");

		ugd->main_view = ivug_main_view_create(ugd->base, ugd->ivug_param->mode, ugd->ivug_param->view_by);

		PERF_CHECK_END(LVL1, "main_view_create");

		if (ugd->main_view == NULL)	//set main view.
		{
			MSG_IMAGEVIEW_ERROR("Main View Layout Loading Fail");
			ugd->bError = true;
			ugd->bErrMsg = strdup("Layout Loading Fail");
			goto ON_CREATE_ERROR;
		}

// Load list.
		PERF_CHECK_BEGIN(LVL1, "main_view_set_list");

		if ( ivug_main_view_set_list(ugd->main_view, ugd->ivug_param) == false)
		{
			MSG_IMAGEVIEW_ERROR("Cannot load media list.");
			// Need popup?
			ugd->bError = true;
			ugd->bErrMsg = strdup(IDS_UNABLE_TO_OPEN_FILE);
			goto ON_CREATE_ERROR;
		}

		Evas_Object *layout = ivug_main_view_object_get(ugd->main_view);
		elm_object_part_content_set(ugd->base, "elm.swallow.content", layout);	//swallow

		PERF_CHECK_END(LVL1, "main_view_set_list");
	}

	PERF_CHECK_END(LVL0, "On Create");

	PERF_CHECK_BEGIN(LVL0, "On Create -> On Start");

//	evas_object_size_hint_weight_set(ugd->base, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	int ux, uy, uw, uh;

	evas_object_geometry_get(ugd->base, &ux, &uy, &uw, &uh);

	MSG_IMAGEVIEW_HIGH("Image Viewer : %s Base(0x%08x) Geometry(%d,%d,%d,%d)", __func__, ugd->base, ux, uy, uw, uh);

	return ugd->base;

ON_CREATE_ERROR:
	if(ugd->main_view)
	{
		ivug_main_view_destroy(ugd->main_view);
		ugd->main_view = NULL;
	}

	if (ugd->base == NULL)
	{
		ugd->base = elm_layout_add((Evas_Object *)ug_get_window());
		elm_layout_theme_set(ugd->base, "layout", "application", "default");
	}

	return ugd->base;
}

static void on_start(ui_gadget_h ug, service_h service, void *priv)
{
	PERF_CHECK_END(LVL0, "On Create -> On Start");

	PERF_CHECK_BEGIN(LVL0, "On Start");

	if (!ug || !priv)
	{
		MSG_IMAGEVIEW_ERROR("Invalid UG. UG=0x%08x Priv=0x%08x", ug, priv);
		return ;
	}

	struct ug_data *ugd = (struct ug_data *)priv;

	int ux, uy, uw, uh;

	evas_object_geometry_get(ugd->base, &ux, &uy, &uw, &uh);

	MSG_IMAGEVIEW_HIGH("Image Viewer : %s BaseGeometry(%d,%d,%d,%d)", __func__, ux, uy, uw, uh);

	if(ugd->bError == true)
	{
		PERF_CHECK_END(LVL0, "On Start");
		MSG_IMAGEVIEW_ERROR("UG create has ERROR");
		ivug_timeout_popup_show(ugd->base, _on_timeout_response, ug, IDS_ERROR, ugd->bErrMsg);
		return;
	}
	if ( ugd->ivug_param->mode == IVUG_MODE_SLIDESHOW || ugd->ivug_param->footsteps)
	{
		PERF_CHECK_BEGIN(LVL1, "slideshow_view_start");
		ivug_slideshow_view_start(ugd->ss_view);
		PERF_CHECK_END(LVL1, "slideshow_view_start");
	}
	else if ( ugd->main_view )
	{
		PERF_CHECK_BEGIN(LVL1, "main_view_start");
		ivug_main_view_start(ugd->main_view);
		PERF_CHECK_END(LVL1, "main_view_start");
	}
	else if ( ugd->crop_ug )
	{
		ivug_crop_ug_start(ugd->crop_ug);
	}


	PERF_CHECK_END(LVL0, "On Start");
}

static void on_pause(ui_gadget_h ug, service_h service, void *priv)
{
	MSG_IMAGEVIEW_HIGH("Image Viewer : %s", __func__);

	if (!ug || !priv)
	{
		MSG_IMAGEVIEW_ERROR("Invalid UG. UG=0x%08x Priv=0x%08x", ug, priv);
		return ;
	}

	struct ug_data *ugd = (struct ug_data *)priv;

	if(ugd->ivug_param == NULL)
	{
		MSG_IMAGEVIEW_ERROR("UG is destroying");
		return;
	}

	if ( ugd->ivug_param->mode == IVUG_MODE_SLIDESHOW || ugd->ivug_param->footsteps)
	{
		ivug_slideshow_view_pause(ugd->ss_view);
	}
	else if ( ugd->main_view )
	{
		ivug_main_view_pause(ugd->main_view);
	}

}

static void on_resume(ui_gadget_h ug, service_h service, void *priv)
{
	MSG_IMAGEVIEW_HIGH("Image Viewer : %s", __func__);

	if (!ug || !priv)
	{
		IVUG_DEBUG_MSG("Invalid UG. UG=0x%08x Priv=0x%08x", ug, priv);
		return ;
	}

	struct ug_data *ugd = (struct ug_data *)priv;

	if ( ugd->ivug_param->mode == IVUG_MODE_SLIDESHOW || ugd->ivug_param->footsteps)
	{
		ivug_slideshow_view_resume(ugd->ss_view);
	}
	else if ( ugd->main_view )
	{
		ivug_main_view_resume(ugd->main_view);
	}

}

static void on_destroy(ui_gadget_h ug, service_h service, void *priv)
{
	MSG_IMAGEVIEW_HIGH("Image Viewer : %s(0x%08x) UG=0x%08x", __func__, on_destroy, ug);

	PERF_CHECK_BEGIN(LVL0, "On Destroy");

	if (!ug || !priv)
	{
		MSG_IMAGEVIEW_ERROR("Invalid UG. UG=0x%08x Priv=0x%08x", ug, priv);
		return ;
	}

	struct ug_data *ugd = (struct ug_data *)priv;

#ifdef TRACE_RENDER
	evas_event_callback_del_full(evas_object_evas_get(ugd->base), EVAS_CALLBACK_RENDER_PRE, _on_render_pre, ugd);
	evas_event_callback_del_full(evas_object_evas_get(ugd->base), EVAS_CALLBACK_RENDER_POST, _on_render_post, ugd);
#endif

	MSG_IMAGEVIEW_HIGH("On Destroy : ug=0x%08x", ug);

	int ret = vconf_ignore_key_changed(VCONFKEY_SYSMAN_MMC_STATUS, _on_mmc_state_changed);
	if ( ret == -1 )
	{
		MSG_MAIN_ERROR("Unegister %s is failed", VCONFKEY_SYSMAN_MMC_STATUS);
	}

	if(ugd->bErrMsg)
	{
		free(ugd->bErrMsg);
		ugd->bErrMsg = NULL;
	}

	if(ugd->setas_view)
	{
		PERF_CHECK_BEGIN(LVL1, "SetAsView");
		ivug_setas_view_destroy(ugd->setas_view);
		ugd->setas_view = NULL;
		PERF_CHECK_END(LVL1, "SetAsView");
	}

	if(ugd->crop_ug)
	{
		ivug_crop_ug_destroy(ugd->crop_ug);
		ugd->crop_ug = NULL;
	}
	//destroy main view.
	if ( ugd->main_view )
	{
		PERF_CHECK_BEGIN(LVL1, "MainView");
		ivug_main_view_destroy(ugd->main_view);
		ugd->main_view = NULL;
		PERF_CHECK_END(LVL1, "MainView");
	}

	if ( ugd->ss_view )
	{
		PERF_CHECK_BEGIN(LVL1, "SlideShowView");
		ivug_slideshow_view_destroy(ugd->ss_view);
		ugd->ss_view = NULL;
		PERF_CHECK_END(LVL1, "SlideShowView");
	}
	//delete param.
	if(ugd->ivug_param)
	{
		ivug_param_delete(ugd->ivug_param);
		ugd->ivug_param = NULL;
	}

	//finalize data
	PERF_CHECK_BEGIN(LVL1, "Context");
	if ( !ivug_context_deinit(ug))
	{
		MSG_IMAGEVIEW_ERROR("ivug_main_deinit failed");
	}
	PERF_CHECK_END(LVL1, "Context");

	if (ugd->base)
	{
		PERF_CHECK_BEGIN(LVL1, "Base layout");
		evas_object_del(ugd->base);
		ugd->base = NULL;
		PERF_CHECK_END(LVL1, "Base layout");
	}

	MSG_IMAGEVIEW_HIGH("Destroyed ug");

	PERF_CHECK_END(LVL0, "On Destroy");
}

static bool _data_print(service_h service, const char *key, void *user_data)
{
	char *value;

	service_get_extra_data(service, key, &value);

	MSG_IVUG_HIGH("  %s : %s", key, value);

	free(value);

	return true;
}

static void on_message(ui_gadget_h ug, service_h msg, service_h service, void *priv)
{
	MSG_IMAGEVIEW_HIGH("Image Viewer : %s UG=0x%08x", __func__, ug);	//on message

	if (!ug || !priv)
	{
		MSG_IMAGEVIEW_ERROR("Invalid UG. UG=0x%08x Priv=0x%08x", ug, priv);
		return;
	}

	int ret = service_foreach_extra_data(msg, _data_print, NULL);

	if(SERVICE_ERROR_NONE != ret)
	{
		MSG_IVUG_ERROR("service_foreach_extra_data ERROR");
	}

	struct ug_data *ugd = (struct ug_data *)priv;

	//ivug_msg_type msg_type = IVUG_MSG_NONE;

	if(ugd->main_view == NULL)
	{
		MSG_IMAGEVIEW_ERROR("main view is NULL");
		return;
	}

	ivug_callback_call(gGetCallbackHandle(), msg, NULL, NULL);

	return;
}

static void on_event(ui_gadget_h ug, enum ug_event event, service_h service, void *priv)
{
	if (!ug || !priv)
	{
		MSG_IMAGEVIEW_ERROR("Invalid UG. UG=0x%08x Priv=0x%08x", ug, priv);
		return;
	}
	MSG_IMAGEVIEW_HIGH("Image Viewer : %s UG=0x%08x event=%d", __func__, ug, event);	//on message

	switch (event) {
	case UG_EVENT_LOW_MEMORY:
		MSG_IMAGEVIEW_HIGH("Get Event : Low Memory");
		break;
	case UG_EVENT_LOW_BATTERY:
		MSG_IMAGEVIEW_HIGH("Get Event : Low battery");
		break;
	case UG_EVENT_LANG_CHANGE:
		MSG_IMAGEVIEW_HIGH("Get Event : Language changed");
		break;

	case UG_EVENT_ROTATE_PORTRAIT:
	case UG_EVENT_ROTATE_LANDSCAPE:
	case UG_EVENT_ROTATE_PORTRAIT_UPSIDEDOWN:
	case UG_EVENT_ROTATE_LANDSCAPE_UPSIDEDOWN:
		// Rotate event is not used now.. plz, use only resized callback.
		break;

	default:
		MSG_IMAGEVIEW_ERROR("Unknown event type : %d", event);
		break;
	}
}

#if 0
#include <dlfcn.h>
#include <stdio.h>

#include <sys/time.h>

static int __load(int argc, char **argv)
{

	void *handle = NULL;

	struct timeval start_point, end_point;

	double operating_time;

	gettimeofday(&start_point, NULL);

	handle = dlopen(argv[1], RTLD_LAZY );

	gettimeofday(&end_point, NULL);

	operating_time = (double)(end_point.tv_sec)+(double)(end_point.tv_usec)/1000000.0-(double)(start_point.tv_sec)-(double)(start_point.tv_usec)/1000000.0;

	printf("Dlopen(%s) %f usec\n",argv[1], operating_time);

	if (handle == NULL)
	{
		printf("Cannot open : %s", argv[1]);
		return -1;
	}

//	dlclose(handle);

	return 0;
}
#endif

extern "C" UG_MODULE_API int UG_MODULE_INIT(struct ug_module_ops *ops)
{
	struct ug_data *ugd;

	MSG_IMAGEVIEW_HIGH("UG_MODULE_INIT. Ver=12.0");

	if (!ops)
	{
		MSG_IMAGEVIEW_ERROR("OPS Pointer is NULL");
		return -1;
	}

	ugd = (struct ug_data *)calloc(1, sizeof(struct ug_data));	//alloc ug_data memory
	if (!ugd)
	{
		MSG_IMAGEVIEW_ERROR("Cannot allocate memory.");
		return -1;
	}

	IV_PERF_INIT();

	ops->create = on_create;
	ops->start = on_start;
	ops->pause = on_pause;
	ops->resume = on_resume;
	ops->destroy = on_destroy;
	ops->message = on_message;
	ops->event = on_event;
	ops->priv = ugd;
	ops->opt = (ug_option)(UG_OPT_INDICATOR_ENABLE | UG_OPT_OVERLAP_ENABLE);

	PERF_CHECK_BEGIN(LVL0, "UG_MODULE_INIT -> On Create");

#if 0
	char *name[] = {
		"abc1",
		"/usr/ug/lib/libug-image-viewer-efl.so",
	};

	__load(2, name);

#endif

	return 0;
}

extern "C" UG_MODULE_API void UG_MODULE_EXIT(struct ug_module_ops *ops)
{
	struct ug_data *ugd;
	if (!ops)
	{
		MSG_IMAGEVIEW_ERROR("OPS Pointer is NULL");
		return;
	}

	ugd = (struct ug_data *)ops->priv;

	if (ugd)
	{
		free(ugd);
	}

	MSG_IMAGEVIEW_HIGH("UG_MODULE_EXIT");

#ifdef TA_SAVETO_FILE
	FILE *fp = NULL;

	fp = fopen(DATADIR"/"PACKAGE"/TimeAnal", "a");

	if ( fp != NULL )
	{
		PERF_SHOW_RESULT(fp);

		fclose(fp);
	}
	else
	{
		MSG_IMAGEVIEW_HIGH("Cannot open file : %s", DATADIR"/"PACKAGE"/TimeAnal");
	}
#else
	PERF_SHOW_RESULT(stderr);
#endif

	IV_PERF_DEINIT();

}


#define _CONSTRUCTOR __attribute__ ((constructor))
#define _DESTRUCTOR __attribute__ ((destructor))

_CONSTRUCTOR void _DLLInit(void)
{
	printf("Image Viewer - Called constructor\n");
	MSG_IMAGEVIEW_HIGH("Image Viewer - Called constructor");
}

_DESTRUCTOR void _DLLExit(void)
{
	printf("Image Viewer - Called destructor\n");
	MSG_IMAGEVIEW_HIGH("Image Viewer - Called destructor");
}


int main()
{
/*	For detecting undefined symbol */
	return -1;
}

