/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __IVUG_MAINVIEW_PRIV_H__
#define __IVUG_MAINVIEW_PRIV_H__

#include "ivug-define.h"

#ifdef __cplusplus
extern "C" {
#endif

struct _Ivug_MainView{
	Evas_Object *parent;
	Evas_Object *layout;		/*Layout widget based on EDJ*/

	Evas_Object *navi_bar;
	Evas_Object *empty_layout;
	Elm_Object_Item *navi_it;
	Evas_Object* slider;			//image viewer slider

	Evas_Object *back_btn;

	Evas_Object *area_obj;		//lcd area except indicator

	Evas_Object* selectioninfo;

	Evas_Object *zoom_slider;
	int zoom_level;

/*
   |------------------|
   | navi_bar		  |
   |------------------|
   | tool_bar		  |
   |------------------|
   |				  |
   |				  |
   |				  |
   |				  |
   |------------------|
   | ctrl_bar		  |
   |------------------|
*/

// Toolbar
	Evas_Object *tool_bar;
	ivug_toolbar tool_bar_type;

// Controlbar
	Evas_Object *ctrl_bar;
	ivug_ctrlbar ctrl_bar_type;

	Evas_Object *btn_more;
	Evas_Object *btn_share;
	Evas_Object *btn_download;

	bool bSetThmByUser;

	Evas_Object* popup; 			//popup
	Evas_Object* ctx_popup; 		//context popup 1depth
	Evas_Object* ctx_popup2; 		//context popup 2depth

	Evas_Object* longpress_popup;		//long press popup
	bool bLongpressEnd;

	Evas_Object *best_icon;

// child view.
	Ivug_NameView *pNameView;
	Ivug_SetAsView *pSetAsView;

	Ivug_DetailsView *pDetailsView;
	IvugCropView *pCropView;

// List
	Media_List *mList;

//flag
	bool bShowMenu;

	ivug_mode mode;
	ivug_view_by view_by;

	ui_gadget_h ext_ug;

	Ecore_Event_Handler *keydown_handler;

	Ecore_Timer *exit_timer;
	Ecore_Timer *hide_timer;
	int hide_count;

	Ecore_Timer *popup_timer;

	char *album_name;

// Slide show;
	SlideShow *ssHandle;

	bool bClickPrevent;
	bool bStandAlone;	//if true, it is process not ug
};

void _update_title(struct _Ivug_MainView *pMainView);

bool ivug_is_agif(const char *filepath);

#ifdef __cplusplus
}
#endif


#endif 		// __IVUG_MAINVIEW_PRIV_H__

