/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <ui-gadget-module.h>		// ug_destroy_me, ug_send_result
#include <vconf.h>
#include <vconf-keys.h>
#include <utilX.h>			// Key event
#include <math.h>			// round

#include "ivug-common.h"
#include "ivug-parameter.h"

#include "ivug-main-view.h"
#include "ivug-main-view-toolbar.h"
#include "ivug-main-view-menu.h"

#include "ivug-filter.h"

#include "ivug-ext-ug.h"

#include "ivug-util.h"
#include "ivug-medialist.h"
#include "ivug-media.h"

#include "ivug-popup.h"
#include "ivug-selectioninfo.h"

#include "ivug-slider-item.h"
#include "ivug-slider.h"

#include "ivug-setas-view.h"
#include "ivug-name-view.h"
#include "ivug-details-view.h"

#include "ivug-listpopup.h"
#include "ivug-slideshow.h"


#include "ivug-ext-ug.h"

#include "ivug-drm-common.h"
#include "EFLUtil.h"


#include "ivug-image.h"

#include "ivug-main-view-priv.h"

#undef HIDE_INDICATOR		// For testing.
#define TEST_MOUSE_EVENT

#define DEFAULT_THUMBNAIL_PATH			IMAGE_PATH"/T01_Nocontents_broken.png"

#define LONGTAP_TIMEOUT	(2.0) // 2secs

// EDJE
#define IVUG_MAIN_EDJ 	EDJ_PATH"/ivug-main.edj"
///////////////////////////////////////////////////////////////////////////////////////////////////
//
//
static bool _destory_slideshow_and_ug(Ivug_MainView *pMainView, int state, bool bMmc_out);
static void _on_slideshow_finished(void *data, Evas_Object *obj, void *event_info);

void _update_title(Ivug_MainView *pMainView)
{
#define MAX_TITLE_LEN	256

	static char title[MAX_TITLE_LEN];

	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);

	if (mdata == NULL)
	{
		MSG_MAIN_ERROR("slider data is NULL");
		return;
	}

	if (pMainView->view_by == IVUG_VIEW_BY_FOLDER
		|| pMainView->view_by == IVUG_VIEW_BY_ALL
		|| pMainView->view_by == IVUG_VIEW_BY_PLACES)
	{
		snprintf(title, sizeof(title), "%d/%d",
			mdata->index + 1,
			ivug_medialist_get_count(pMainView->mList));	//get title.
	}
	else if(mdata->fileurl != NULL)
	{
		const char* filename = ivug_get_filename(mdata->fileurl);
		if(filename)
		{
			snprintf(title, sizeof(title), "%s", filename);
		}
	}
	else
	{
		MSG_MAIN_ERROR("set Title error");
	}

	IV_ASSERT(pMainView->navi_bar != NULL);

	MSG_MAIN_HIGH("Update title : %s", title);

	elm_object_item_text_set(pMainView->navi_it, title);

}


static void
_on_longpress_popup_selected(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

// Destroy copy popup
	pMainView->longpress_popup = NULL;		// object is removed automatically

	int response = (int)event_info;

	Media_Item *mtiem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mtiem);
	if (mdata == NULL)
	{
		MSG_MAIN_ERROR("slider data is NULL");
		return;
	}

	if(response == LPPOPUP_RESPONSE_COPY)
	{
		//get current file path.

		char *path = NULL;
		int len = 0;
		// This Will add to the article
		char buf[IVUG_MAX_FILE_PATH_LEN] = {0,};

		{
			path = mdata->filepath;
			len = strlen(mdata->filepath)+strlen("file://")+1;
			snprintf(buf, IVUG_MAX_FILE_PATH_LEN, "file://%s", mdata->filepath);
		}

		if (len < IVUG_MAX_FILE_PATH_LEN)
		{
			MSG_MAIN_HIGH("file path = %s, file url =%s", path, buf);
			elm_cnp_selection_set(pMainView->layout, ELM_SEL_TYPE_CLIPBOARD, ELM_SEL_FORMAT_IMAGE, buf, sizeof(buf));
			_ivug_selectioninfo_create(pMainView, IDS_COPIED_TO_CLIPBOARD);
		}
		else
		{
			MSG_MAIN_ERROR("slider file path is too long");
		}
	}
}


static Eina_Bool _on_key_down(void *user_data, int type, void *event)
{
	if (!user_data) {
		MSG_IMAGEVIEW_ERROR("user data is NULL");
		return ECORE_CALLBACK_PASS_ON;
	}

	Ivug_MainView *pMainView = (Ivug_MainView *)user_data;

	Ecore_Event_Key *key_event = (Ecore_Event_Key *) event;

	if (!strcmp(key_event->keyname, KEY_END))
	{
		MSG_IMAGEVIEW_HIGH("Bakc(End) key");
	}
	else if (!strcmp(key_event->keyname, KEY_SELECT))
	{
		MSG_IMAGEVIEW_HIGH("Home key");
	}
	else if (!strcmp(key_event->keyname, KEY_POWER))
	{
		MSG_IMAGEVIEW_HIGH("Power key");
		if ( pMainView->ssHandle )
		{
			ivug_ss_stop(pMainView->ssHandle);
		}
	}

	MSG_IMAGEVIEW_LOW("Key down : %s", key_event->keyname);

	return ECORE_CALLBACK_PASS_ON;
}

static Eina_Bool _on_exit_timer_expired(void *data)
{
	ivug_retv_if(!data, ECORE_CALLBACK_CANCEL);

	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	pMainView->exit_timer = NULL;

	DESTROY_ME();

	return ECORE_CALLBACK_CANCEL;
}

static void _on_resize(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = static_cast<Ivug_MainView *>(data);
	IV_ASSERT(pMainView != NULL);

	Evas_Coord x,y,w,h;
	evas_object_geometry_get(obj, &x, &y, &w, &h);
	/**
	*  during slideshow, after rotation, resize cb of slideshow obj cannot be invoked,
	    so invoke it manually
	*/
	if(pMainView->ssHandle)
		ivug_ss_resize(pMainView->ssHandle);
	MSG_MAIN_HIGH("MainView resized geomtery XYWH(%d,%d,%d,%d) Rotate=%d", x, y, w, h, gGetRotationDegree());

	if(pMainView->ctx_popup)	// when rotated context popup will disapper at EFL
	{
		evas_object_del(pMainView->ctx_popup);
		pMainView->ctx_popup = NULL;
	}

	ivug_slider_zoom_reset(pMainView->slider);

	pMainView->zoom_level = 1;
	elm_slider_value_set(pMainView->zoom_slider, 1.0);
}

static bool _destory_slideshow_and_ug(Ivug_MainView *pMainView,
	int state,
	bool bMmc_out)
{
	IV_ASSERT(pMainView != NULL);
	evas_object_smart_callback_del_full(ivug_ss_object_get(pMainView->ssHandle),
		"slideshow,finished", _on_slideshow_finished, pMainView);

	ivug_allow_lcd_off();
	/*from gallery ablum*/
	if(bMmc_out)
	{
		if (pMainView->ssHandle)
		{
			MSG_MAIN_HIGH("image viewer end cause slide show ended");
			ivug_ss_delete(pMainView->ssHandle);
			pMainView->ssHandle = NULL;
		}

		DESTROY_ME();
		return true;
	}

	return false;
}


void _ivug_main_on_mmc_state_changed(keynode_t* node, void *data)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	if (NULL == pMainView) {
		return;
	}

	if ( pMainView->ext_ug )
	{
		MSG_IMAGEVIEW_HIGH("Extern UG is running");
	}

	if(pMainView->ssHandle)
	{
		ivug_ss_set_stop(pMainView->ssHandle);
		_destory_slideshow_and_ug(pMainView, SLIDE_SHOW_STOPPED, true);
	} else {
		if(pMainView->exit_timer == NULL)
		{
			pMainView->exit_timer = ecore_timer_add(0.2, _on_exit_timer_expired, data);
		}
	}
}

static void _show_zoom_icon(Ivug_MainView *pMainView)
{
	ivug_ret_if(!pMainView);

	MSG_MAIN_ERROR("_show_zoom_icon");

	/*unsigned int zoom_level = ivug_slider_zoom_level_get(pMainView->slider);
	if(zoom_level == 0)
	{
		edje_object_signal_emit(_EDJ(pMainView->empty_layout), "elm,state,btn_zoom_out,disable","elm");
		edje_object_signal_emit(_EDJ(pMainView->empty_layout), "elm,state,btn_zoom_in,enable","elm");
	}
	else if(zoom_level == ZOOM_LEVEL_MAX)
	{
		edje_object_signal_emit(_EDJ(pMainView->empty_layout), "elm,state,btn_zoom_in,disable","elm");
		edje_object_signal_emit(_EDJ(pMainView->empty_layout), "elm,state,btn_zoom_out,enable","elm");
	}
	else*/
	{
		edje_object_signal_emit(_EDJ(pMainView->empty_layout), "elm,state,btn_zoom_out,enable","elm");
		edje_object_signal_emit(_EDJ(pMainView->empty_layout), "elm,state,btn_zoom_in,enable","elm");
	}

	edje_object_signal_emit(_EDJ(pMainView->empty_layout), "elm,state,zoom_slider,show","elm");
}

static void _hide_zoom_icon(Ivug_MainView *pMainView)
{
	ivug_ret_if(!pMainView);

	MSG_MAIN_ERROR("_hide_zoom_icon");

	edje_object_signal_emit(_EDJ(pMainView->empty_layout), "elm,state,btn_zoom_out,hide","elm");
	edje_object_signal_emit(_EDJ(pMainView->empty_layout), "elm,state,btn_zoom_in,hide","elm");
	edje_object_signal_emit(_EDJ(pMainView->empty_layout), "elm,state,zoom_slider,hide","elm");
}

static void _update_main_view(Ivug_MainView *pMainView)
{
	IV_ASSERT(pMainView != NULL);

	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);

	if(mdata == NULL)
	{
		MSG_MAIN_ERROR("mdata is NULL");
		return;
	}

	PERF_CHECK_BEGIN(LVL6, "Set menu title");

	_update_title(pMainView);

	PERF_CHECK_END(LVL6, "Set menu title");

	slide_state_t state = ivug_slider_get_state(pMainView->slider);

	if(mdata->slide_type == SLIDE_TYPE_IMAGE && mdata->bIsDRMContent == false && state != SLIDE_STATE_ERROR)
	{
		_show_zoom_icon(pMainView);
	}
	else
	{
		_hide_zoom_icon(pMainView);
	}

	return;
}


static void
_ivug_main_drm_popup_purchase_cb(void *data, Evas_Object *obj, void *event_info)
{
	int response_id = (int)event_info;
	IV_ASSERT(data != NULL);

	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	MSG_MAIN_HIGH("DRM popup selected : %d", response_id);

	if(response_id == POPUP_RESPONSE_CANCEL)
	{
		MSG_MAIN_HIGH("cancel selected");
#ifdef USE_SLIDE_SKIP
		MSG_MAIN_HIGH("shift again");
		ivug_slider_skip(pMainView->slider);
#else
		ivug_slider_load(pMainView->slider);
#endif
	}
	else
	{
		MSG_MAIN_HIGH("unlock process");

		Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
		Media_Data *mdata = ivug_medialist_get_data(mitem);

		char *url = ivug_drm_get_right_url(mdata->filepath);
		if (url != NULL)
		{
			MSG_MAIN_HIGH("path = %s, right url : %s", mdata->filepath, url);
			ivug_ext_launch_browser(url);
			free(url);
		}
		else
		{
			MSG_MAIN_ERROR("ivug_drm_get_right_url error. path = %s", mdata->filepath);
		}
	}
}


static void
_ivug_main_drm_popup_response_cb(void *data, Evas_Object *obj, void *event_info)
{
	int response_id = (int)event_info;
	IV_ASSERT(data != NULL);

	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	MSG_MAIN_HIGH("DRM popup selected : %d", response_id);

	if(response_id == POPUP_RESPONSE_CANCEL)
	{
		MSG_MAIN_HIGH("cancel selected");
#ifdef USE_SLIDE_SKIP
		MSG_MAIN_HIGH("shift again");
		ivug_slider_skip(pMainView->slider);
#else
		ivug_slider_load(pMainView->slider);
#endif

	}
	else
	{
		ivug_slider_load(pMainView->slider);
	}
}


void _ivug_main_view_drm_process(drm_constraints result, int info, void *data)
{
#define MAX_DBUFFER_SIZE	2048

	ivug_ret_if(!data);
	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	char string[MAX_DBUFFER_SIZE] = "";
	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);
	slide_state_t state = ivug_slider_get_state(pMainView->slider);

	if(mdata == NULL)
	{
		MSG_MAIN_ERROR("mdata is NULL");
		return;
	}

	switch(result)
	{
		case IVUG_DRM_RESULT_LIMITED:
			MSG_MAIN_HIGH("Limited");
			snprintf(string, sizeof(string),
					IDS_DRM_UNLOCK,
					ecore_file_file_get(mdata->filepath));
			ivug_2btn_popup_show(pMainView->layout, NULL, string, _ivug_main_drm_popup_purchase_cb, pMainView);
			break;
		case IVUG_DRM_RESULT_UNLIMITED:
			MSG_MAIN_HIGH("Unlimited");
			if(state != SLIDE_STATE_LOADED)
			{
				ivug_slider_load(pMainView->slider);
			}
			break;
		case IVUG_DRM_RESULT_COUNT:
			MSG_MAIN_HIGH("Count = %d", info);
			if(info > 2)
			{
				ivug_slider_load(pMainView->slider);
			}
			else if(info == 2)
			{
				snprintf(string, sizeof(string),
					IDS_DRM_COUNT_BASED_OPEN_MANY, 2,
					ecore_file_file_get(mdata->filepath));
				ivug_2btn_popup_show(pMainView->layout, NULL, string, _ivug_main_drm_popup_response_cb, pMainView);
			}
			else
			{
				snprintf(string, sizeof(string),
					IDS_DRM_COUNT_BASED_OPEN_ONE,
					ecore_file_file_get(mdata->filepath));
				ivug_2btn_popup_show(pMainView->layout, NULL, string, _ivug_main_drm_popup_response_cb, pMainView);
			}
			break;
		case IVUG_DRM_RESULT_INTERVAL_DAYS:
			MSG_MAIN_HIGH("Interval days = %d", info);
			snprintf(string, sizeof(string),
					IDS_DRM_INTERVAL_BASED_DAYS,
					ecore_file_file_get(mdata->filepath),
					info);
			ivug_2btn_popup_show(pMainView->layout, NULL, string, _ivug_main_drm_popup_response_cb, pMainView);
			break;
		case IVUG_DRM_RESULT_ACCUMULATED:
			MSG_MAIN_HIGH("Accumulated time = %d", info);
			break;
		case IVUG_DRM_RESULT_DATETIME:
		case IVUG_DRM_RESULT_TIMED_COUNT:
		case IVUG_DRM_RESULT_SYSTEM:
			ivug_slider_load(pMainView->slider);
			break;
		default :
			MSG_MAIN_WARN("Unknown resule : %d", result);
			ivug_slider_load(pMainView->slider);
			break;
	}
}


static void
_on_slider_clicked(void *data, Evas_Object *obj, void *event_info)
{
	MSG_MAIN_HIGH("Main View Clicked");
	IV_ASSERT(data != NULL);

	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	if(pMainView->bClickPrevent == true)
	{
		pMainView->bClickPrevent = false;
		return;
	}

	if (pMainView->bShowMenu)
	{
		ivug_main_view_hide_menu_bar(pMainView);
	}
	else
	{
		ivug_main_view_show_menu_bar(pMainView);
	}

}

static void
_on_slider_icon_clicked(void *data, Evas_Object *obj, void *event_info)
{
	MSG_MAIN_HIGH("Main View Video icon Clicked");
	IV_ASSERT(data != NULL);

	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);
	if (mdata == NULL)
	{
		MSG_MAIN_ERROR("slide data is NULL");
		return;
	}

// "Add tag" in video player shows only when launching from gallery.
	else if (pMainView->mode == IVUG_MODE_FILE || pMainView->mode == IVUG_MODE_DISPLAY || pMainView->mode == IVUG_MODE_SINGLE )
	{
		MSG_MAIN_HIGH("Launching simple video player. Mode=%d", pMainView->mode);
		ivug_ext_launch_videoplayer_simple(mdata->filepath);
	}
	else
	{
		MSG_MAIN_HIGH("Launching normal video player");
		ivug_ext_launch_videoplayer(mdata->filepath);
	}

}

static Eina_Bool _ivug_long_press_timer_expired(void *data)
{
	ivug_retv_if(!data, ECORE_CALLBACK_CANCEL);

	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	MSG_SETAS_HIGH("long press timer expired");

	pMainView->popup_timer = NULL;

	if(pMainView->bLongpressEnd == false)	// still press
	{
		if(pMainView->longpress_popup)
		{
			IVUG_DEBUG_WARNING("copy popup remove");
			evas_object_del(pMainView->longpress_popup);
			pMainView->longpress_popup = NULL;
		}
	}


	return ECORE_CALLBACK_CANCEL;
}

static void
_on_slider_long_press_start(void *data, Evas_Object *obj, void *event_info)
{
	Elm_Gesture_Taps_Info *p = (Elm_Gesture_Taps_Info *) event_info;
	ivug_ret_if(!data||!p);

	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	MSG_MAIN_HIGH("LongPressed XY(%d,%d)", p->x, p->y);

	int x = p->x;
	int y = p->y;

	if ( ivug_slider_get_state(pMainView->slider) != SLIDE_STATE_LOADED)
	{
		MSG_MAIN_ERROR("Long pressed. but state is not ready");
		return;
	}

	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);
	if(mdata == NULL)
	{
		MSG_MAIN_ERROR("mdata is NULL");
		return;
	}

	if(mdata->slide_type != SLIDE_TYPE_IMAGE)
	{
		MSG_MAIN_ERROR("Long pressed. but it is not image");
		return;
	}

	pMainView->longpress_popup = ivug_longpress_popup_show(pMainView->slider, x, y, _on_longpress_popup_selected, pMainView);
	if (pMainView->longpress_popup == NULL)
	{
		IVUG_DEBUG_WARNING("long press popup create failed");
		return ;
	}

	pMainView->bLongpressEnd = false;

	if(pMainView->popup_timer)
	{
		ecore_timer_del(pMainView->popup_timer);
	}
	pMainView->popup_timer = ecore_timer_add(LONGTAP_TIMEOUT, _ivug_long_press_timer_expired, pMainView);
}

static void
_on_slider_long_press_end(void *data, Evas_Object *obj, void *event_info)
{
	//Elm_Gesture_Taps_Info *p = (Elm_Gesture_Taps_Info *) event_info;
	//ivug_ret_if(!data||!p);
	ivug_ret_if(!data);

	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	pMainView->bLongpressEnd = true;

	if(pMainView->popup_timer)
	{
		ecore_timer_del(pMainView->popup_timer);
		pMainView->popup_timer = NULL;
	}

	if((int)event_info == LONGTAP_CANCELED)
	{
		if(pMainView->longpress_popup)
		{
			IVUG_DEBUG_WARNING("copy popup remove");
			evas_object_del(pMainView->longpress_popup);
			pMainView->longpress_popup = NULL;
		}
	}
}

static void _on_remove_main_view_ui(Ivug_MainView *pMainView)
{
	IV_ASSERT(pMainView != NULL);

	ivug_main_view_del_hide_timer(pMainView);

	// Slider should be removed before mList is freed.
	if ( pMainView->slider )
	{
		evas_object_del(pMainView->slider);
		pMainView->slider = NULL;
	}

	if ( pMainView->pCropView )
	{
		MSG_MAIN_HIGH("Crop View Destroy");
		ivug_crop_view_destroy(pMainView->pCropView);
		pMainView->pCropView = NULL;
	}

	if ( pMainView->pNameView )
	{
		MSG_MAIN_HIGH("Name View Destroy");
		ivug_name_view_destroy(pMainView->pNameView);
		pMainView->pNameView = NULL;
	}

	if ( pMainView->pSetAsView )
	{
		MSG_MAIN_HIGH("SetAS View Destroy");
		ivug_setas_view_destroy(pMainView->pSetAsView);
		pMainView->pSetAsView = NULL;
	}

	if ( pMainView->pDetailsView )
	{
		MSG_MAIN_HIGH("Details View Destroy");

		ivug_details_view_destroy(pMainView->pDetailsView);
		pMainView->pDetailsView = NULL;
	}


	if(pMainView->layout)		// Remove topmost parent in main view.
	{
		evas_object_del(pMainView->layout);
		pMainView->layout = NULL;
	}

	if(pMainView->empty_layout)
	{
		evas_object_del(pMainView->empty_layout);
		pMainView->empty_layout = NULL;
	}

	if ( pMainView->mList )
	{
		MSG_MAIN_HIGH("Remove media list");
		ivug_medialist_del(pMainView->mList);		// ivug_medialist_del() is not working on destroy cb.
		pMainView->mList = NULL;
	}
}


static void
_on_slider_item_changed(void *data, Evas_Object *obj, void *event_info)
{
	IV_ASSERT(data != NULL);

	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	Slide_Item *sitem = (Slide_Item *)event_info;

	MSG_MAIN_HIGH("Image slider changed callback. ");

	if ( sitem == NULL || sitem->mitem == NULL)
	{
		// There is no item existed. terminate UG
		MSG_MAIN_ERROR("No item is remained. exit UG. Sitem=0x%08x", sitem);
		_on_remove_main_view_ui(pMainView);
		DESTROY_ME();
		return;
	}

	if(pMainView->selectioninfo != NULL)
	{
		MSG_MAIN_MED("Delete small noti");
		evas_object_del(pMainView->selectioninfo);
		pMainView->selectioninfo = NULL;
	}

	if (pMainView->longpress_popup)
	{
		MSG_MAIN_MED("Delete copy popup");
		evas_object_del(pMainView->longpress_popup);
		pMainView->longpress_popup = NULL;
	}

	drm_data_t *drm_data = ivug_slider_item_check_drm_constraint(sitem);
	if(drm_data)
	{
		_ivug_main_view_drm_process(drm_data->constraints, drm_data->data, pMainView);
		free(drm_data);
	}

// Update Main View.
	if ( pMainView->bShowMenu == true )
	{
		PERF_CHECK_BEGIN(LVL5, "Update main view");
		_update_main_view(pMainView);
		PERF_CHECK_END(LVL5, "Update main view");
	}

	Media_Data *mdata = ivug_medialist_get_data(ivug_slider_item_data_get(sitem));

	if(mdata->bIsDRMContent == true && ivug_drm_get_forwarding_info(mdata->filepath) == false)
	{
		elm_object_disabled_set(pMainView->btn_share, EINA_TRUE);
	}

	pMainView->zoom_level = 1;
	elm_slider_value_set(pMainView->zoom_slider, 1.0);

	ivug_slider_zoom_reset(pMainView->slider);
}


static void
_on_slider_item_state_changed(void *data, Evas_Object *obj, void *event_info)
{
// Only called center item state is changed/
	MSG_MAIN_HIGH("Image slider item state changed callback. ");
	ivug_ret_if(!data);

	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	Slide_Item *sitem = (Slide_Item *)event_info;

	if ( sitem == NULL || sitem->mitem == NULL)
	{
		// There is no item existed. terminate UG
		MSG_MAIN_ERROR("No item is remained. exit UG");
		DESTROY_ME();

		return;
	}

	if(sitem->state == SLIDE_STATE_DOWNLOAD_FAILED)
	{
		MSG_MAIN_ERROR("Download failed");
		_ivug_selectioninfo_create(pMainView, IDS_DOWNLOAD_FAILED);
	}

}



static void _on_receive_mouse_down (void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	if (NULL == pMainView) {
		return;
	}

	ivug_main_view_del_hide_timer(pMainView);

	MSG_MAIN_HIGH("Event layer clicked : %s %s Layer=%d",evas_object_name_get(obj), evas_object_type_get(obj), evas_object_layer_get(obj));
}

#ifdef TEST_MOUSE_EVENT
static void _on_receive_mouse_down1 (void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	char *name = (char *)data;

	MSG_MAIN_HIGH("Event layer mouse down : %s %s Layer=%d",evas_object_name_get(obj), name, evas_object_layer_get(obj));
}

static void _on_receive_mouse_up1 (void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	char *name = (char *)data;

	MSG_MAIN_HIGH("Event layer mouse up : %s %s Layer=%d",evas_object_name_get(obj), name, evas_object_layer_get(obj));
}

#endif


static void _on_receive_mouse_up (void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	if (NULL == pMainView) {
		return;
	}

	ivug_main_view_set_hide_timer(pMainView);
}

static void
_on_menu_state_changed(void *data, Evas_Object *obj, const char *emission, const char *source)
{
	Ivug_MainView *pMainView = static_cast<Ivug_MainView *>(data);
	if (NULL == pMainView) {
		return;
	}

	MSG_MAIN_HIGH("Receive %s %s", emission, source);

	if ( strncmp(emission, "menu,show,finished", strlen(emission)) == 0 )
	{
		elm_object_disabled_set(pMainView->ctrl_bar, EINA_FALSE);
	}
	else
	{
		evas_object_hide(pMainView->ctrl_bar);
		elm_object_disabled_set(pMainView->ctrl_bar, EINA_TRUE);
	}
}


static void _win_profile_changed_cb(void *data, Evas_Object * obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	const char *profile = elm_config_profile_get();

	if (!strcmp(profile, "desktop"))
	{  // desktop mode

		// hide naviframe's back key
		elm_object_item_part_content_set(pMainView->navi_it, "prev_btn", NULL);
	}
	else
	{   // mobile

		// show naviframe's back key
		elm_object_item_part_content_set(pMainView->navi_it, "prev_btn", pMainView->back_btn);
	}
}


///////////////////////////////////////////////////////////////////////////////////////////////////
//
// External APIs
//

static char * _ivug_get_folder_name(char *filepath)
{
	char *folder = ecore_file_dir_get(filepath);

	char *name = NULL;

	name = strrchr(folder, '/');
	if ( (name != NULL) && ((name+1) != NULL) )
	{
		return strdup(name + 1);
	}

	return NULL;
}


bool
ivug_main_view_set_list(Ivug_MainView *pMainView, ivug_parameter *ivug_param)
{
	MSG_MAIN_HIGH("Load media list.");

	PERF_CHECK_BEGIN(LVL2, "create media list");
	if (NULL == pMainView) {
		return false;
	}

	Media_List *mlist = ivug_medialist_create();
	if (mlist == NULL)
	{
		MSG_MAIN_ERROR("Creating media list failed");
		return false;
	}
	PERF_CHECK_END(LVL2, "create media list");

	PERF_CHECK_BEGIN(LVL2, "create filter");

	Filter_struct *filter = ivug_param_create_filter(ivug_param);
	if (filter == NULL)
	{
		MSG_MAIN_ERROR("Creating filter failed");
		ivug_medialist_del(mlist);
		return false;
	}

	//do not use ivug_param->view_by after here

	PERF_CHECK_END(LVL2, "create filter");

	Media_Item *current = NULL;
	Media_Data *pData = NULL;


	{
		PERF_CHECK_BEGIN(LVL2, "media list load");
		current = ivug_medialist_load(mlist, filter);
		PERF_CHECK_END(LVL2, "media list load");
	}

	if(current == NULL)
	{
		MSG_MAIN_ERROR("Media list load failed");
		goto LOAD_LIST_FAILED;
	}

	pData = ivug_medialist_get_data(current);
	if(pData == NULL)
	{
		MSG_MAIN_ERROR("current data is NULL");
		goto LOAD_LIST_FAILED;
	}
	if(pData->fileurl == NULL)
	{
		MSG_MAIN_ERROR("current fileurl is NULL");
		goto LOAD_LIST_FAILED;
	}

	if(filter->view_by == IVUG_VIEW_BY_ALL)
	{
		pMainView->album_name = strdup(IDS_ALL_ALBUMS);
	}
	else if(filter->view_by == IVUG_VIEW_BY_FOLDER)
	{
		/*
		media_handle m_handle = ivug_db_get_folder_handle(ecore_file_dir_get(ivug_param->filepath));
		if(m_handle == NULL)
		{
			MSG_IVUG_FATAL("View by Folder. but media handle is NULL" );
			return NULL;
		}

		pMainView->album_name = ivug_db_get_folder_name(m_handle);
		*/
		pMainView->album_name = _ivug_get_folder_name(ivug_param->filepath);
		if(pMainView->album_name == NULL)
		{
			pMainView->album_name = strdup(IDS_NO_NAME);
		}
	}

	pMainView->mList = mlist;
	PERF_CHECK_BEGIN(LVL2, "media list set");

	if(!ivug_slider_set_medialist(pMainView->slider, mlist, current)) {
			MSG_MAIN_ERROR("ivug_slider_set_medialist error");
			goto LOAD_LIST_FAILED;
	}

	PERF_CHECK_END(LVL2, "media list set");

	ivug_data_filter_delete(filter);

	return true;

LOAD_LIST_FAILED:
	if(filter)
		ivug_data_filter_delete(filter);
	if(mlist)
		ivug_medialist_del(mlist);
	pMainView->mList = NULL;
	return false;
}

static void
_on_slideshow_finished(void *data, Evas_Object *obj, void *event_info)
{
	MSG_MAIN_HIGH("_on_slideshow_finished");
	IV_ASSERT(data != NULL);

	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	int ss_state = (int)event_info;
	bool bDestoryed = false;
	bDestoryed = _destory_slideshow_and_ug(pMainView, ss_state, false);
	if (bDestoryed) {
		return;
	}

	Media_Item * item = NULL;

	if(pMainView->ssHandle)
	{

		item = ivug_ss_item_get(pMainView->ssHandle);
		ivug_ss_delete(pMainView->ssHandle);
		pMainView->ssHandle = NULL;

		if(item)
			ivug_slider_set_item(pMainView->slider, item, SLIDE_INSTANTLY);
	}


	ivug_slider_load(pMainView->slider);
	ivug_main_view_show_menu_bar(pMainView);
}

static void
_ivug_main_view_zoom_out_cb(void *data, Evas_Object *obj, const char *emission, const char *source)
{
	ivug_ret_if(!obj);
	ivug_ret_if(!data);

	MSG_MAIN_ERROR("_ivug_main_view_zoom_out_cb");

	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	double d_value = elm_slider_value_get(pMainView->zoom_slider);

	MSG_MAIN_HIGH("d_value = %lf", d_value);

	int value = (int)round(d_value);

	MSG_MAIN_HIGH("value = %d", value);

	if(value > 1)
	{
		ivug_main_view_del_hide_timer(pMainView);

		ivug_slider_zoom_out(pMainView->slider);
		pMainView->zoom_level--;
		elm_slider_value_set(pMainView->zoom_slider, value-1.0);

		ivug_main_view_set_hide_timer(pMainView);
	}

	_show_zoom_icon(pMainView);
}

static void
_ivug_main_view_zoom_in_cb(void *data, Evas_Object *obj, const char *emission, const char *source)
{
	ivug_ret_if(!obj);
	ivug_ret_if(!data);

	MSG_MAIN_ERROR("_ivug_main_view_zoom_in_cb");

	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	double d_value = elm_slider_value_get(pMainView->zoom_slider);

	MSG_MAIN_HIGH("d_value = %lf", d_value);

	int value = (int)round(d_value);

	MSG_MAIN_HIGH("value = %d", value);

	if(value <= ZOOM_LEVEL_MAX)
	{
		ivug_main_view_del_hide_timer(pMainView);

		ivug_slider_zoom_in(pMainView->slider);
		pMainView->zoom_level++;
		elm_slider_value_set(pMainView->zoom_slider, value+1.0);

		ivug_main_view_set_hide_timer(pMainView);
	}

	_show_zoom_icon(pMainView);
}

static void _zoom_slider_change_cb(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	double d_value = elm_slider_value_get(pMainView->zoom_slider);

	MSG_MAIN_HIGH("d_value = %lf", d_value);

	int value = (int)round(d_value);

	MSG_MAIN_HIGH("value = %d", value);

	while(pMainView->zoom_level > 0
		&& pMainView->zoom_level <= ZOOM_LEVEL_MAX+1
		&& value != pMainView->zoom_level)
	{
		if(value > pMainView->zoom_level)
		{
			ivug_slider_zoom_in(pMainView->slider);
			pMainView->zoom_level++;
		}
		else if(value < pMainView->zoom_level)
		{
			ivug_slider_zoom_out(pMainView->slider);
			pMainView->zoom_level--;
		}
	}
}

static void _zoom_slider_start_cb(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	ivug_main_view_del_hide_timer(pMainView);
}

static void _zoom_slider_end_cb(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	ivug_main_view_set_hide_timer(pMainView);
}

Ivug_MainView *
ivug_main_view_create(Evas_Object* parent, ivug_mode mode, ivug_view_by view_by)
{
	IV_ASSERT(parent != NULL);

	MSG_MAIN_HIGH("Creating main view. Parent=0x%08x Mode=%d ViewBy=%d", parent, mode, view_by);

	PERF_CHECK_BEGIN(LVL2, "Create layout");

//create main view layout
	Ivug_MainView *pMainView = (Ivug_MainView *)calloc(1, sizeof(Ivug_MainView) );
	IV_ASSERT(pMainView != NULL);

// Set default value
	pMainView->parent = parent;
	pMainView->mode = mode;
	pMainView->view_by = view_by;

	pMainView->bShowMenu = true;

	pMainView->hide_count = -1;

	Evas_Object *layout = EFL::create_layout(parent, IVUG_MAIN_EDJ, "mainview");
	if ( layout == NULL)	//if failed
	{
		MSG_MAIN_ERROR("main layout create failed");
		free(pMainView);
		return NULL;
	}

	pMainView->layout = layout;
	evas_object_name_set(pMainView->layout, "Main Layout");

	PERF_CHECK_END(LVL2, "Create layout");

	edje_object_signal_callback_add(_EDJ(pMainView->layout),
				"menu,hide,finished",
				"edc",
				_on_menu_state_changed,
				(void *)pMainView);

	edje_object_signal_callback_add(_EDJ(pMainView->layout),
				"menu,show,finished",
				"edc",
				_on_menu_state_changed,
				(void *)pMainView);

// Navigation bar
	PERF_CHECK_BEGIN(LVL2, "elm_naviframe_add");

	pMainView->navi_bar = elm_naviframe_add(layout);
	if ( pMainView->navi_bar == NULL)
	{
		MSG_MAIN_ERROR("navigation bar failed");
		ivug_main_view_destroy(pMainView);
		return NULL;
	}
	evas_object_name_set(pMainView->navi_bar, "Main naviframe");

// Layout life cycle is controlled by application explictily.
	elm_naviframe_content_preserve_on_pop_set(pMainView->navi_bar, EINA_TRUE);

#ifdef USE_CUSTOM_STYLE
	elm_object_theme_set(pMainView->navi_bar, gGetSystemTheme() );
#endif
	const char *profile = elm_config_profile_get();
	if (!strcmp(profile,"mobile"))
	{
		elm_object_style_set(pMainView->navi_bar, "ivug-main/default");
	}
	else if (!strcmp(profile,"desktop"))
	{
		elm_object_style_set(pMainView->navi_bar, "ivug-main/noindicator");
	}
	else
	{
		MSG_MAIN_ERROR("Unknown profile : %s", profile);
	}

	elm_object_part_content_set(layout, "mainview.navibar", pMainView->navi_bar);	//swallow

	PERF_CHECK_END(LVL2, "elm_naviframe_add");

	PERF_CHECK_BEGIN(LVL2, "elm_layout_add");

	pMainView->empty_layout = EFL::create_layout(layout, IVUG_MAIN_EDJ, "navi_content");
	if (pMainView->empty_layout == NULL)
	{
		IVUG_DEBUG_WARNING("layout create failed");
		ivug_main_view_destroy(pMainView);
		return NULL;
	}

	evas_object_name_set(pMainView->empty_layout, "Empty layout");
	PERF_CHECK_END(LVL2,"elm_layout_add");

	if(pMainView->view_by != IVUG_VIEW_BY_FILE && pMainView->view_by != IVUG_VIEW_BY_INVAILD)
	{
		PERF_CHECK_BEGIN(LVL2, "Create thumblist");


		PERF_CHECK_END(LVL2, "Create thumblist");
	}

//create slider
	PERF_CHECK_BEGIN(LVL2, "ivug_slider_add");

	pMainView->slider = ivug_slider_add(pMainView->empty_layout);
	if (pMainView->slider == NULL)
	{
		MSG_MAIN_ERROR("image slider create failed");
		ivug_main_view_destroy(pMainView);
		return NULL;
	}

	elm_object_part_content_set(pMainView->empty_layout, "mainview.slider", pMainView->slider);	//swallow

	PERF_CHECK_END(LVL2, "ivug_slider_add");

	Evas_Object *slider = elm_slider_add(pMainView->empty_layout);
	elm_slider_indicator_format_set(slider, "%1.0f");
	elm_slider_min_max_set(slider, 1, 5);
	elm_slider_value_set(slider, 1);

	evas_object_smart_callback_add(slider, "changed", _zoom_slider_change_cb, pMainView);
	evas_object_smart_callback_add(slider, "slider,drag,start", _zoom_slider_start_cb, pMainView);
	evas_object_smart_callback_add(slider, "slider,drag,stop", _zoom_slider_end_cb, pMainView);

	elm_object_part_content_set(pMainView->empty_layout, "zoom_slider_area", slider);	//swallow

	pMainView->zoom_slider = slider;
	pMainView->zoom_level = 1;

	if (pMainView->mode != IVUG_MODE_DISPLAY)
	{
		PERF_CHECK_BEGIN(LVL2, "ivug_button_add");
		pMainView->back_btn = ivug_button_add(pMainView->navi_bar, "naviframe/end_btn/default", IDS_BACK, NULL, on_btn_back_clicked, pMainView);
		PERF_CHECK_END(LVL2, "ivug_button_add");
	}
	else
	{
		PERF_CHECK_BEGIN(LVL2, "ivug_button_add");
		pMainView->back_btn = ivug_button_add(pMainView->navi_bar, "naviframe/end_btn/default", IDS_BACK, NULL, on_btn_selectcancel_clicked, pMainView);
		PERF_CHECK_END(LVL2, "ivug_button_add");
	}

	PERF_CHECK_BEGIN(LVL2, "elm_naviframe_item_push");
	pMainView->navi_it = elm_naviframe_item_push(pMainView->navi_bar, NULL, pMainView->back_btn, NULL, pMainView->empty_layout, NULL);
	PERF_CHECK_END(LVL2, "elm_naviframe_item_push");

	//Evas_Object *conformant = (Evas_Object *)ug_get_conformant();
	//elm_object_signal_emit(conformant, "elm,state,indicator,overlap", "");

	evas_object_smart_callback_add((Evas_Object *)ug_get_window(), "profile,changed", _win_profile_changed_cb, pMainView);

	_create_main_menu(pMainView);

	PERF_CHECK_BEGIN(LVL2, "add event handler");

//	elm_object_part_content_set(layout, "mainview.slider", pMainView->slider);	//swallow

//register mouse call back.
	evas_object_smart_callback_add(pMainView->slider, "slider,clicked", _on_slider_clicked, pMainView);
	evas_object_smart_callback_add(pMainView->slider, "slider,longpress,start", _on_slider_long_press_start, pMainView);
	evas_object_smart_callback_add(pMainView->slider, "slider,longpress,end", _on_slider_long_press_end, pMainView);
	evas_object_smart_callback_add(pMainView->slider, "slider,clicked,icon", _on_slider_icon_clicked, pMainView);

	evas_object_smart_callback_add(pMainView->slider, "slider,item,changed", _on_slider_item_changed, pMainView);
	evas_object_smart_callback_add(pMainView->slider, "slider,item,state,changed", _on_slider_item_state_changed, pMainView);

	//MSG_MAIN_HIGH("binded main win address is %p\n", main_win);

#ifdef TEST_MOUSE_EVENT
	evas_object_event_callback_add(pMainView->navi_bar, EVAS_CALLBACK_MOUSE_DOWN, _on_receive_mouse_down1, (void *)"navigarion");
	evas_object_event_callback_add(pMainView->layout, EVAS_CALLBACK_MOUSE_DOWN, _on_receive_mouse_down1, (void *)"layout");
	evas_object_event_callback_add(pMainView->slider, EVAS_CALLBACK_MOUSE_DOWN, _on_receive_mouse_down1, (void *)"slider");
	evas_object_event_callback_add(pMainView->slider, EVAS_CALLBACK_MOUSE_UP, _on_receive_mouse_up1, (void *)"slider");
	evas_object_event_callback_add(const_cast<Evas_Object *>(edje_object_part_object_get(_EDJ(pMainView->layout) , "clip")), EVAS_CALLBACK_MOUSE_DOWN, _on_receive_mouse_down1, (void *)"clip");
	evas_object_event_callback_add(const_cast<Evas_Object *>(edje_object_part_object_get(_EDJ(pMainView->layout) , "mainview.navibar")), EVAS_CALLBACK_MOUSE_DOWN, _on_receive_mouse_down1, (void *)"navi swallow");
	evas_object_event_callback_add(pMainView->ctrl_bar, EVAS_CALLBACK_MOUSE_DOWN, _on_receive_mouse_down1, (void *)"controlbar");
#endif

	evas_object_event_callback_add(pMainView->layout, EVAS_CALLBACK_RESIZE, _on_resize, pMainView);

	// zoom icon
	edje_object_signal_callback_add(_EDJ(pMainView->empty_layout),
					"mainview,zoom_out,clicked",
					"",
					_ivug_main_view_zoom_out_cb,
					(void *)pMainView);
	edje_object_signal_callback_add(_EDJ(pMainView->empty_layout),
					"mainview,zoom_in,clicked",
					"",
					_ivug_main_view_zoom_in_cb,
					(void *)pMainView);

	_show_zoom_icon(pMainView);

	pMainView->keydown_handler = ecore_event_handler_add(ECORE_EVENT_KEY_DOWN, _on_key_down, (void *)pMainView);

	{
		int bx, by, bw, bh;
		evas_object_geometry_get(pMainView->navi_bar, &bx, &by, &bw, &bh);

		MSG_MAIN_HIGH("Navibar Created. (%d,%d,%d,%d)", bx, by, bw, bh);
	}

	{
		int bx, by, bw, bh;
		evas_object_geometry_get(pMainView->slider, &bx, &by, &bw, &bh);

		MSG_MAIN_HIGH("Slider Created. (%d,%d,%d,%d)", bx, by, bw, bh);
	}

	{
		int bx, by, bw, bh;
		evas_object_geometry_get(pMainView->empty_layout, &bx, &by, &bw, &bh);

		MSG_MAIN_HIGH("Empty Created. (%d,%d,%d,%d)", bx, by, bw, bh);
	}

	int bx, by, bw, bh;
	evas_object_geometry_get(pMainView->layout, &bx, &by, &bw, &bh);

	MSG_MAIN_HIGH("Main View Created. (%d,%d,%d,%d)", bx, by, bw, bh);

// For debugging.
	DELETE_NOTIFY(pMainView->layout);

	PERF_CHECK_END(LVL2, "add event handler");

	return pMainView;

}

void ivug_main_view_start_slideshow(Ivug_MainView *pMainView, Eina_Bool bSlideFirst)
{
	IV_ASSERT(pMainView != NULL);

	Media_Item *current = NULL;

	// Stop animation & movie play before slideshow is started.
	ivug_slider_pause(pMainView->slider);
	current = ivug_slider_get_item(pMainView->slider);

	ivug_prohibit_lcd_off();
	//ivug_main_view_hide_menu_bar(pMainView);

	pMainView->ssHandle = ivug_ss_create(pMainView->layout);

// Register callback
	evas_object_smart_callback_add(ivug_ss_object_get(pMainView->ssHandle),  "slideshow,finished", _on_slideshow_finished, pMainView);

	ivug_main_view_hide_menu_bar(pMainView);

	ivug_ss_start(pMainView->ssHandle, current, pMainView->mList, bSlideFirst);
}


Evas_Object *
ivug_main_view_object_get(Ivug_MainView *pMainView)
{
	IV_ASSERT(pMainView != NULL);

	return pMainView->layout;
}

void
ivug_main_view_start(Ivug_MainView *pMainView)
{
	IV_ASSERT(pMainView != NULL);


	{
		ivug_main_view_set_hide_timer(pMainView);
	}


	PERF_CHECK_BEGIN(LVL2, "slider start");
	ivug_slider_start(pMainView->slider);
	PERF_CHECK_END(LVL2, "slider start");


	pMainView->area_obj = evas_object_rectangle_add(evas_object_evas_get(pMainView->layout));
	evas_object_size_hint_expand_set(pMainView->area_obj, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_fill_set(pMainView->area_obj, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_color_set(pMainView->area_obj, 0,0,0,0);
	elm_object_part_content_set(pMainView->layout, "mainview.area", pMainView->area_obj);

//	evas_object_show(pMainView->area_obj);
}


void
ivug_main_view_resume(Ivug_MainView *pMainView)
{
	IV_ASSERT(pMainView != NULL);
	if (pMainView->slider == NULL) {
		MSG_MAIN_HIGH("slider is NULL");
		return;
	}
	MSG_MAIN_HIGH("Main View Update");

	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);

	IV_ASSERT(mdata != NULL);

	if (mdata->slide_type == SLIDE_TYPE_IMAGE || mdata->slide_type == SLIDE_TYPE_VIDEO )
	{
		if(mdata->filepath && ecore_file_exists(mdata->filepath)) // Current file is not deleted
		{
			MSG_MAIN_HIGH("Current filepath : %s", mdata->filepath);
		}
		else
		{
			/*if file is not exist ug terminated temporally */
			MSG_MAIN_ERROR("Center file is not exist. stype=%d name=%s", mdata->slide_type, mdata->filepath);
			DESTROY_ME();
			return ;
		}
	}
	else
	{
		MSG_MAIN_ERROR("Unhandled slide type : %d", mdata->slide_type);
	}

// When resume, menu bar will be appeared.
	if(!(pMainView->pNameView
		|| pMainView->pSetAsView
		|| pMainView->pCropView))	//child view is not exist
	{
		ivug_main_view_show_menu_bar(pMainView);
	}

	ivug_slider_resume(pMainView->slider);
}

void
ivug_main_view_pause(Ivug_MainView *pMainView)
{
	IV_ASSERT(pMainView != NULL);

	if ( pMainView->ssHandle )
	{
		ivug_ss_stop(pMainView->ssHandle);
	}
	if (pMainView->slider) {
		ivug_slider_pause(pMainView->slider);
	}

	if(pMainView->bShowMenu == true)
	{
		ivug_main_view_del_hide_timer(pMainView);
	}
	// Stop photocam If AGIF
}


void
ivug_main_view_destroy(Ivug_MainView *pMainView)
{
	IV_ASSERT(pMainView != NULL);

	MSG_MAIN_HIGH("ENTER : Main View Destroy. pMainView=0x%08x", pMainView);

	ivug_main_view_del_hide_timer(pMainView);

	if (pMainView->popup_timer)
	{
		ecore_timer_del(pMainView->popup_timer);
		pMainView->popup_timer = NULL;
	}

	if (pMainView->popup)
	{
		evas_object_del(pMainView->popup);
		pMainView->popup = NULL;
	}

	if(pMainView->ctx_popup)
	{
		evas_object_del(pMainView->ctx_popup);
		pMainView->ctx_popup = NULL;
	}

	if (pMainView->ext_ug)
	{
		MSG_MAIN_HIGH("External UG Destroy");
		ug_destroy(pMainView->ext_ug);
		pMainView->ext_ug = NULL;
	}

	if (pMainView->exit_timer)
	{
		ecore_timer_del(pMainView->exit_timer);
		pMainView->exit_timer = NULL;
	}

	if ( pMainView->zoom_slider )
	{
		evas_object_del(pMainView->zoom_slider);
		pMainView->zoom_slider = NULL;
	}

	MSG_MAIN_HIGH("Unregister system notifications");

	if(pMainView->keydown_handler)
	{
		ecore_event_handler_del(pMainView->keydown_handler);
		pMainView->keydown_handler = NULL;
	}

	if(pMainView->album_name)
	{
		free(pMainView->album_name);
	}

	if(pMainView->area_obj)
	{
		evas_object_del(pMainView->area_obj);
		pMainView->area_obj = NULL;
	}

// Who remove medialist?????
	_on_remove_main_view_ui(pMainView);

	EFL::dump_obj(pMainView->parent, 0);
	free(pMainView);

	MSG_MAIN_HIGH("LEAVE : Main View Destroy.");

	return ;
}

void ivug_main_view_set_hide_timer(Ivug_MainView *pMainView)
{
	IV_ASSERT(pMainView != NULL);

	return;
}

void ivug_main_view_del_hide_timer(Ivug_MainView *pMainView)
{
	IV_ASSERT(pMainView != NULL);

	return;
}

void
ivug_main_view_show_menu_bar(Ivug_MainView *pMainView)
{
	MSG_MAIN_HIGH("Show Menu");
	if (NULL == pMainView) {
		return;
	}

	pMainView->bShowMenu = true;
	evas_object_show(pMainView->ctrl_bar);

	_update_main_view(pMainView);

#ifdef HIDE_INDICATOR
	elm_win_indicator_mode_set(ug_get_window(), ELM_WIN_INDICATOR_HIDE);
#else
	if(gIsDesktopMode() == false)
	{
		elm_win_indicator_mode_set((Evas_Object *)ug_get_window(), ELM_WIN_INDICATOR_SHOW);
	}
#endif
//	edje_object_signal_emit(_EDJ(pMainView->layout), "mainview,show,menu", "elm");

	edje_object_signal_emit(_EDJ(pMainView->empty_layout), "elm,state,show", "user");

	elm_naviframe_item_title_visible_set(pMainView->navi_it, EINA_TRUE);
	elm_object_item_signal_emit(pMainView->navi_it, "elm,state,toolbar,open", "");

	ivug_slider_set_menu_visibility(pMainView->slider, EINA_TRUE);

#ifdef TEST_MOUSE_EVENT
	Eina_List *l = const_cast<Eina_List *>(evas_object_clipees_get(pMainView->navi_bar));

	MSG_MAIN_HIGH("Clipper clips %i objects", eina_list_count(l));

	void *obj;
	Eina_List *l1;

	EINA_LIST_FOREACH(l, l1, obj)
	{
		MSG_MAIN_HIGH("Object Type : %s", evas_object_name_get((Evas_Object *)obj));
	}

	Evas_Object *above;
	Evas_Object *below;

	above = evas_object_above_get(pMainView->navi_bar);
	MSG_MAIN_HIGH("Navi Above : 0x%08x %s %s", above, evas_object_name_get(above), evas_object_type_get(above));

	below = evas_object_below_get(pMainView->navi_bar);
	MSG_MAIN_HIGH("Navi Below : 0x%08x %s %s", below, evas_object_name_get(below), evas_object_type_get(below));

	above = evas_object_above_get(pMainView->slider);
	MSG_MAIN_HIGH("Slider Above : 0x%08x %s %s", above, evas_object_name_get(above), evas_object_type_get(above));

	below = evas_object_below_get(pMainView->slider);
	MSG_MAIN_HIGH("Slider Below : 0x%08x %s %s", below, evas_object_name_get(below), evas_object_type_get(below));
#endif

	ivug_main_view_set_hide_timer(pMainView);

}

void
ivug_main_view_hide_menu_bar(Ivug_MainView *pMainView)
{
	MSG_MAIN_HIGH("Hide Menu");
	if (NULL == pMainView) {
		return;
	}

	ivug_main_view_del_hide_timer(pMainView);

	pMainView->bShowMenu = false;

	elm_object_disabled_set(pMainView->ctrl_bar, EINA_TRUE);

	edje_object_signal_emit(_EDJ(pMainView->empty_layout), "elm,state,hide", "user");

	elm_win_indicator_mode_set((Evas_Object *)ug_get_window(), ELM_WIN_INDICATOR_HIDE);

//	edje_object_signal_emit(_EDJ(pMainView->layout), "mainview,hide,menu","elm");
	elm_naviframe_item_title_visible_set(pMainView->navi_it, EINA_FALSE);
	elm_object_item_signal_emit(pMainView->navi_it, "elm,state,toolbar,close", "");

	// zoom icon
	_hide_zoom_icon(pMainView);

	ivug_slider_set_menu_visibility(pMainView->slider, EINA_FALSE);

}

