/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Ecore_X.h>
#include "ivug-common.h"

#include "ivug-name-view.h"
#include "ivug-popup.h"

#include "ivug-media.h"

#include "ivug-util.h"

static void
_on_timeout_response(void *data, Evas_Object *obj, void *event_info)
{
	IV_ASSERT(data != NULL);

	Ivug_NameView *pNameView = (Ivug_NameView *)data;

	if ( pNameView->fnresponse )
	{
		(pNameView->fnresponse)(NAME_VIEW_RESPONSE_CANCEL, NULL, pNameView->clientdata);
	}

	//destroy after pop and transit ended
	//ivug_name_view_destroy(pNameView);
}

static void _on_resized(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	IV_ASSERT(data != NULL);

	Ivug_NameView *pNameView = (Ivug_NameView *)data;

	Elm_Object_Item *navi_it = elm_naviframe_top_item_get(pNameView->navibar);

	if(pNameView->sip_show == true)
	{
		int rot = gGetRotationDegree();
		if(rot == 90 || rot == 270)
		{
			elm_naviframe_item_title_visible_set(navi_it, EINA_FALSE);
		}
		else
		{
			elm_naviframe_item_title_visible_set(navi_it, EINA_TRUE);
		}
	}
	else
	{
		elm_naviframe_item_title_visible_set(navi_it, EINA_TRUE);
	}
}

static void
_on_keypad_show(void *data, Evas_Object *obj, void *event_info)
{
	IV_ASSERT(data != NULL);

	Ivug_NameView *pNameView = (Ivug_NameView *)data;

	pNameView->sip_show = true;

	Elm_Object_Item *navi_it = elm_naviframe_top_item_get(pNameView->navibar);

	int rot = gGetRotationDegree();
	if(rot == 90 || rot == 270)
	{
		elm_naviframe_item_title_visible_set(navi_it, EINA_FALSE);
	}
}

static void
_on_keypad_hide(void *data, Evas_Object *obj, void *event_info)
{
	IV_ASSERT(data != NULL);

	Ivug_NameView *pNameView = (Ivug_NameView *)data;

	pNameView->sip_show = false;

	Elm_Object_Item *navi_it = elm_naviframe_top_item_get(pNameView->navibar);

	elm_naviframe_item_title_visible_set(navi_it, EINA_TRUE);
}


static void
_on_invalid_name_timeout_response(void *data, Evas_Object *obj, void *event_info)
{
	IV_ASSERT(data != NULL);

	Ivug_NameView *pNameView = (Ivug_NameView *)data;

	ecore_imf_context_input_panel_show(elm_entry_imf_context_get(pNameView->entry));
}

static void
_ivug_rename_view_enter_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	MSG_IMAGEVIEW_HIGH("inside ...");
	Ivug_NameView *pNameView = (Ivug_NameView *)data;
	IV_ASSERT(pNameView != NULL);

	Evas_Object *entry = elm_object_part_content_get(pNameView->editfield, "elm.icon.entry");

	char *name = elm_entry_markup_to_utf8 ( elm_entry_entry_get(entry) );

	MSG_IMAGEVIEW_HIGH("name = %s", name);

	if(name)
	{
		char *new_name = NULL;
		new_name = ivug_strip_string(name);
		if(new_name == NULL)
		{
			MSG_IMAGEVIEW_ERROR( "ivug_strip_string failed");
			ecore_imf_context_input_panel_hide(elm_entry_imf_context_get(entry));
			ivug_timeout_popup_show(pNameView->layout, _on_timeout_response, pNameView, IDS_ERROR, IDS_ENTRY_IS_EMPTY);
			free (name);
			return;
		}

		if ( pNameView->fnresponse )
		{
			(pNameView->fnresponse)(NAME_VIEW_RESPONSE_OK, new_name, pNameView->clientdata);
		}

		free(name);

		//destroy after pop and transit ended
		//ivug_name_view_destroy(pNameView);
	}
	else
	{
		ecore_imf_context_input_panel_hide(elm_entry_imf_context_get(entry));
	}
}

void
_ivug_name_view_maxlength_reached(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_NameView *pNameView = (Ivug_NameView *)data;
	IV_ASSERT(pNameView != NULL);

	//todo : show popup

}

static void
_ivug_name_view_done_cb(void *data, Evas_Object* obj, void* event_info )
{
	MSG_IMAGEVIEW_HIGH( "pNameView Done ");
	Ivug_NameView *pNameView = (Ivug_NameView *)data;
	IV_ASSERT(pNameView != NULL);

	Evas_Object *entry = elm_object_part_content_get(pNameView->editfield, "elm.icon.entry");

	char *name = elm_entry_markup_to_utf8 ( elm_entry_entry_get(entry) );

	MSG_IMAGEVIEW_HIGH("name = %s", name);

	if(name)
	{
		char *new_name = NULL;
		new_name = ivug_strip_string(name);
		if(new_name == NULL)
		{
			MSG_IMAGEVIEW_ERROR( "ivug_strip_string failed");
			ecore_imf_context_input_panel_hide(elm_entry_imf_context_get(entry));
			ivug_timeout_popup_show(pNameView->layout, _on_timeout_response, pNameView, IDS_ERROR, IDS_ENTRY_IS_EMPTY);
			free (name);
			return;
		}

		if ( pNameView->fnresponse )
		{
			(pNameView->fnresponse)(NAME_VIEW_RESPONSE_OK, new_name, pNameView->clientdata);
		}

		free(name);
	}

	//destroy after pop and transit ended
	//ivug_name_view_destroy(pNameView);
}

static void
_ivug_name_view_cancel_cb(void *data, Evas_Object *obj, void *event_info)
{
	MSG_IMAGEVIEW_HIGH("pNameView Back");
	Ivug_NameView *pNameView = (Ivug_NameView *)data;
	IV_ASSERT(pNameView != NULL);

	if(pNameView->sip_show == true)
	{
		ecore_imf_context_input_panel_hide(elm_entry_imf_context_get(pNameView->entry));
	}
	else
	{
	if ( pNameView->fnresponse )
	{
		(pNameView->fnresponse)(NAME_VIEW_RESPONSE_CANCEL, NULL, pNameView->clientdata);
	}

	//destroy after pop and transit ended
	//ivug_name_view_destroy(pNameView);
}
}

static void _ivug_name_view_on_entry_changed(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_NameView *pNameView = (Ivug_NameView *)data;
	IV_ASSERT(pNameView != NULL);

	if (elm_object_focus_get(obj))
	{
		if (elm_entry_is_empty(obj))
			elm_object_signal_emit(pNameView->editfield, "elm,state,eraser,hide", "");
		else
			elm_object_signal_emit(pNameView->editfield, "elm,state,eraser,show", "");
	}

	Evas_Object *entry;

	entry = elm_object_part_content_get(pNameView->editfield, "elm.icon.entry");

	char *content = elm_entry_markup_to_utf8 ( elm_entry_entry_get(entry) );

	if (strlen(content) == 0)
	{
		MSG_IMAGEVIEW_HIGH("ISF1 : %s", content);
		elm_object_part_text_set(pNameView->entry, "elm.guide", IDS_ENTER_NAME);
		//elm_object_disabled_set(pNameView->btn_done, EINA_TRUE);
	}
	else if(ivug_validate_file_name(content) == false)
	{
		MSG_IMAGEVIEW_HIGH("invalid char ISF : %s", content);
		char *new_str = strdup(content);
		if (new_str) {
			if (strlen(content) > 0)
				new_str[strlen(content) - 1] = '\0';
			elm_entry_entry_set(pNameView->entry, new_str);
			elm_entry_cursor_end_set(pNameView->entry);
			free(new_str);
			new_str = NULL;
		}
		ecore_imf_context_input_panel_hide(elm_entry_imf_context_get(pNameView->entry));
		ivug_timeout_popup_show(pNameView->layout, _on_invalid_name_timeout_response,
			pNameView, NULL, IDS_INVALID_NAME);
	}
	else
	{
		MSG_IMAGEVIEW_HIGH("ISF : %s", content);
		elm_object_part_text_set(pNameView->entry, "elm.guide", IDS_NULL);
		elm_object_disabled_set(pNameView->btn_done, EINA_FALSE);
	}

	free(content );

}

static void _ivug_name_view_on_entry_changed_user(void *data, Evas_Object *obj, void *event_info)
{
	MSG_IMAGEVIEW_HIGH("Entry changed by user");
}

static void _ivug_name_view_on_entry_preedit_changed(void *data, Evas_Object *obj, void *event_info)
{
	MSG_IMAGEVIEW_HIGH("_ivug_name_view_on_entry_preedit_changed");

	_ivug_name_view_on_entry_changed(data, obj, event_info);
}

static void _ivug_name_view_on_entry_focused(void *data, Evas_Object *obj, void *event_info)
{
	MSG_IMAGEVIEW_HIGH("_ivug_name_view_on_entry_focused");
	if (!elm_entry_is_empty(obj))
		elm_object_signal_emit(data, "elm,state,eraser,show", "");
	elm_object_signal_emit(data, "elm,state,rename,hide", "");
}

static void _ivug_name_view_on_entry_unfocused(void *data, Evas_Object *obj, void *event_info)
{
	MSG_IMAGEVIEW_HIGH("_ivug_name_view_on_entry_unfocused");

	elm_object_signal_emit(data, "elm,state,eraser,hide", "");
	elm_object_signal_emit(data, "elm,state,rename,show", "");
}


static void
_ivug_name_view_eraser_clicked_cb(void *data, Evas_Object *obj, const char *emission, const char *source)
{
	elm_object_focus_set(data, EINA_TRUE); // After button is clicked, entry should get focus again.
	elm_entry_entry_set(data, "");
}

static Evas_Object *_ivug_name_view_editfield_create(void *data, Evas_Object *obj)
{
	Ivug_NameView *pNameView = (Ivug_NameView *)data;

	Evas_Object *editfield = elm_layout_add(obj);
	elm_layout_theme_set(editfield, "layout", "item/editfield", "default");

	Evas_Object *entry = elm_entry_add(editfield);
	elm_entry_single_line_set(entry, EINA_TRUE);
	elm_entry_scrollable_set(entry, EINA_TRUE);
	elm_object_part_text_set(entry, "elm.guide", IDS_ENTER_NAME);

	evas_object_smart_callback_add(entry, "activated", _ivug_rename_view_enter_click_cb, pNameView);
	evas_object_smart_callback_add(entry, "changed", _ivug_name_view_on_entry_changed, pNameView);
	evas_object_smart_callback_add(entry, "preedit,changed", _ivug_name_view_on_entry_preedit_changed, pNameView);
	evas_object_smart_callback_add(entry, "changed,user", _ivug_name_view_on_entry_changed_user, pNameView);
	evas_object_smart_callback_add(entry, "focused", _ivug_name_view_on_entry_focused, editfield);
	evas_object_smart_callback_add(entry, "unfocused", _ivug_name_view_on_entry_unfocused, editfield);

	elm_object_part_content_set(editfield, "elm.icon.entry", entry);

	//MSG_IMAGEVIEW_HIGH("entry = %s", elm_entry_entry_get(entry));

	Evas_Object *button = elm_button_add(editfield);
	elm_object_style_set(button, "editfield_clear");   // Make "X" marked button by changing style.
	elm_object_part_content_set(editfield, "elm.icon.eraser", button); // Add eraser button to current editfield layout.
	evas_object_smart_callback_add(button, "clicked", _ivug_name_view_eraser_clicked_cb, entry);

	pNameView->entry = entry;

	pNameView->editfield = editfield;

	return pNameView->editfield;

}

static Evas_Object *_ivug_name_view_create_content(Ivug_NameView *pNameView, Evas_Object *parent)
{
	ivug_retvm_if(!pNameView, NULL, "pNameView is NULL");
	ivug_retvm_if(!parent, NULL, "parent is NULL");

	Eina_Bool ret = EINA_FALSE;

	Evas_Object *content_lay = elm_layout_add(parent);
	evas_object_size_hint_weight_set(content_lay, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_fill_set(content_lay, EVAS_HINT_FILL, EVAS_HINT_FILL );
	ret = elm_layout_file_set(content_lay, EDJ_PATH"/ivug-name-view.edj", "name_view");
	MSG_IMAGEVIEW_HIGH("ret is %d", ret);
	ivug_retvm_if(ret == EINA_FALSE, NULL, "elm_layout_file_set ailed");

	Evas_Object *editfield = _ivug_name_view_editfield_create(pNameView, content_lay);
	elm_object_part_content_set(content_lay, "entry_part", editfield);

	evas_object_show(editfield);

	return content_lay;
}

Ivug_NameView *
ivug_name_view_create(Evas_Object *parent, const char *title)
{
	MSG_IMAGEVIEW_HIGH( "... ");

	IV_ASSERT(parent != NULL);

	Ivug_NameView *pNameView = calloc(1, sizeof(Ivug_NameView));
	if ( pNameView == NULL)
	{
		MSG_IMAGEVIEW_ERROR("Cannot allocated memory");
		return NULL;
	}

	MSG_IMAGEVIEW_HIGH( "Add Name View creation");

	pNameView->layout = ivug_default_layout_add(parent);

	if ( pNameView->layout == NULL)
	{
		MSG_IMAGEVIEW_ERROR("Cannot create layout");
		free(pNameView);
		return NULL;
	}

	pNameView->bg = ivug_bg_add(pNameView->layout, IVUG_DEFAULT_BG_COLOR, IVUG_DEFAULT_BG_COLOR, IVUG_DEFAULT_BG_COLOR);

	if ( pNameView->bg == NULL)
	{
		MSG_IMAGEVIEW_ERROR("Cannot create bg");
		free(pNameView);
		return NULL;
	}

	elm_object_part_content_set(pNameView->layout, "elm.swallow.bg", pNameView->bg);

	Evas_Object *navi_bar;
	navi_bar = elm_naviframe_add(pNameView->layout);

	if ( navi_bar == NULL)
	{
		MSG_IMAGEVIEW_ERROR("Cannot create navigation bar");
		free(pNameView);
		return NULL;
	}

	//using default naviframe

	elm_object_part_content_set(pNameView->layout, "elm.swallow.content", navi_bar );
	evas_object_show(navi_bar);

	pNameView->navibar = navi_bar;

	pNameView->content = _ivug_name_view_create_content(pNameView, navi_bar);
	if(pNameView->content == NULL)
	{
		MSG_IMAGEVIEW_ERROR("Cannot create navigation bar");
		free(pNameView);
		return NULL;
	}

	Evas_Object *cancel_button, *done_button;

	cancel_button = ivug_button_add(navi_bar, "naviframe/end_btn/default", NULL, NULL, _ivug_name_view_cancel_cb, pNameView );

	done_button = ivug_button_add(navi_bar, "naviframe/toolbar/default", IDS_DONE, NULL, _ivug_name_view_done_cb, pNameView );

	pNameView->btn_done = done_button;

	//elm_object_disabled_set(done_button, EINA_TRUE);

	Elm_Object_Item *navi_it = NULL;
	navi_it = elm_naviframe_item_push(pNameView->navibar, title, cancel_button, NULL, pNameView->content, NULL);

	Evas_Object *conformant = ug_get_conformant();

	evas_object_smart_callback_add(conformant, "virtualkeypad,state,on", _on_keypad_show, pNameView);
	evas_object_smart_callback_add(conformant, "virtualkeypad,state,off", _on_keypad_hide, pNameView);
	evas_object_smart_callback_add(conformant, "clipboard,state,on", _on_keypad_show, pNameView);
	evas_object_smart_callback_add(conformant, "clipboard,state,off", _on_keypad_hide, pNameView);

	evas_object_event_callback_add(pNameView->content, EVAS_CALLBACK_RESIZE, _on_resized, (void *)pNameView);

	elm_object_signal_emit(conformant, "elm,state,indicator,nooverlap", "");

	//ivug_name_view_set_focus(pNameView);

	elm_object_item_part_content_set(navi_it, "toolbar_button1", done_button);
	//elm_object_item_part_content_set(navi_it, "title_toolbar_button1", done_button);
	//elm_object_item_part_content_set(navi_it, "title_prev_btn", cancel_button);
	//elm_object_item_signal_emit(navi_it, "elm,state,sip,shown", "");

	return pNameView;
}


void
ivug_name_view_destroy(Ivug_NameView *pNameView)
{
	ivug_ret_if(pNameView == NULL);

	MSG_IMAGEVIEW_HIGH( "Destroy Name View");

	if(pNameView->entry)
	{
		evas_object_del(pNameView->entry);
		pNameView->entry = NULL;
	}

	if(pNameView->editfield)
	{
		evas_object_del(pNameView->editfield);
		pNameView->editfield = NULL;
	}

	if(pNameView->content)
	{
		evas_object_del(pNameView->content);
		pNameView->content = NULL;
	}

	if(pNameView->navibar)
	{
		evas_object_del(pNameView->navibar);
		pNameView->navibar = NULL;
	}

	if(pNameView->layout)
	{
		evas_object_del(pNameView->layout);
		pNameView->layout = NULL;
	}

	Evas_Object *conformant = ug_get_conformant();

	evas_object_smart_callback_del(conformant, "virtualkeypad,state,on", _on_keypad_show);
	evas_object_smart_callback_del(conformant, "virtualkeypad,state,off", _on_keypad_hide);
	evas_object_smart_callback_del(conformant, "clipboard,state,on", _on_keypad_show);
	evas_object_smart_callback_del(conformant, "clipboard,state,off", _on_keypad_hide);

	free(pNameView);
	pNameView = NULL;

	MSG_IMAGEVIEW_HIGH("Name view removed");
}


Evas_Object *
ivug_name_view_object_get(Ivug_NameView *pNameView)
{
	IV_ASSERT(pNameView != NULL);

	return pNameView->layout;

}

void
ivug_name_view_set_response_callback(Ivug_NameView *pNameView, FNResponse resp, void *data)
{
	IV_ASSERT(pNameView != NULL);

	if ( pNameView->fnresponse != NULL )
	{
		MSG_IMAGEVIEW_WARN("Old response callback is overwritten");
	}

	pNameView->fnresponse = resp;
	pNameView->clientdata = data;

}

void
ivug_name_view_set_title(Ivug_NameView *pNameView, const char *title)
{
	IV_ASSERT(pNameView!=NULL);

	Elm_Object_Item *navi_it;

	navi_it = elm_naviframe_top_item_get(pNameView->navibar);

	elm_object_item_part_text_set(navi_it, "elm.text.title", title);
}

void
ivug_name_view_set_entry(Ivug_NameView *pNameView, const char *str)
{
	IV_ASSERT(pNameView!=NULL);

	elm_entry_entry_set(pNameView->entry, str);

	elm_entry_cursor_end_set(pNameView->entry);
}

void
ivug_name_view_set_focus(Ivug_NameView *pNameView)
{
	elm_entry_cursor_end_set(pNameView->entry);
	evas_object_show(pNameView->entry);
	elm_object_focus_set(pNameView->entry, EINA_TRUE);	// show keypad
}

void
ivug_name_view_set_max_length(Ivug_NameView *pNameView, int max_len)
{
	IV_ASSERT(pNameView!=NULL);

	MSG_IMAGEVIEW_HIGH("ivug_name_view_set_max_length to %d", max_len);

	Evas_Object *entry = pNameView->entry;

	static Elm_Entry_Filter_Limit_Size limit_filter_data;
	limit_filter_data.max_char_count = 0;
	limit_filter_data.max_byte_count = max_len;
	elm_entry_markup_filter_append(entry, elm_entry_filter_limit_size, &limit_filter_data);

	evas_object_smart_callback_add(entry, "maxlength,reached", _ivug_name_view_maxlength_reached, (void *) pNameView);
}

