/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <Elementary.h>

#include "ivug-define.h"

#include "ivug-setas-view-callerid.h"
#include "ivug-setas-view.h"

#include "ivug-selectioninfo.h"

#include "ivug-image.h"
#include "ivug-drm-common.h"
#include "EFLUtil.h"
#include "ivug-scissorbox.h"

#include "ivug-db.h"

#define EDJ_PATH PREFIX"/res/edje/"PACKAGE

#define SETAS_EDJ_FILE EDJ_PATH"/ivug-setas.edj"

#define CLICK_TIME_MAX		(500)				// click time.
#define CLICK_WIDTH_MIN	(elm_config_finger_size_get() >> 1)	//click width, if mouse move under this value and under click time, it is click.
#define CLICK_INTERVAL_TIME 0.15

/* it is temp define, because scroll, scroll,anim,end is called earlyer.
   So find bound of scissor box again after SETAS_INTERVAL_TIME */
#define SETAS_INTERVAL_TIME 0.1
#define SETAS_INTERVAL_TIME_FIRST_LOAD 0.3

#define CROP_BOX_INTERVAL_TIME 0.3

typedef struct {
	Evas_Object *layout;
	Evas_Object *notify;
	Evas_Object *notify_area;
	Evas_Object *photocam;

	Evas_Object *contents_area;

	Evas_Object *gesture;

	Evas_Object *btn_ok;

	bool bShowMenu;		// Current state for button's visibility

	char *filepath;
	char *drm_filepath;

	Evas_Coord_Rectangle rect;

	int prev_x;
	int prev_y;
	int prev_w;
	int prev_h;

	struct {
		Evas_Object *box;
		double ratio;

		Eina_Bool bAspectPreserve;
	} crop;

	Ecore_Timer *box_timer;

} TSetAsData;

#define SETAS_DATA(obj) \
		(TSetAsData *)evas_object_data_get((Evas_Object *)(obj), "pSetAsData")

static void _on_moved(void *data, Evas *e, Evas_Object *obj, void *event_info);
static void _on_resized(void *data, Evas *e, Evas_Object *obj, void *event_info);

static void _put_scissorbox_default(TSetAsData *pSetAsData)
{
	Evas_Object *image = ivug_image_internal_image_get(pSetAsData->photocam);

	int image_w = 0;
	int image_h = 0;
	double ratio = pSetAsData->crop.ratio;
	Evas_Coord_Rectangle rect = {0,};

	ivug_image_image_size_get(pSetAsData->photocam, &image_w, &image_h);	// Get original image size.

	/*int degree = ivug_image_rotate_get(pSetAsData->photocam);
	if(degree == 90 || degree == 270)	//rotated
	{
		int temp = image_w;
		image_w = image_h;
		image_h = temp;
	}*/

	int lcd_x, lcd_y, lcd_w, lcd_h;

	evas_object_geometry_get(pSetAsData->contents_area, &lcd_x, &lcd_y, &lcd_w, &lcd_h);
	MSG_SETAS_HIGH("lcd_x=%d, lcd_y=%d, lcd_w=%d, lcd_h=%d", lcd_x, lcd_y, lcd_w, lcd_h);

	int px, py, pw, ph;
	ivug_image_region_get(pSetAsData->photocam, &px, &py, &pw, &ph);

	px = (lcd_w - pw) / 2;	// do not use previous px, px is invalid because transition effect

	MSG_SETAS_WARN("region XYWH(%d,%d,%d,%d)",	px, py, pw, ph);

	int ix, iy, iw, ih;

	if ( image_w > image_h )	// Landscape image
	{
		if(ratio > 1.0) // height is larger
		{
			iw = image_h / (2 * ratio);
			ih = image_h / 2;
		}
		else
		{
			iw = image_h / 2;
			ih = (image_h * ratio) / 2;
		}
	}
	else
	{
		if(ratio > 1.0) // height is larger
		{
			iw = image_w / (2 * ratio);
			ih = image_w / 2;
		}
		else
		{
			iw = image_w / 2;
			ih = (image_w * ratio) / 2;
		}
	}

	ix = (image_w - iw)/2;
	iy = (image_h - ih)/2;


	rect.x = (double)ix * pw / image_w + px;
	rect.y = (double)iy * ph / image_h + py;

	rect.w = (double)iw * pw / image_w;
	rect.h = (double)ih * ph / image_h;

	MSG_SETAS_WARN("Zoom Put Box in LCD XYWH(%d,%d,%d,%d)",
		rect.x, rect.y, rect.w, rect.h);

	ivug_scissorbox_region_set(pSetAsData->crop.box, rect.x, rect.y, rect.w, rect.h);

	ivug_scissorbox_attach(pSetAsData->crop.box, image);		// Attach to photocam

	ivug_scissorbox_boundary_set(pSetAsData->crop.box, px, py, pw, ph);

	pSetAsData->prev_x = px;	//lcd_x is wrong because transition effect
	pSetAsData->prev_y = py;
	pSetAsData->prev_w = pw;
	pSetAsData->prev_h = ph;
}

static Eina_Bool _put_scissorbox_timer_cb(void *data)
{
	TSetAsData *pSetAsData = SETAS_DATA(data);

	pSetAsData->box_timer = NULL;

	_put_scissorbox_default(pSetAsData);

	elm_object_disabled_set(pSetAsData->btn_ok, EINA_FALSE);

	return ECORE_CALLBACK_CANCEL;
}


static void
_on_btn_ok_clicked(void *data, Evas_Object *obj, void *event_info)
{
	TSetAsData *pSetAsData = SETAS_DATA(data);
	IV_ASSERT(pSetAsData != NULL);

	MSG_SETAS_HIGH("OK button clicked");
	evas_object_smart_callback_call(pSetAsData->layout, "ok,clicked", NULL);

	evas_object_event_callback_del_full(pSetAsData->layout, EVAS_CALLBACK_MOVE, _on_moved, pSetAsData->layout);
	evas_object_event_callback_del_full(pSetAsData->layout, EVAS_CALLBACK_RESIZE, _on_resized, pSetAsData->layout);

	if ( pSetAsData->crop.box )
	{
		evas_object_del(pSetAsData->crop.box);
		pSetAsData->crop.box = NULL;
	}

}

static void
_on_photocam_loaded(void *data, Evas_Object *obj, void *event_info)
{
	TSetAsData *pSetAsData = SETAS_DATA(data);
	MSG_SETAS_HIGH("Photocam preloaded");

#if 0
	typedef enum _Evas_Load_Error
	{
	   EVAS_LOAD_ERROR_NONE = 0, /**< No error on load */
	   EVAS_LOAD_ERROR_GENERIC = 1, /**< A non-specific error occurred */
	   EVAS_LOAD_ERROR_DOES_NOT_EXIST = 2, /**< File (or file path) does not exist */
	   EVAS_LOAD_ERROR_PERMISSION_DENIED = 3, /**< Permission deinied to an existing file (or path) */
	   EVAS_LOAD_ERROR_RESOURCE_ALLOCATION_FAILED = 4, /**< Allocation of resources failure prevented load */
	   EVAS_LOAD_ERROR_CORRUPT_FILE = 5, /**< File corrupt (but was detected as a known format) */
	   EVAS_LOAD_ERROR_UNKNOWN_FORMAT = 6 /**< File is not a known format */
	} Evas_Load_Error; /**< Load error you can get from loading of files - see evas_load_error_str() too */
#endif

	Evas_Load_Error error = static_cast<Evas_Load_Error>((int)event_info);

	if ( error != EVAS_LOAD_ERROR_NONE )
	{
		MSG_SETAS_ERROR("Error occured during decoding. Error=%d", error);

		if ( error == EVAS_LOAD_ERROR_UNKNOWN_FORMAT )
		{
			evas_object_smart_callback_call(pSetAsData->layout, "load,failed", (void *)SETAS_ERROR_TYPE_UNKNOWN_FORMAT);
		}
		else if ( error == EVAS_LOAD_ERROR_PERMISSION_DENIED )
		{
			evas_object_smart_callback_call(pSetAsData->layout, "load,failed", (void *)SETAS_ERROR_TYPE_PERMISSION_DENIED);
		}
		else
		{
			evas_object_smart_callback_call(pSetAsData->layout, "load,failed", (void *)SETAS_ERROR_TYPE_INVALID_FILE);
		}

		return;
	}

	MSG_SETAS_HIGH("Photocam loaded.");

	pSetAsData->box_timer = ecore_timer_add(CROP_BOX_INTERVAL_TIME, _put_scissorbox_timer_cb, (void *)pSetAsData->layout);

	evas_object_smart_callback_call(pSetAsData->layout, "loaded", NULL);

	pSetAsData->notify_area = const_cast<Evas_Object *>(edje_object_part_object_get(_EDJ(pSetAsData->layout),
																"setasview.contents.swallow.notify"));

	pSetAsData->notify = ivug_notify_static_create(pSetAsData->notify_area, IDS_CALLER_ID_NOTIFY, NOTIFY_ALIGN_BOTTOM);
}


static void _on_moved(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
//	TSetAsData *pSetAsData = SETAS_DATA(data);

}

static void _on_resized(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	TSetAsData *pSetAsData = SETAS_DATA(data);
	///////////////////////////////////////////////////////////////////
	int org_x, org_y, org_w, org_h;

	MSG_SETAS_MED("prev_x=%d, prev_y=%d, prev_w=%d, prev_h=%d", pSetAsData->prev_x, pSetAsData->prev_y, pSetAsData->prev_w, pSetAsData->prev_h);

	org_x = pSetAsData->prev_x;
	org_y = pSetAsData->prev_y;
	org_w = pSetAsData->prev_w;
	org_h = pSetAsData->prev_h;

	if(org_w == 0 || org_h == 0)
		return;

	int org_sx, org_sy, org_sw, org_sh;
	ivug_scissorbox_region_get(pSetAsData->crop.box, &org_sx, &org_sy, &org_sw, &org_sh);

	MSG_SETAS_MED("org_sx=%d, org_sy=%d, org_sw=%d, org_sh=%d", org_sx, org_sy, org_sw, org_sh);

	int dx = org_sx-org_x;
	int dy = org_sy-org_y;

	int lcd_x, lcd_y, lcd_w, lcd_h;

	evas_object_geometry_get(pSetAsData->contents_area, &lcd_x, &lcd_y, &lcd_w, &lcd_h);
	MSG_SETAS_MED("lcd_x=%d, lcd_y=%d, lcd_w=%d, lcd_h=%d", lcd_x, lcd_y, lcd_w, lcd_h);

	int ph, pw;

	int img_w, img_h;
	int ext_w, ext_h;

	img_w = org_w;
	img_h = org_h;
	ext_w = lcd_w;
	ext_h = lcd_h;

	ph = (img_h * ext_w) / img_w;

	if (ph > ext_h)
	{
		pw = (img_w * ext_h) / img_h;
		ph = ext_h;
	}
	else
	{
		pw = ext_w;
	}

	double zoom = 0.0;

	if ( img_w > img_h)
	{
		zoom = (double)pw / img_w;
	}
	else
	{
		zoom = (double)ph / img_h;
	}

	int sx, sy, sw, sh;

	sw = org_sw*zoom;
	sh = org_sh*zoom;
	dx *= zoom;
	dy *= zoom;

	sx = lcd_x + (ext_w-img_w*zoom)/2 + dx;
	sy = lcd_y + (ext_h-img_h*zoom)/2 + dy;


	int bound_x, bound_y, bound_w, bound_h;

	bound_x = lcd_x+(ext_w-img_w*zoom)/2;
	bound_y = lcd_y+(ext_h-img_h*zoom)/2;
	bound_w = img_w*zoom;
	bound_h = img_h*zoom;

	ivug_scissorbox_boundary_set(pSetAsData->crop.box, bound_x, bound_y, bound_w, bound_h);

	///////////////////////////////////////////////////////////////////

	MSG_SETAS_MED("sx=%d, sy=%d, sw=%d, sh=%d", sx, sy, sw, sh);

	ivug_scissorbox_region_set(pSetAsData->crop.box, sx, sy, sw, sh);

	pSetAsData->prev_x = bound_x;
	pSetAsData->prev_y = bound_y;
	pSetAsData->prev_w = bound_w;
	pSetAsData->prev_h = bound_h;

	MSG_SETAS_MED("prev_x=%d, prev_y=%d, prev_w=%d, prev_h=%d", pSetAsData->prev_x, pSetAsData->prev_y, pSetAsData->prev_w, pSetAsData->prev_h);
}

static Evas_Event_Flags _finger_tap_end(void *data , void *event_info)
{
	Elm_Gesture_Taps_Info *p = (Elm_Gesture_Taps_Info *) event_info;

	MSG_SETAS_HIGH("Finger tab end. evt=%d", p->timestamp);

	if ( p->n != 1 ) return EVAS_EVENT_FLAG_NONE;

	TSetAsData *pSetAsData = (TSetAsData *)data;
	IV_ASSERT(pSetAsData != NULL);

	if(pSetAsData->bShowMenu)
	{
		MSG_SETAS_HIGH("Hide");
		elm_win_indicator_mode_set((Evas_Object*)ug_get_window(), ELM_WIN_INDICATOR_HIDE);
		evas_object_hide(pSetAsData->notify);
	}
	else
	{
		MSG_SETAS_HIGH("Show");
		if(gIsDesktopMode() == false)
		{
			elm_win_indicator_mode_set((Evas_Object *)ug_get_window(), ELM_WIN_INDICATOR_SHOW);
		}
		evas_object_show(pSetAsData->notify);
	}

	pSetAsData->bShowMenu = !pSetAsData->bShowMenu;

	return EVAS_EVENT_FLAG_NONE;
}

void _add_gesture_layer(Evas_Object *parent, TSetAsData *pSetAsData)
{
	IV_ASSERT(pSetAsData != NULL);

	Evas_Object *gesture = pSetAsData->gesture;

	gesture = elm_gesture_layer_add(parent);
	elm_gesture_layer_hold_events_set(gesture, EINA_FALSE);

	elm_gesture_layer_cb_set(gesture, ELM_GESTURE_N_TAPS, ELM_GESTURE_STATE_END, _finger_tap_end, (void *)pSetAsData);

	if ( elm_gesture_layer_attach(gesture, pSetAsData->photocam) == EINA_FALSE)
	{
		MSG_SETAS_ERROR("Cannot attach event rect");
	}

	evas_object_name_set(gesture, "CallerID:Getsture");
}

Evas_Object *_ivug_setas_callerid_create_layout(Evas_Object *parent)
{
	TSetAsData *pSetAsData = (TSetAsData *)calloc(1, sizeof(TSetAsData));

	if ( pSetAsData == NULL )
	{
		MSG_SETAS_ERROR("Cannot alloc pSetasData");
		return NULL;
	}

	Evas_Object *photocam = NULL;

	pSetAsData->layout = ivug_layout_add2(parent, SETAS_EDJ_FILE, "setas_view");
	if ( pSetAsData->layout == NULL )
	{
		MSG_SETAS_ERROR("Cannot create layout.");
		goto error;
	}

	MSG_SETAS_HIGH("Create setas layout");
	pSetAsData->contents_area = ivug_default_layout_add(parent);
	evas_object_name_set(pSetAsData->contents_area, "contets_area");
	if ( pSetAsData->contents_area == NULL )
	{
		MSG_SETAS_ERROR("Cannot create contets_area.");
	}

	elm_object_part_content_set(pSetAsData->layout, "setasview.contents.swallow", pSetAsData->contents_area);

	pSetAsData->crop.box = ivug_scissorbox_add(pSetAsData->contents_area);

	if ( pSetAsData->crop.box == NULL )
	{
		MSG_SETAS_ERROR("Cannot create layout.");
		goto error;
	}

	evas_object_smart_member_add(pSetAsData->crop.box, pSetAsData->layout);

	photocam = ivug_image_create(pSetAsData->layout);
	ivug_image_animated_set(photocam, EINA_FALSE);	// Show first frame only when AGIF
	ivug_image_hold_set(photocam , EINA_TRUE);			// Disable mouse events

	evas_object_name_set(photocam, "setas_photocam");

	pSetAsData->photocam = photocam;

	evas_object_smart_callback_add(photocam, "loaded", _on_photocam_loaded, (void *)pSetAsData->layout);
	elm_object_part_content_set(pSetAsData->layout, "setasview.photocam", photocam);
	evas_object_show(photocam);

	pSetAsData->bShowMenu = true;
	//_add_gesture_layer(pSetAsData->layout, pSetAsData);

	evas_object_data_set(pSetAsData->layout, "pSetAsData", pSetAsData);
	evas_object_event_callback_add(pSetAsData->layout, EVAS_CALLBACK_MOVE, _on_moved, pSetAsData->layout);
	evas_object_event_callback_add(pSetAsData->layout, EVAS_CALLBACK_RESIZE, _on_resized, pSetAsData->layout);

	return pSetAsData->layout;

error:
	if ( pSetAsData->crop.box )
		evas_object_del(pSetAsData->crop.box);

	if ( pSetAsData->layout )
		evas_object_del(pSetAsData->layout);

	if ( pSetAsData )
		free(pSetAsData);

	return NULL;
}

Evas_Object *_ivug_setas_callerid_delete_layout(Evas_Object *obj)
{
	TSetAsData *pSetAsData = SETAS_DATA(obj);
	IV_ASSERT(pSetAsData != NULL);

	if(pSetAsData->box_timer)
	{
		ecore_timer_del(pSetAsData->box_timer);
		pSetAsData->box_timer = NULL;
	}

	if(pSetAsData->photocam)
	{
		MSG_SETAS_HIGH("Removing photocam");
		evas_object_del(pSetAsData->photocam);
		pSetAsData->photocam = NULL;
	}

	if(pSetAsData->notify)
	{
		evas_object_del(pSetAsData->notify);
		pSetAsData->notify = NULL;
	}

	if(pSetAsData->contents_area)
	{
		evas_object_del(pSetAsData->contents_area);
		pSetAsData->contents_area = NULL;
	}

	if(pSetAsData->layout)
	{
		evas_object_event_callback_del_full(pSetAsData->layout, EVAS_CALLBACK_MOVE, _on_moved, pSetAsData->layout);
		evas_object_event_callback_del_full(pSetAsData->layout, EVAS_CALLBACK_RESIZE, _on_resized, pSetAsData->layout);
		evas_object_del(pSetAsData->layout);
		pSetAsData->layout = NULL;
	}

	if(pSetAsData->crop.box)
	{
		evas_object_del(pSetAsData->crop.box);
		pSetAsData->crop.box = NULL;
	}
	if(pSetAsData->filepath)
	{
		free(pSetAsData->filepath);
		pSetAsData->filepath = NULL;
	}

	if(pSetAsData->drm_filepath)
	{
		ivug_remove_file(pSetAsData->drm_filepath);
		free(pSetAsData->drm_filepath);
		pSetAsData->drm_filepath = NULL;
	}

	if(pSetAsData->btn_ok)
	{
		evas_object_del(pSetAsData->btn_ok);
		pSetAsData->btn_ok = NULL;
	}

	free(pSetAsData);

	return NULL;
}

bool _ivug_setas_callerid_load_file(Evas_Object *obj, const char *filepath)
{
	TSetAsData *pSetAsData = SETAS_DATA(obj);

	MSG_SETAS_HIGH("Load image file : %s", filepath);

	pSetAsData->filepath = strdup(filepath);

	Evas_Load_Error error = EVAS_LOAD_ERROR_NONE;
	error = ivug_image_file_set(pSetAsData->photocam, filepath, "noAnim");

	if ( error != EVAS_LOAD_ERROR_NONE )
	{
		MSG_SETAS_ERROR("FileSet Error=%d", error);

		if ( error == EVAS_LOAD_ERROR_UNKNOWN_FORMAT )
		{
			evas_object_smart_callback_call(pSetAsData->layout, "load,failed", (void *)SETAS_ERROR_TYPE_UNKNOWN_FORMAT);
			return false;
		}
		else if ( error == EVAS_LOAD_ERROR_PERMISSION_DENIED )
		{
			evas_object_smart_callback_call(pSetAsData->layout, "load,failed", (void *)SETAS_ERROR_TYPE_PERMISSION_DENIED);
			return false;
		}
		else
		{
			evas_object_smart_callback_call(pSetAsData->layout, "load,failed", (void *)SETAS_ERROR_TYPE_INVALID_FILE);
			return false;
		}

		return false;
	}

	return true;
}

Evas_Object *_ivug_setas_callerid_internal_image_get(Evas_Object *obj)
{
	TSetAsData *pSetAsData = SETAS_DATA(obj);

	MSG_SETAS_HIGH("Internal image get!");

	return pSetAsData->photocam;
}

bool _ivug_setas_callerid_select_region_get(Evas_Object *obj, int *x, int *y, int *w, int *h)
{
	TSetAsData *pSetAsData = SETAS_DATA(obj);

	if ( pSetAsData->crop.box == NULL)
	{
		return false;
	}

	ivug_scissorbox_region_get(pSetAsData->crop.box, x, y, w, h);

	return true;
}


void _ivug_setas_callerid_aspect_ratio_set(Evas_Object *obj, double ratio)
{
	TSetAsData *pSetAsData = SETAS_DATA(obj);

	pSetAsData->crop.bAspectPreserve = TRUE;
	pSetAsData->crop.ratio = ratio;

	ivug_scissorbox_ratio_fix( pSetAsData->crop.box, EINA_TRUE);
}

void _ivug_setas_callerid_create_menu(Evas_Object *obj, Evas_Object *navi_bar)
{
	TSetAsData *pSetAsData = SETAS_DATA(obj);

	pSetAsData->btn_ok = ivug_button_add(navi_bar, "naviframe/toolbar/default",
								IDS_SAVE, NULL, _on_btn_ok_clicked, (void *)pSetAsData->layout);

	Elm_Object_Item *navi_it = elm_naviframe_top_item_get(navi_bar);
	elm_object_item_part_content_set(navi_it, "toolbar_button1", pSetAsData->btn_ok);

	elm_object_disabled_set(pSetAsData->btn_ok, EINA_TRUE);
}

