/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>

#include "ivug-define.h"

#include "ivug-setas-view-wallpaper.h"
#include "ivug-setas-view.h"

#include "ivug-file-info.h"
#include "ivug-image.h"
#include "ivug-drm-common.h"
#include "EFLUtil.h"
#include "ivug-selectioninfo.h"

#define EDJ_PATH PREFIX"/res/edje/"PACKAGE

#define SETAS_EDJ_FILE EDJ_PATH"/ivug-setas.edj"

#define CLICK_TIME_MAX		(500)				// click time.
#define CLICK_WIDTH_MIN	(elm_config_finger_size_get() >> 1)	//click width, if mouse move under this value and under click time, it is click.
#define CLICK_INTERVAL_TIME 0.15

/* it is temp define, because scroll, scroll,anim,end is called earlyer.
   So find bound of scissor box again after SETAS_INTERVAL_TIME */
#define SETAS_INTERVAL_TIME 0.1
#define SETAS_INTERVAL_TIME_FIRST_LOAD 0.3

typedef struct {
	Evas_Object *layout;
	Evas_Object *photocam;
	Evas_Object *notify;

	Evas_Object *contents_area;

	Evas_Object *btn_ok;

	bool bShowMenu;		// Current state for button's visibility
	char *drm_filepath;
} TSetAsData;

#define SETAS_DATA(obj) \
		(TSetAsData *)evas_object_data_get((Evas_Object *)(obj), "pSetAsData")

static void
_on_btn_ok_clicked(void *data, Evas_Object *obj, void *event_info)
{
	TSetAsData *pSetAsData = SETAS_DATA(data);
	IV_ASSERT(pSetAsData != NULL);

	MSG_SETAS_HIGH("OK button clicked");

	evas_object_smart_callback_call(pSetAsData->layout, "ok,clicked", NULL);
}

static void
_on_photocam_loaded(void *data, Evas_Object *obj, void *event_info)
{
	TSetAsData *pSetAsData = SETAS_DATA(data);
	MSG_SETAS_HIGH("Photocam preloaded");
#if 0
	typedef enum _Evas_Load_Error
	{
	   EVAS_LOAD_ERROR_NONE = 0, /**< No error on load */
	   EVAS_LOAD_ERROR_GENERIC = 1, /**< A non-specific error occurred */
	   EVAS_LOAD_ERROR_DOES_NOT_EXIST = 2, /**< File (or file path) does not exist */
	   EVAS_LOAD_ERROR_PERMISSION_DENIED = 3, /**< Permission deinied to an existing file (or path) */
	   EVAS_LOAD_ERROR_RESOURCE_ALLOCATION_FAILED = 4, /**< Allocation of resources failure prevented load */
	   EVAS_LOAD_ERROR_CORRUPT_FILE = 5, /**< File corrupt (but was detected as a known format) */
	   EVAS_LOAD_ERROR_UNKNOWN_FORMAT = 6 /**< File is not a known format */
	} Evas_Load_Error; /**< Load error you can get from loading of files - see evas_load_error_str() too */
#endif

	Evas_Load_Error error = static_cast<Evas_Load_Error>((int)event_info);

	if ( error != EVAS_LOAD_ERROR_NONE )
	{
		MSG_SETAS_ERROR("Error occured during decoding. Error=%d", error);

		if ( error == EVAS_LOAD_ERROR_UNKNOWN_FORMAT )
		{
			evas_object_smart_callback_call(pSetAsData->layout, "load,failed", (void *)SETAS_ERROR_TYPE_UNKNOWN_FORMAT);
		}
		else if ( error == EVAS_LOAD_ERROR_PERMISSION_DENIED )
		{
			evas_object_smart_callback_call(pSetAsData->layout, "load,failed", (void *)SETAS_ERROR_TYPE_PERMISSION_DENIED);
		}
		else
		{
			evas_object_smart_callback_call(pSetAsData->layout, "load,failed", (void *)SETAS_ERROR_TYPE_INVALID_FILE);
		}

		return;
	}

	MSG_SETAS_HIGH("Photocam loaded.");

	elm_object_disabled_set(pSetAsData->btn_ok, EINA_FALSE);

	evas_object_smart_callback_call(pSetAsData->layout, "loaded", NULL);
}


static void _on_moved(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
//	TSetAsData *pSetAsData = SETAS_DATA(data);

}

static void _on_resized(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
//	TSetAsData *pSetAsData = SETAS_DATA(data);

}

static void
_on_bg_clicked(void *data, Evas_Object* obj, const char *emission, const char *source )
{
	MSG_SETAS_HIGH("BG Clicked");

	TSetAsData *pSetAsData = SETAS_DATA(data);

	MSG_SETAS_HIGH("Bar flag is = %d", pSetAsData->bShowMenu);
	if(pSetAsData->bShowMenu == true)
	{
		pSetAsData->bShowMenu = false;
	}
	else
	{
		pSetAsData->bShowMenu = true;
	}
}

Evas_Object *_ivug_setas_create_layout(Evas_Object *parent)
{
	TSetAsData *pSetAsData = (TSetAsData *)calloc(1, sizeof(TSetAsData));

	if ( pSetAsData == NULL )
	{
		MSG_SETAS_ERROR("Cannot alloc pSetasData");
		return NULL;
	}

	pSetAsData->layout = ivug_layout_add2(parent , SETAS_EDJ_FILE, "setas_view");
	evas_object_name_set(pSetAsData->layout, "setas_layout");
	if ( pSetAsData->layout == NULL )
	{
		MSG_SETAS_ERROR("Cannot create layout.");
		free(pSetAsData);
		return NULL;
	}
	evas_object_event_callback_add(pSetAsData->layout, EVAS_CALLBACK_MOVE, _on_moved, pSetAsData->layout);
	evas_object_event_callback_add(pSetAsData->layout, EVAS_CALLBACK_RESIZE, _on_resized, pSetAsData->layout);

	Evas_Object *photocam;
	photocam = ivug_image_create(pSetAsData->layout);
	ivug_image_animated_set(photocam, EINA_FALSE);	// Show first frame only when AGIF
	evas_object_name_set(photocam, "setas_photocam");

	evas_object_smart_callback_add(photocam, "loaded", _on_photocam_loaded, (void *)pSetAsData->layout);
	elm_object_part_content_set(pSetAsData->layout, "setasview.photocam", photocam);

	evas_object_show(photocam);

	pSetAsData->photocam = photocam;

	edje_object_signal_callback_add(_EDJ(pSetAsData->layout),
				"setasview,bg,clicked", "edc",
				_on_bg_clicked,
				(void *)pSetAsData->layout);

	pSetAsData->bShowMenu = true;

	MSG_SETAS_HIGH("Create setas layout");
	evas_object_data_set(pSetAsData->layout, "pSetAsData", pSetAsData);

	pSetAsData->contents_area = ivug_default_layout_add(parent);
	evas_object_name_set(pSetAsData->contents_area, "contets_area");
	if ( pSetAsData->contents_area == NULL )
	{
		MSG_SETAS_ERROR("Cannot create contets_area.");
	}

	elm_object_part_content_set(pSetAsData->layout, "setasview.contents", pSetAsData->contents_area);

	pSetAsData->notify = ivug_notify_static_create(pSetAsData->contents_area, IDS_WALLPAPER_NOTIFY, NOTIFY_ALIGN_BOTTOM);

	evas_object_show(pSetAsData->layout);
	return pSetAsData->layout;
}

Evas_Object *_ivug_setas_delete_layout(Evas_Object *obj)
{
	TSetAsData *pSetAsData = SETAS_DATA(obj);
	IV_ASSERT(pSetAsData != NULL);

	if(pSetAsData->photocam)
	{
		MSG_SETAS_HIGH("Removing photocam");
		evas_object_del(pSetAsData->photocam);
		pSetAsData->photocam = NULL;
	}

	if(pSetAsData->notify)
	{
		evas_object_del(pSetAsData->notify);
		pSetAsData->notify = NULL;
	}

	if(pSetAsData->contents_area)
	{
		evas_object_del(pSetAsData->contents_area);
		pSetAsData->contents_area = NULL;
	}

	if(pSetAsData->layout)
	{
		evas_object_event_callback_del_full(pSetAsData->layout, EVAS_CALLBACK_MOVE, _on_moved, pSetAsData->layout);
		evas_object_event_callback_del_full(pSetAsData->layout, EVAS_CALLBACK_RESIZE, _on_resized, pSetAsData->layout);
		evas_object_del(pSetAsData->layout);
		pSetAsData->layout = NULL;
	}

	if(pSetAsData->drm_filepath)
	{
		ivug_remove_file(pSetAsData->drm_filepath);
		free(pSetAsData->drm_filepath);
		pSetAsData->drm_filepath = NULL;
	}

	if(pSetAsData->btn_ok)
	{
		evas_object_del(pSetAsData->btn_ok);
		pSetAsData->btn_ok = NULL;
	}

	free(pSetAsData);

	return NULL;
}

bool _ivug_setas_load_file(Evas_Object *obj, const char *filepath)
{
	TSetAsData *pSetAsData = SETAS_DATA(obj);

	MSG_SETAS_HIGH("Load image file : %s", filepath);

	Evas_Load_Error error = EVAS_LOAD_ERROR_NONE;
	error = ivug_image_file_set(pSetAsData->photocam, filepath, "noAnim");

	if ( error != EVAS_LOAD_ERROR_NONE )
	{
		MSG_SETAS_ERROR("FileSet Error=%d", error);

		if ( error == EVAS_LOAD_ERROR_UNKNOWN_FORMAT )
		{
			evas_object_smart_callback_call(pSetAsData->layout, "load,failed", (void *)SETAS_ERROR_TYPE_UNKNOWN_FORMAT);
			return false;
		}
		else if ( error == EVAS_LOAD_ERROR_PERMISSION_DENIED )
		{
			evas_object_smart_callback_call(pSetAsData->layout, "load,failed", (void *)SETAS_ERROR_TYPE_PERMISSION_DENIED);
			return false;
		}
		else
		{
			evas_object_smart_callback_call(pSetAsData->layout, "load,failed", (void *)SETAS_ERROR_TYPE_INVALID_FILE);
			return false;
		}

		return false;
	}

	return true;
}

Evas_Object *_ivug_setas_internal_image_get(Evas_Object *obj)
{
	TSetAsData *pSetAsData = SETAS_DATA(obj);

	return pSetAsData->photocam;
}

void _ivug_setas_view_create_menu(Evas_Object *obj, Evas_Object *navi_bar)
{
	TSetAsData *pSetAsData = SETAS_DATA(obj);

	pSetAsData->btn_ok = ivug_button_add(navi_bar, "naviframe/toolbar/default",
								IDS_SAVE, NULL, _on_btn_ok_clicked, (void *)pSetAsData->layout);

	Elm_Object_Item *navi_it = elm_naviframe_top_item_get(navi_bar);
	elm_object_item_part_content_set(navi_it, "toolbar_button1", pSetAsData->btn_ok);

	elm_object_disabled_set(pSetAsData->btn_ok, EINA_TRUE);
}

