/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __IVUG_MEDIA_H__
#define __IVUG_MEDIA_H__

#include "ivug-datatypes.h"
#include "ivug-db.h"

#include <Eina.h>
#include <stdbool.h>

//slide type
typedef enum {
	SLIDE_TYPE_NONE = 0x00,
	SLIDE_TYPE_IMAGE,
	SLIDE_TYPE_VIDEO,
	SLIDE_TYPE_UNKNOWN,		// File is exists but it's type is unknown.
} Media_Type;

// Meida_Data can be one in File, DB

/*
	This struct represents data for slide.
	thumbnail_path is slide's thumbnail image path.
	fileurl is saved item in media service. and drm.
	filepath is local file path. if fileurl is http://.../test.jpg, filepath is NULL, when complete download filepath change to saved local file path.
*/
typedef struct {
	media_handle m_handle;

	int index;

//Data
	Media_Type slide_type;			//image, video
	bool bIsDRMContent;

	UUID mediaID;					// Unique Media ID

	char* thumbnail_path;			// thumbnail image file path.

	char* fileurl;					// file url.
	char* filepath;					// file path in local file system.
	char* drm_filepath;

	Ivug_DB_h *thumb_handle;
} Media_Data;


#ifdef __cplusplus
extern "C" {
#endif

bool ivug_mediadata_set_tag(Media_Data *data, const char *newtag);
bool ivug_mediadata_set_favorite(Media_Data *data, bool bFavorite);

bool ivug_mediadata_delete(Media_Data * mdata);

bool ivug_mediadata_free(Media_Data * mdata);


#ifdef __cplusplus
}
#endif

#endif // __IVUG_MEDIA_H__
