/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __IVUG_STRING_H__
#define __IVUG_STRING_H__

#include <libintl.h>
#include <app.h>
#include "ivug-define.h"

/* Original code is in appcore-common.h */
#define gettext_noop(str) (str) /**< keyword for xgettext  to extract translatable strings */
#define N_(str) gettext_noop(str)  /**< gettext_noop alias */


#define LOCALE_PATH 						PREFIX"/res/locale"

#ifndef PACKAGE
#error PACKAGE is not defined
#endif

#ifndef LOCALE_PATH
#error LOCALE_PATH is not defined
#endif

//string
#define IVUG_TEXT_DOMAIN PACKAGE
#define IVUG_LOCALEDIR LOCALE_PATH

/***********************************************/
/* String has to be sorted in ascending order. */
/***********************************************/

// Not yet translated


#define IDS_DRM_INTERVAL_BASED_DAYS	_("You can use %s for %d days. Would you like to open it now?")
#define IDS_DRM_TIME_BASED_GET	_("%s is about to be locked. Do you want to get more licenses?")
#define IDS_DRM_COUNT_BASED_OPEN_ONE	_("You have one more use of %s. Do you want to open it now?")
#define IDS_DRM_COUNT_BASED_OPEN_MANY	_("You have %d more uses of %s. Do you want to open it now?")
#define IDS_DRM_COUNT_BASED_ONE_MORE_GET	_("You have one more use of %s. Do you want to get more licenses?")
#define IDS_DRM_COUNT_BASED_NO_MORE_GET	_("You have no more use of %s. Do you want to get more licenses?")
#define IDS_DRM_UNLOCK	_("%s is currently locked. Do you want to unlock it now?")

#define IDS_COLLABORATIVE_PLAY _("Collaborative play")


#define IDS_ADD_NAME			(dgettext(IVUG_TEXT_DOMAIN, "IDS_FT_OPT_ADD_NAME"))
#define IDS_SET_AS_CALLER_ID	(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_POP_SET_AS_CALLER_ID_M_PAST_TENSE"))
#define IDS_MMS					_("MMS")
#define IDS_INVALID_NAME		_("Invalid name")

#define IDS_ORIENTATION 		(dgettext(IVUG_TEXT_DOMAIN, "IDS_MEDIABR_HEADER_ORIENTATION"))
#define IDS_MANUFACTURE 		(dgettext(IVUG_TEXT_DOMAIN, "IDS_MEDIABR_HEADER_MANUFACTURER"))
#define IDS_MODEL				(dgettext(IVUG_TEXT_DOMAIN, "IDS_MEDIABR_HEADER_MODEL"))
#define IDS_FLASH 				(dgettext(IVUG_TEXT_DOMAIN, "IDS_CAM_BODY_FLASH"))
#define IDS_NO_FLASH			_("No Flash")
#define IDS_FOCAL_LENGTH 		(dgettext(IVUG_TEXT_DOMAIN, "IDS_CAM_BODY_FOCAL_LENGTH"))
#define IDS_WHITE_BALANCE		(dgettext(IVUG_TEXT_DOMAIN, "IDS_CAM_BODY_WHITE_BALANCE"))
#define IDS_AUTO				(dgettext(IVUG_TEXT_DOMAIN, "IDS_CAM_BODY_AUTO"))
#define IDS_MANUAL				(dgettext("sys_string", "IDS_COM_BODY_MANUAL"))
#define IDS_APERTURE 			(dgettext(IVUG_TEXT_DOMAIN, "IDS_CAM_BODY_APERTURE"))
#define IDS_EXPOSURE_TIME 		(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_EXPOSURE_TIME_ABB"))
#define IDS_ISO 				(dgettext(IVUG_TEXT_DOMAIN, "IDS_CAM_BODY_ISO"))

#define IDS_UNABLE_TO_OPEN_FILE	_("Unable to open file")
#define IDS_CANNOT_LOAD_FILE_LST	_("Cannot load file list")

#define IDS_NULL				_("")

#define IDS_WIFI_DIRECT			_("Wi-Fi Direct")

#define IDS_ADD_TO_HOME			(dgettext(IVUG_TEXT_DOMAIN, "IDS_MEDIABR_SK2_ADD_TO_HOME"))
#define IDS_CREATE_TAG			(dgettext(IVUG_TEXT_DOMAIN, "IDS_MEDIABR_OPT_CREATE_TAG"))

#define IDS_FACE_TAG			(dgettext(IVUG_TEXT_DOMAIN, "IDS_PBR_OPT_FACE_TAGGING"))

#define IDS_WALLPAPER_NOTIFY	(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_SELECTED_AREA_WILL_BE_APPLIED_TO_THE_WALLPAPER"))
#define IDS_CROP_NOTIFY			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_SELECT_AN_AREA_TO_CROP_THE_PICTURE"))
#define IDS_CALLER_ID_NOTIFY	(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_SELECTED_AREA_WILL_BE_APPLIED_TO_THE_CONTACT_PICTURE"))
#define IDS_PRINT_NOTIFY		(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_SELECTED_AREA_WILL_BE_PRINTED"))

/* Registered string in STMS IV */
#define IDS_ACCUMULATED_TIME 	(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_ACCUMULATED_TIME"))
#define IDS_ADD_TAG				(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_HEADER_ADD_TAG"))
#define IDS_ADDED				(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_POP_ADDED"))
#define IDS_ALL_ALBUMS			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_HEADER_ALL_ALBUMS"))
#define IDS_ANY 				(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_ANY"))
#define IDS_AVAILABLE_USES 		(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_AVAILABLE_USES"))
#define IDS_BASIC 				(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_BASIC"))
#define IDS_BUDDY_PHOTO_SHARE	(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_OPT_BUDDY_PHOTO_SHARE"))
#define IDS_CALLER_IMAGE		(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_CALLER_IMAGE"))
#define IDS_COPY_TO_CLIPBOARD	(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_OPT_COPY_TO_CLIPBOARD"))
#define IDS_COUNT 				(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_COUNT"))
#define IDS_CROP 				(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_CROP"))
#define IDS_DATE_CREATED 		(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_DATE_CREATED"))
#define IDS_DISPLAY 			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_DISPLAY"))
#define IDS_DO_NOT_SEND			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_DO_NOT_SEND"))
#define IDS_EDIT_TAG			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_OPT_EDIT_TAG"))
#define IDS_ENTER_TAG_NAME  	(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_POP_ENTER_TAG_NAME"))
#define IDS_ENTRY_IS_EMPTY 		(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_POP_ENTRY_IS_EMPTY"))
#define IDS_EXECUTE 			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_EXECUTE"))
#define IDS_FORMAT 				(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_FILE_FORMAT"))
#define IDS_FILE_LOCATION 		(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_FILE_LOCATION"))
#define IDS_FILE_TRANSFER		(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_OPT_FILE_TRANSFER"))
#define IDS_FORWARDING 			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_FORWARDING"))
#define IDS_HOME_AND_LOCK_SCREEN_WALLPAPER (dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_OPT_HOME_AND_LOCK_SCREEN_WALLPAPER"))
#define IDS_HOME_AND_LOCK_SCREENS (dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_HOME_AND_LOCK_SCREENS"))
#define IDS_HOME_SCREEN			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_HOME_SCREEN"))
#define IDS_HOME_SCREEN_WALLPAPER (dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_HEADER_HOME_SCREEN_WALLPAPER_ABB"))
#define IDS_IMPOSSIBLE 			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_IMPOSSIBLE"))
#define IDS_INDIVIDUAL 			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_INDIVIDUAL"))
#define IDS_INTERVAL 			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_INTERVAL"))
#define IDS_INVALID_IMAGE_FILE (dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_POP_INVALID_IMAGE_FILE"))
#define IDS_LATITUDE 			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_LATITUDE"))
#define IDS_LOCK_SCREEN			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_LOCK_SCREEN"))
#define IDS_LOCK_SCREEN_WALLPAPER (dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_HEADER_LOCK_SCREEN_WALLPAPER_ABB"))
#define IDS_LONGITUDE 			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_LONGITUDE"))
#define IDS_MANUALLY_DETECT		(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_OPT_MANUALLY_DETECT"))
#define IDS_MAX_CHAR_LENGTH_REACHED (dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_POP_MAXIMUM_NUMBER_OF_CHARACTERS_HPD_REACHED"))
#define IDS_ME					(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_ME"))
#define IDS_N_A 				(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_N_A"))
#define IDS_NEW_TAG				(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_NEW_TAG"))
#define IDS_PERMISSION_DENIED 	(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_POP_PERMISSION_DENIED"))
#define IDS_POSSIBLE 			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_POSSIBLE"))
#define IDS_PRINT 				(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_PRINT"))
#define IDS_PRINT_VIA_BLUETOOH	(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_OPT_PRINT_VIA_BLUETOOTH"))
#define IDS_REMOVE_TAG			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_OPT_REMOVE_TAG"))
#define IDS_RESOLUTION 			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_RESOLUTION"))
#define IDS_RIGHT_STATUS 		(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_RIGHT_STATUS"))
#define IDS_ROTATE_LEFT 		(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_OPT_ROTATE_LEFT"))
#define IDS_ROTATE_RIGHT 		(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_OPT_ROTATE_RIGHT"))
#define IDS_SD_CARD_REMOVED		(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_POP_SD_CARD_REMOVED"))
#define IDS_SELECT_DEVICE		(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_POP_SELECT_DEVICE"))
#define IDS_SERVER_NAME 		(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_SERVER_NAME"))
#define IDS_SET_AS				(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_OPT_SET_AS"))
#define IDS_SET_WALLPAPER_FOR_HOME_SCREEN_AND_LOCK_SCREEN (dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_POP_SET_WALLPAPER_FOR_HOME_SCREEN_AND_LOCK_SCREEN"))
#define IDS_SHARE		  		(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_OPT_SHARE"))
#define IDS_SHARE_VIA			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_SHARE_VIA"))
#define IDS_SLIDE_SHOW			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_OPT_SLIDE_SHOW"))
#define IDS_TAG			 		(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_OPT_TAG"))
#define IDS_TIMED_COUNT 		(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_TIMED_COUNT"))
#define IDS_TRIM				(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_OPT_TRIM"))
#define IDS_UNLIMITED 			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_UNLIMITED"))
#define IDS_USE_MOTION 			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_OPT_USE_MOTION"))
#define IDS_VALIDITY 			(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_VALIDITY"))
#define IDS_VENDOR 				(dgettext(IVUG_TEXT_DOMAIN, "IDS_IV_BODY_VENDOR"))


// System string.
#define IDS_BLUETOOTH 		(dgettext("sys_string", "IDS_COM_BODY_BLUETOOTH"))
#define IDS_EMAIL 			(dgettext("sys_string", "IDS_COM_BODY_EMAIL"))
#define IDS_MESSAGE 		(dgettext("sys_string", "IDS_COM_BODY_MESSAGE"))
#define IDS_WI_FI			(dgettext("sys_string", "IDS_COM_BODY_WI_FI"))
#define IDS_NFC				(dgettext("sys_string", "IDS_COM_BODY_NFC"))

#define IDS_FACEBOOK 		(dgettext("sys_string", "IDS_COM_BODY_FACEBOOK"))
#define IDS_PICASA  		(dgettext("sys_string", "IDS_COM_BODY_PICASA"))
#define IDS_TWITTER 		(dgettext("sys_string", "IDS_COM_BODY_TWITTER"))
#define IDS_YOUTUBE 		(dgettext("sys_string", "IDS_COM_BODY_YOU_TUBE"))

#define IDS_APPLICATION_NOT_INSTALLED (dgettext("sys_string", "IDS_COM_BODY_APPLICATION_NOT_INSTALLED"))
#define IDS_BACK 			(dgettext("sys_string", "IDS_COM_BODY_BACK"))
#define IDS_CANCEL 			(dgettext("sys_string", "IDS_COM_SK_CANCEL"))
#define IDS_CLOSE 			(dgettext("sys_string", "IDS_COM_BODY_CLOSE"))
#define IDS_CHANGE_NAME		(dgettext("sys_string", "IDS_COM_BODY_CHANGE_NAME"))
#define IDS_COPY			(dgettext("sys_string", "IDS_COM_BODY_COPY"))
#define IDS_COPIED_TO_CLIPBOARD		(dgettext("sys_string", "IDS_COM_POP_COPIED_TO_CLIPBOARD"))
#define IDS_DETAILS			(dgettext("sys_string", "IDS_COM_BODY_DETAILS"))
#define IDS_DELETE_MSG		(dgettext("sys_string", "IDS_COM_POP_DELETE_Q"))
#define IDS_DELETE 			(dgettext("sys_string", "IDS_COM_BODY_DELETE"))
#define IDS_DELETED 		(dgettext("sys_string", "IDS_COM_POP_DELETED"))
#define IDS_DONE 			(dgettext("sys_string", "IDS_COM_SK_DONE"))
#define IDS_DOWNLOAD 		(dgettext("sys_string", "IDS_COM_SK_DOWNLOAD"))
#define IDS_DOWNLOAD_FAILED	(dgettext("sys_string", "IDS_COM_POP_DOWNLOAD_FAILED"))
#define IDS_EDIT			(dgettext("sys_string", "IDS_COM_BODY_EDIT"))
#define IDS_ENTER_NAME		(dgettext("sys_string", "IDS_COM_BODY_ENTER_NAME"))
#define IDS_ERROR			(dgettext("sys_string", "IDS_COM_POP_ERROR"))
#define IDS_FAILED 			(dgettext("sys_string", "IDS_COM_POP_FAILED"))
#define IDS_FAVOURITES		(dgettext("sys_string", "IDS_COM_BODY_FAVOURITES"))
#define IDS_LOADING 		(dgettext("sys_string", "IDS_COM_BODY_LOADING"))
#define IDS_LOCATION 		(dgettext("sys_string", "IDS_COM_BODY_LOCATION"))
#define IDS_MORE 			(dgettext("sys_string", "IDS_COM_BODY_MORE"))
#define IDS_MOVE			(dgettext("sys_string", "IDS_COM_BODY_MOVE"))
#define IDS_NAME 			(dgettext("sys_string", "IDS_COM_BODY_DETAILS_NAME"))
#define IDS_NO	 			(dgettext("sys_string", "IDS_COM_SK_NO"))
#define IDS_NO_NAME			(dgettext("sys_string", "IDS_COM_BODY_NO_NAME"))
#define IDS_OFF 			(dgettext("sys_string", "IDS_COM_BODY_OFF"))
#define IDS_OK 				(dgettext("sys_string", "IDS_COM_SK_OK"))
#define IDS_ON 				(dgettext("sys_string", "IDS_COM_BODY_ON"))
#define IDS_PLAY			(dgettext("sys_string", "IDS_COM_SK_PLAY"))
#define IDS_SAVE 			(dgettext("sys_string", "IDS_COM_SK_SAVE"))
#define IDS_SAVED 			(dgettext("sys_string", "IDS_COM_POP_SAVED"))
#define IDS_SEND			(dgettext("sys_string", "IDS_COM_BODY_SEND"))
#define IDS_SET				(dgettext("sys_string", "IDS_COM_SK_SET"))
#define IDS_SIZE 			(dgettext("sys_string", "IDS_COM_BODY_SIZE"))
#define IDS_SUCCESS 		(dgettext("sys_string", "IDS_COM_POP_SUCCESS"))
#define IDS_SYSTEM			(dgettext("sys_string", "IDS_COM_BODY_SYSTEM"))
#define IDS_TIME 			(dgettext("sys_string", "IDS_COM_POP_TIME"))
#define IDS_TYPE 			(dgettext("sys_string", "IDS_COM_POP_TYPE"))
#define IDS_UNKNOWN 		(dgettext("sys_string", "IDS_COM_BODY_UNKNOWN"))
#define IDS_YES 			(dgettext("sys_string", "IDS_COM_SK_YES"))


#endif // __IVUG_STRING_H__


