/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

#include "ivug-debug.h"
#include "EvasSmartObj.h"
#include "Ecore.h"

using namespace std;

class CSelectBox;

class CScissorBox : public CEvasSmartObject<CScissorBox> {
public:
	enum eType {
		IMAGE_VIEW,
		IMAGE_EDIT,
	};

public:
	CScissorBox() : CEvasSmartObject<CScissorBox>(), m_box(NULL), m_boxtype(IMAGE_VIEW), m_bOpaque(false), m_target(NULL), m_bPreserveRatio(false), m_bSetRegion(false), m_job(NULL) {
		CEvasSmartObject<CScissorBox>::SetClassName("ScissorBox");
	};

	virtual ~CScissorBox() {
		MSG_HIGH("~CScissorBox freed");
	};

	Evas_Object *CreateObject(Evas_Object *parent);

	void SetOpaque(bool bOpaque);
	void SetRegion(int x, int y, int w, int h);
	void SetBound(int x, int y, int w, int h);
	void SetPreserveRatio(bool bPreserveRatio);
	void SetBoxType(eType type);

	void Attach(Evas_Object *image);

	const CRect &GetRegion() const;

	void draw();
	void remove();
	void resize(int w, int h) {
		MSG_HIGH("Scissor box resized. WH(%d,%d)", w, h);
	};

	static void _on_resize(void *data, Evas *e, Evas_Object *obj, void *event_info)
	{
		//CScissorBox *scissorbox = (CScissorBox *)data;

		//scissorbox->OnTargetChanged();
	};

	static void _on_move(void *data, Evas *e, Evas_Object *obj, void *event_info) {
		// CScissorBox *scissorbox = (CScissorBox *)data;

		//scissorbox->OnTargetChanged();
	};

	static void _OnChangedJob(void *data) {
		CScissorBox *scissorbox = (CScissorBox *)data;

		scissorbox->OnChanged();
	}

	void OnTargetChanged();

	void OnChanged() {
		m_job = NULL;
		ApplyChanges();
	};

	void ApplyChanges();

private:
	void RegisterCB() {
		MSG_ASSERT(m_target != NULL);

		evas_object_event_callback_add(m_target, EVAS_CALLBACK_RESIZE, _on_resize, this);
		evas_object_event_callback_add(m_target, EVAS_CALLBACK_MOVE, _on_move, this);

	}

	void UnRegisterCB() {
		evas_object_event_callback_del_full(m_target, EVAS_CALLBACK_RESIZE, _on_resize, this);
		evas_object_event_callback_del_full(m_target, EVAS_CALLBACK_MOVE, _on_move, this);
	};


private:
	CSelectBox *m_box;

	eType m_boxtype;

	bool m_bOpaque;
	Evas_Object *m_opaque[4];

	Evas_Object *m_target;

	bool m_bPreserveRatio;
	bool m_bSetRegion;		// If region is set already,
	Ecore_Job *m_job;

	CRect tRect;		// Target rectangle

	Evas_Object *m_parent;	// parent layout
};



