/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>

#include "ivug-scissorbox.h"
#include "ivug-debug.h"

#include "ScissorBox.h"

#define SCISSORBOX_CLASS(obj) \
	static_cast<CScissorBox *>(evas_object_data_get((obj), "CScissorBox"))


IVAPI Evas_Object *ivug_scissorbox_add(Evas_Object *parent)
{
	CScissorBox *scissorbox = new CScissorBox;

	Evas_Object *obj = scissorbox->CreateObject(parent);
	MSG_ASSERT(obj != NULL);

	evas_object_data_set(obj, "CScissorBox", scissorbox);

	MSG_HIGH("Create scissorBox object. class=0x%08x obj=0x%08x", scissorbox, obj);
	return obj;
}

IVAPI void ivug_scissorbox_type_set(Evas_Object *obj, eScissorBoxType type)
{
	MSG_ASSERT(obj != NULL);
	CScissorBox *scissorbox = SCISSORBOX_CLASS(obj);

	if ( type == SCISSOR_BOX_TYPE_1 )
	{
		scissorbox->SetBoxType(CScissorBox::IMAGE_VIEW);
	}
	else
	{
		scissorbox->SetBoxType(CScissorBox::IMAGE_EDIT);
	}

}

IVAPI void ivug_scissorbox_attach(Evas_Object *obj, Evas_Object *pObj)
{
	CScissorBox *scissorbox = SCISSORBOX_CLASS(obj);
	MSG_ASSERT(pObj != NULL);

	scissorbox->Attach(pObj);
}

IVAPI void ivug_scissorbox_region_get(Evas_Object *obj, int *x, int *y, int *w, int *h)
{
	MSG_ASSERT(obj != NULL);
	CScissorBox *scissorbox = SCISSORBOX_CLASS(obj);

	const CRect &region = scissorbox->GetRegion();

	if ( x ) *x = region.Left();
	if ( y ) *y = region.Top();
	if ( w ) *w = region.Width();
	if ( h ) *h = region.Height();

}

IVAPI void ivug_scissorbox_region_set(Evas_Object *obj, int x, int y, int w, int h)
{
	MSG_ASSERT(obj != NULL);
	CScissorBox *scissorbox = SCISSORBOX_CLASS(obj);

	scissorbox->SetRegion(x,y,w,h);
}

IVAPI void ivug_scissorbox_boundary_set(Evas_Object *obj, int x, int y, int w, int h)
{
	MSG_ASSERT(obj != NULL);
	CScissorBox *scissorbox = SCISSORBOX_CLASS(obj);

	scissorbox->SetBound(x,y,w,h);
}


IVAPI void ivug_scissorbox_ratio_fix(Evas_Object *obj, Eina_Bool bFix)
{
	MSG_ASSERT(obj != NULL);
	CScissorBox *scissorbox = SCISSORBOX_CLASS(obj);

	if ( bFix == EINA_TRUE )
	{
		scissorbox->SetPreserveRatio(true);
	}
	else
	{
		scissorbox->SetPreserveRatio(false);
	}

}

IVAPI void ivug_scissorbox_set_opaque(Evas_Object *obj, Eina_Bool bOpaque)
{
	MSG_ASSERT(obj != NULL);
	CScissorBox *scissorbox = SCISSORBOX_CLASS(obj);

	if ( bOpaque == EINA_TRUE )
	{
		scissorbox->SetOpaque(true);
	}
	else
	{
		scissorbox->SetOpaque(false);
	}
}


